@@ -1,3 +1,12 @@
+///|/ Copyright (c) Prusa Research 2016 - 2023 Lukáš Matěna @lukasmatena, Vojtěch Bubník @bubnikv, Pavel Mikuš @Godrak, Lukáš Hejl @hejllukas
+///|/ Copyright (c) SuperSlicer 2023 Remi Durand @supermerill
+///|/ Copyright (c) 2016 Sakari Kapanen @Flannelhead
+///|/ Copyright (c) Slic3r 2011 - 2015 Alessandro Ranellucci @alranel
+///|/ Copyright (c) 2013 Mark Hindess
+///|/ Copyright (c) 2011 Michael Moon
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #include <assert.h>
 #include <stdio.h>
 #include <memory>
@@ -8,15 +17,20 @@
 #include "../Print.hpp"
 #include "../PrintConfig.hpp"
 #include "../Surface.hpp"
+// for Arachne based infills
 #include "../PerimeterGenerator.hpp"
 
 #include "FillBase.hpp"
 #include "FillRectilinear.hpp"
 #include "FillLightning.hpp"
 #include "FillConcentric.hpp"
+#include "FillEnsuring.hpp"
+#include "Polygon.hpp"
 
 namespace Slic3r {
 
+//static constexpr const float NarrowInfillAreaThresholdMM = 3.f;
+
 struct SurfaceFillParams
 {
 	// Zero based extruder ID.
@@ -50,7 +64,7 @@ struct SurfaceFillParams
     Flow 			flow;
 
 	// For the output
-    ExtrusionRole	extrusion_role = ExtrusionRole(0);
+	ExtrusionRole	extrusion_role{ ExtrusionRole::None };
 
 	// Various print settings?
 
@@ -79,8 +93,7 @@ struct SurfaceFillParams
 		RETURN_COMPARE_NON_EQUAL(flow.height());
 		RETURN_COMPARE_NON_EQUAL(flow.nozzle_diameter());
 		RETURN_COMPARE_NON_EQUAL_TYPED(unsigned, bridge);
-		RETURN_COMPARE_NON_EQUAL_TYPED(unsigned, extrusion_role);
-		return false;
+		return this->extrusion_role.lower(rhs.extrusion_role);
 	}
 
 	bool operator==(const SurfaceFillParams &rhs) const {
@@ -125,8 +138,8 @@ std::vector<SurfaceFill> group_fills(const Layer &layer)
     bool 												has_internal_voids = false;
 	for (size_t region_id = 0; region_id < layer.regions().size(); ++ region_id) {
 		const LayerRegion  &layerm = *layer.regions()[region_id];
-		region_to_surface_params[region_id].assign(layerm.fill_surfaces.size(), nullptr);
-	    for (const Surface &surface : layerm.fill_surfaces.surfaces)
+		region_to_surface_params[region_id].assign(layerm.fill_surfaces().size(), nullptr);
+	    for (const Surface &surface : layerm.fill_surfaces())
 	        if (surface.surface_type == stInternalVoid)
 	        	has_internal_voids = true;
 	        else {
@@ -148,17 +161,18 @@ std::vector<SurfaceFill> group_fills(const Layer &layer)
 
 		        params.extrusion_role =
 		            is_bridge ?
-		                erBridgeInfill :
+		                ExtrusionRole::BridgeInfill :
 		                (surface.is_solid() ?
-		                    (surface.is_top() ? erTopSolidInfill : erSolidInfill) :
-		                    erInternalInfill);
+		                    (surface.is_top() ? ExtrusionRole::TopSolidInfill : ExtrusionRole::SolidInfill) :
+							ExtrusionRole::InternalInfill);
 		        params.bridge_angle = float(surface.bridge_angle);
 		        params.angle 		= float(Geometry::deg2rad(region_config.fill_angle.value));
 		        
 		        // Calculate the actual flow we'll be using for this infill.
 		        params.bridge = is_bridge || Fill::use_bridge_flow(params.pattern);
 				params.flow   = params.bridge ?
-					layerm.bridging_flow(extrusion_role) :
+					// Always enable thick bridges for internal bridges.
+					layerm.bridging_flow(extrusion_role, surface.is_bridge() && ! surface.is_external()) :
 					layerm.flow(extrusion_role, (surface.thickness == -1) ? layer.height : surface.thickness);
 
 				// Calculate flow spacing for infill pattern generation.
@@ -184,7 +198,7 @@ std::vector<SurfaceFill> group_fills(const Layer &layer)
 		        auto it_params = set_surface_params.find(params);
 		        if (it_params == set_surface_params.end())
 		        	it_params = set_surface_params.insert(it_params, params);
-		        region_to_surface_params[region_id][&surface - &layerm.fill_surfaces.surfaces.front()] = &(*it_params);
+		        region_to_surface_params[region_id][&surface - &layerm.fill_surfaces().surfaces.front()] = &(*it_params);
 		    }
 	}
 
@@ -196,9 +210,9 @@ std::vector<SurfaceFill> group_fills(const Layer &layer)
 
 	for (size_t region_id = 0; region_id < layer.regions().size(); ++ region_id) {
 		const LayerRegion &layerm = *layer.regions()[region_id];
-	    for (const Surface &surface : layerm.fill_surfaces.surfaces)
+	    for (const Surface &surface : layerm.fill_surfaces())
 	        if (surface.surface_type != stInternalVoid) {
-	        	const SurfaceFillParams *params = region_to_surface_params[region_id][&surface - &layerm.fill_surfaces.surfaces.front()];
+	        	const SurfaceFillParams *params = region_to_surface_params[region_id][&surface - &layerm.fill_surfaces().surfaces.front()];
 				if (params != nullptr) {
 	        		SurfaceFill &fill = surface_fills[params->idx];
                     if (fill.region_id == size_t(-1)) {
@@ -285,7 +299,7 @@ std::vector<SurfaceFill> group_fills(const Layer &layer)
 		        params.extruder 	 = layerm.region().extruder(frSolidInfill);
 	            params.pattern 		 = fill_type_monotonic(layerm.region().config().top_fill_pattern) ? ipMonotonic : ipRectilinear;
 	            params.density 		 = 100.f;
-		        params.extrusion_role = erInternalInfill;
+		        params.extrusion_role = ExtrusionRole::InternalInfill;
 		        params.angle 		= float(Geometry::deg2rad(layerm.region().config().fill_angle.value));
 		        // calculate the actual flow we'll be using for this infill
 				params.flow = layerm.flow(frSolidInfill);
@@ -301,7 +315,15 @@ std::vector<SurfaceFill> group_fills(const Layer &layer)
 		}
     }
 
-	return surface_fills;
+    // Use ipEnsuring pattern for all internal Solids.
+    {
+        for (size_t surface_fill_id = 0; surface_fill_id < surface_fills.size(); ++surface_fill_id)
+            if (SurfaceFill &fill = surface_fills[surface_fill_id]; fill.surface.surface_type == stInternalSolid) {
+                fill.params.pattern = ipEnsuring;
+            }
+    }
+
+    return surface_fills;
 }
 
 #ifdef SLIC3R_DEBUG_SLICE_PROCESSING
@@ -325,12 +347,104 @@ void export_group_fills_to_svg(const char *path, const std::vector<SurfaceFill>
 }
 #endif
 
-// friend to Layer
+static void insert_fills_into_islands(Layer &layer, uint32_t fill_region_id, uint32_t fill_begin, uint32_t fill_end)
+{
+	if (fill_begin < fill_end) {
+    	// Sort the extrusion range into its LayerIsland.
+	    // Traverse the slices in an increasing order of bounding box size, so that the islands inside another islands are tested first,
+	    // so we can just test a point inside ExPolygon::contour and we may skip testing the holes.
+	    auto point_inside_surface = [&layer](const size_t lslice_idx, const Point &point) {
+	        const BoundingBox &bbox = layer.lslices_ex[lslice_idx].bbox;
+	        return point.x() >= bbox.min.x() && point.x() < bbox.max.x() &&
+	               point.y() >= bbox.min.y() && point.y() < bbox.max.y() &&
+	               layer.lslices[lslice_idx].contour.contains(point);
+	    };
+	    Point point = layer.get_region(fill_region_id)->fills().entities[fill_begin]->first_point();
+	    int lslice_idx = int(layer.lslices_ex.size()) - 1;
+	    for (; lslice_idx >= 0; -- lslice_idx)
+	        if (point_inside_surface(lslice_idx, point))
+	        	break;
+	    assert(lslice_idx >= 0);
+	    if (lslice_idx >= 0) {
+	    	LayerSlice &lslice = layer.lslices_ex[lslice_idx];
+	    	// Find an island.
+	    	LayerIsland *island = nullptr;
+	    	if (lslice.islands.size() == 1) {
+	    		// Cool, just save the extrusions in there.
+	    		island = &lslice.islands.front();
+	    	} else {
+	    		// The infill was created for one of the infills.
+	    		// In case of ironing, the infill may not fall into any of the infill expolygons either.
+	    		// In case of some numerical error, the infill may not fall into any of the infill expolygons either.
+	    		// 1) Try an exact test, it should be cheaper than a closest region test.
+	    		for (LayerIsland &li : lslice.islands) {
+	    			const BoundingBoxes &bboxes     = li.fill_expolygons_composite() ?
+	    				layer.get_region(li.perimeters.region())->fill_expolygons_composite_bboxes() :
+	    				layer.get_region(li.fill_region_id)->fill_expolygons_bboxes();
+	    			const ExPolygons 	&expolygons = li.fill_expolygons_composite() ? 
+	    				layer.get_region(li.perimeters.region())->fill_expolygons_composite() :
+	    				layer.get_region(li.fill_region_id)->fill_expolygons();
+	    			for (uint32_t fill_expolygon_id : li.fill_expolygons)
+	    				if (bboxes[fill_expolygon_id].contains(point) && expolygons[fill_expolygon_id].contains(point)) {
+	    					island = &li;
+	    					goto found;
+	    				}
+	    		}
+	    		// 2) Find closest fill_expolygon, branch and bound by distance to bounding box.
+				{
+					struct Island {
+						uint32_t island_idx;
+						uint32_t expolygon_idx;
+						double   distance2;
+					};
+	    			std::vector<Island> islands_sorted;
+	    			for (uint32_t island_idx = 0; island_idx < uint32_t(lslice.islands.size()); ++ island_idx) {
+	    				const LayerIsland   &li     = lslice.islands[island_idx];
+	    				const BoundingBoxes &bboxes = li.fill_expolygons_composite() ?
+	    					layer.get_region(li.perimeters.region())->fill_expolygons_composite_bboxes() :
+	    					layer.get_region(li.fill_region_id)->fill_expolygons_bboxes();
+	    				for (uint32_t fill_expolygon_id : li.fill_expolygons)
+							islands_sorted.push_back({ island_idx, fill_expolygon_id, bbox_point_distance_squared(bboxes[fill_expolygon_id], point) });
+	    			}
+	    			std::sort(islands_sorted.begin(), islands_sorted.end(), [](auto &l, auto &r){ return l.distance2 < r.distance2; });
+	    			auto dist_min2 = std::numeric_limits<double>::max();
+	    			for (uint32_t sorted_bbox_idx = 0; sorted_bbox_idx < uint32_t(islands_sorted.size()); ++ sorted_bbox_idx) {
+	    				const Island &isl = islands_sorted[sorted_bbox_idx];
+	    				if (isl.distance2 > dist_min2)
+							// Branch & bound condition.
+	    					break;
+	    				LayerIsland		  &li         = lslice.islands[isl.island_idx];
+	    				const ExPolygons  &expolygons = li.fill_expolygons_composite() ?
+	    					layer.get_region(li.perimeters.region())->fill_expolygons_composite() :
+	    					layer.get_region(li.fill_region_id)->fill_expolygons();
+	    				double d2 = (expolygons[isl.expolygon_idx].point_projection(point) - point).cast<double>().squaredNorm();
+	    				if (d2 < dist_min2) {
+	    					dist_min2 = d2;
+	    					island = &li;
+	    				}
+	    			}
+				}
+	    	found:;
+	    	}
+	    	assert(island);
+	    	if (island)
+	    		island->add_fill_range(LayerExtrusionRange{ fill_region_id, { fill_begin, fill_end }});
+	    }
+    }
+}
+
+void Layer::clear_fills()
+{
+    for (LayerRegion *layerm : m_regions)
+        layerm->m_fills.clear();
+    for (LayerSlice &lslice : lslices_ex)
+		for (LayerIsland &island : lslice.islands)
+			island.fills.clear();
+}
+
 void Layer::make_fills(FillAdaptive::Octree* adaptive_fill_octree, FillAdaptive::Octree* support_fill_octree, FillLightning::Generator* lightning_generator)
 {
-	for (LayerRegion *layerm : m_regions)
-		layerm->fills.clear();
-
+	this->clear_fills();
 
 #ifdef SLIC3R_DEBUG_SLICE_PROCESSING
 //	this->export_region_fill_surfaces_to_svg_debug("10_fill-initial");
@@ -348,23 +462,28 @@ void Layer::make_fills(FillAdaptive::Octree* adaptive_fill_octree, FillAdaptive:
 	}
 #endif /* SLIC3R_DEBUG_SLICE_PROCESSING */
 
+	size_t first_object_layer_id = this->object()->get_layer(0)->id();
     for (SurfaceFill &surface_fill : surface_fills) {
         // Create the filler object.
         std::unique_ptr<Fill> f = std::unique_ptr<Fill>(Fill::new_from_type(surface_fill.params.pattern));
         f->set_bounding_box(bbox);
-        f->layer_id = this->id();
+		// Layer ID is used for orienting the infill in alternating directions.
+		// Layer::id() returns layer ID including raft layers, subtract them to make the infill direction independent
+		// from raft.
+        f->layer_id = this->id() - first_object_layer_id;
         f->z 		= this->print_z;
         f->angle 	= surface_fill.params.angle;
-        f->adapt_fill_octree = (surface_fill.params.pattern == ipSupportCubic) ? support_fill_octree : adaptive_fill_octree;
+        f->adapt_fill_octree   = (surface_fill.params.pattern == ipSupportCubic) ? support_fill_octree : adaptive_fill_octree;
+        f->print_config        = &this->object()->print()->config();
+        f->print_object_config = &this->object()->config();
 
         if (surface_fill.params.pattern == ipLightning)
             dynamic_cast<FillLightning::Filler*>(f.get())->generator = lightning_generator;
 
-        if (perimeter_generator.value == PerimeterGeneratorType::Arachne && surface_fill.params.pattern == ipConcentric) {
-            FillConcentric *fill_concentric = dynamic_cast<FillConcentric *>(f.get());
-            assert(fill_concentric != nullptr);
-            fill_concentric->print_config        = &this->object()->print()->config();
-            fill_concentric->print_object_config = &this->object()->config();
+        if (surface_fill.params.pattern == ipEnsuring) {
+            auto *fill_ensuring = dynamic_cast<FillEnsuring *>(f.get());
+            assert(fill_ensuring != nullptr);
+            fill_ensuring->print_region_config = &m_regions[surface_fill.region_id]->region().config();
         }
 
         // calculate flow spacing for infill pattern generation
@@ -385,6 +504,8 @@ void Layer::make_fills(FillAdaptive::Octree* adaptive_fill_octree, FillAdaptive:
         // Used by the concentric infill pattern to clip the loops to create extrusion paths.
         f->loop_clipping = coord_t(scale_(surface_fill.params.flow.nozzle_diameter()) * LOOP_CLIPPING_LENGTH_OVER_NOZZLE_DIAMETER);
 
+        LayerRegion &layerm = *m_regions[surface_fill.region_id];
+
         // apply half spacing using this flow's own spacing and generate infill
         FillParams params;
         params.density           = float(0.01 * surface_fill.params.density);
@@ -392,8 +513,8 @@ void Layer::make_fills(FillAdaptive::Octree* adaptive_fill_octree, FillAdaptive:
         params.anchor_length     = surface_fill.params.anchor_length;
         params.anchor_length_max = surface_fill.params.anchor_length_max;
         params.resolution        = resolution;
-        params.use_arachne       = perimeter_generator == PerimeterGeneratorType::Arachne && surface_fill.params.pattern == ipConcentric;
-        params.layer_height      = m_regions[surface_fill.region_id]->layer()->height;
+        params.use_arachne       = (perimeter_generator == PerimeterGeneratorType::Arachne && surface_fill.params.pattern == ipConcentric) || surface_fill.params.pattern == ipEnsuring;
+        params.layer_height      = layerm.layer()->height;
 
         for (ExPolygon &expoly : surface_fill.expolygons) {
 			// Spacing is modified by the filler to indicate adjustments. Reset it for each expolygon.
@@ -424,14 +545,15 @@ void Layer::make_fills(FillAdaptive::Octree* adaptive_fill_octree, FillAdaptive:
 		        }
 		        // Save into layer.
 				ExtrusionEntityCollection* eec = nullptr;
-		        m_regions[surface_fill.region_id]->fills.entities.push_back(eec = new ExtrusionEntityCollection());
+				auto fill_begin = uint32_t(layerm.fills().size());
+		        layerm.m_fills.entities.push_back(eec = new ExtrusionEntityCollection());
 		        // Only concentric fills are not sorted.
 		        eec->no_sort = f->no_sort();
                 if (params.use_arachne) {
                     for (const ThickPolyline &thick_polyline : thick_polylines) {
                         Flow new_flow = surface_fill.params.flow.with_spacing(float(f->spacing));
 
-                        ExtrusionMultiPath multi_path = thick_polyline_to_multi_path(thick_polyline, surface_fill.params.extrusion_role, new_flow, scaled<float>(0.05), float(SCALED_EPSILON));
+                        ExtrusionMultiPath multi_path = PerimeterGenerator::thick_polyline_to_multi_path(thick_polyline, surface_fill.params.extrusion_role, new_flow, scaled<float>(0.05), float(SCALED_EPSILON));
                         // Append paths to collection.
                         if (!multi_path.empty()) {
                             if (multi_path.paths.front().first_point() == multi_path.paths.back().last_point())
@@ -448,28 +570,144 @@ void Layer::make_fills(FillAdaptive::Octree* adaptive_fill_octree, FillAdaptive:
                         surface_fill.params.extrusion_role,
                         flow_mm3_per_mm, float(flow_width), surface_fill.params.flow.height());
                 }
+                insert_fills_into_islands(*this, uint32_t(surface_fill.region_id), fill_begin, uint32_t(layerm.fills().size()));
 		    }
 		}
     }
 
-    // add thin fill regions
-    // Unpacks the collection, creates multiple collections per path.
-    // The path type could be ExtrusionPath, ExtrusionLoop or ExtrusionEntityCollection.
-    // Why the paths are unpacked?
-	for (LayerRegion *layerm : m_regions)
-	    for (const ExtrusionEntity *thin_fill : layerm->thin_fills.entities) {
-	        ExtrusionEntityCollection &collection = *(new ExtrusionEntityCollection());
-	        layerm->fills.entities.push_back(&collection);
-	        collection.entities.push_back(thin_fill->clone());
-	    }
+	for (LayerSlice &lslice : this->lslices_ex)
+		for (LayerIsland &island : lslice.islands) {
+			if (! island.thin_fills.empty()) {
+				// Copy thin fills into fills packed as a collection.
+				// Fills are always stored as collections, the rest of the pipeline (wipe into infill, G-code generator) relies on it.
+				LayerRegion				  &layerm	  = *this->get_region(island.perimeters.region());
+				ExtrusionEntityCollection &collection = *(new ExtrusionEntityCollection());
+				layerm.m_fills.entities.push_back(&collection);
+				collection.entities.reserve(island.thin_fills.size());
+				for (uint32_t fill_id : island.thin_fills)
+					collection.entities.push_back(layerm.thin_fills().entities[fill_id]->clone());
+	    		island.add_fill_range({ island.perimeters.region(), { uint32_t(layerm.m_fills.entities.size() - 1), uint32_t(layerm.m_fills.entities.size()) } });
+			}
+			// Sort the fills by region ID.
+			std::sort(island.fills.begin(), island.fills.end(), [](auto &l, auto &r){ return l.region() < r.region() || (l.region() == r.region() && *l.begin() < *r.begin()); });
+			// Compress continuous fill ranges of the same region.
+			{
+				size_t k = 0;
+				for (size_t i = 0; i < island.fills.size();) {
+					uint32_t region_id = island.fills[i].region();
+					uint32_t begin     = *island.fills[i].begin();
+					uint32_t end       = *island.fills[i].end();
+					size_t   j         = i + 1;
+					for (; j < island.fills.size() && island.fills[j].region() == region_id && *island.fills[j].begin() == end; ++ j)
+						end = *island.fills[j].end();
+					island.fills[k ++] = { region_id, { begin, end } };
+					i = j;
+				}
+				island.fills.erase(island.fills.begin() + k, island.fills.end());
+			}
+		}
 
 #ifndef NDEBUG
 	for (LayerRegion *layerm : m_regions)
-	    for (size_t i = 0; i < layerm->fills.entities.size(); ++ i)
-    	    assert(dynamic_cast<ExtrusionEntityCollection*>(layerm->fills.entities[i]) != nullptr);
+	    for (const ExtrusionEntity *e : layerm->fills())
+    	    assert(dynamic_cast<const ExtrusionEntityCollection*>(e) != nullptr);
 #endif
 }
 
+Polylines Layer::generate_sparse_infill_polylines_for_anchoring(FillAdaptive::Octree* adaptive_fill_octree, FillAdaptive::Octree* support_fill_octree,  FillLightning::Generator* lightning_generator) const
+{
+    std::vector<SurfaceFill>  surface_fills = group_fills(*this);
+    const Slic3r::BoundingBox bbox          = this->object()->bounding_box();
+    const auto                resolution    = this->object()->print()->config().gcode_resolution.value;
+
+    Polylines sparse_infill_polylines{};
+
+    for (SurfaceFill &surface_fill : surface_fills) {
+		if (surface_fill.surface.surface_type != stInternal) {
+			continue;
+		}
+
+        switch (surface_fill.params.pattern) {
+        case ipCount: continue; break;
+        case ipSupportBase: continue; break;
+        case ipEnsuring: continue; break;
+        case ipLightning:
+		case ipAdaptiveCubic:
+        case ipSupportCubic:
+        case ipRectilinear:
+        case ipMonotonic:
+        case ipMonotonicLines:
+        case ipAlignedRectilinear:
+        case ipGrid:
+        case ipTriangles:
+        case ipStars:
+        case ipCubic:
+        case ipLine:
+        case ipConcentric:
+        case ipHoneycomb:
+        case ip3DHoneycomb:
+        case ipGyroid:
+        case ipHilbertCurve:
+        case ipArchimedeanChords:
+        case ipOctagramSpiral: break;
+        }
+
+        // Create the filler object.
+        std::unique_ptr<Fill> f = std::unique_ptr<Fill>(Fill::new_from_type(surface_fill.params.pattern));
+        f->set_bounding_box(bbox);
+        f->layer_id = this->id() - this->object()->get_layer(0)->id(); // We need to subtract raft layers.
+        f->z        = this->print_z;
+        f->angle    = surface_fill.params.angle;
+        f->adapt_fill_octree   = (surface_fill.params.pattern == ipSupportCubic) ? support_fill_octree : adaptive_fill_octree;
+        f->print_config        = &this->object()->print()->config();
+        f->print_object_config = &this->object()->config();
+
+        if (surface_fill.params.pattern == ipLightning)
+            dynamic_cast<FillLightning::Filler *>(f.get())->generator = lightning_generator;
+
+        // calculate flow spacing for infill pattern generation
+        double link_max_length = 0.;
+        if (!surface_fill.params.bridge) {
+#if 0
+            link_max_length = layerm.region()->config().get_abs_value(surface.is_external() ? "external_fill_link_max_length" : "fill_link_max_length", flow.spacing());
+//            printf("flow spacing: %f,  is_external: %d, link_max_length: %lf\n", flow.spacing(), int(surface.is_external()), link_max_length);
+#else
+            if (surface_fill.params.density > 80.) // 80%
+                link_max_length = 3. * f->spacing;
+#endif
+        }
+
+        // Maximum length of the perimeter segment linking two infill lines.
+        f->link_max_length = (coord_t) scale_(link_max_length);
+        // Used by the concentric infill pattern to clip the loops to create extrusion paths.
+        f->loop_clipping = coord_t(scale_(surface_fill.params.flow.nozzle_diameter()) * LOOP_CLIPPING_LENGTH_OVER_NOZZLE_DIAMETER);
+
+        LayerRegion &layerm = *m_regions[surface_fill.region_id];
+
+        // apply half spacing using this flow's own spacing and generate infill
+        FillParams params;
+        params.density           = float(0.01 * surface_fill.params.density);
+        params.dont_adjust       = false; //  surface_fill.params.dont_adjust;
+        params.anchor_length     = surface_fill.params.anchor_length;
+        params.anchor_length_max = surface_fill.params.anchor_length_max;
+        params.resolution        = resolution;
+        params.use_arachne       = false;
+        params.layer_height      = layerm.layer()->height;
+
+        for (ExPolygon &expoly : surface_fill.expolygons) {
+            // Spacing is modified by the filler to indicate adjustments. Reset it for each expolygon.
+            f->spacing                     = surface_fill.params.spacing;
+            surface_fill.surface.expolygon = std::move(expoly);
+            try {
+                Polylines polylines = f->fill_surface(&surface_fill.surface, params);
+                sparse_infill_polylines.insert(sparse_infill_polylines.end(), polylines.begin(), polylines.end());
+            } catch (InfillFailedException &) {}
+        }
+    }
+
+    return sparse_infill_polylines;
+}
+
 // Create ironing extrusions over top surfaces.
 void Layer::make_ironing()
 {
@@ -522,7 +760,8 @@ void Layer::make_ironing()
 				   this->angle == rhs.angle;
 		}
 
-		LayerRegion *layerm		= nullptr;
+		LayerRegion *layerm;
+		uint32_t     region_id;
 
 		// IdeaMaker: ironing
 		// ironing flowrate (5% percent)
@@ -542,8 +781,8 @@ void Layer::make_ironing()
 	std::vector<IroningParams> by_extruder;
     double default_layer_height = this->object()->config().layer_height;
 
-	for (LayerRegion *layerm : m_regions)
-		if (! layerm->slices.empty()) {
+	for (uint32_t region_id = 0; region_id < uint32_t(this->regions().size()); ++region_id)
+		if (LayerRegion *layerm = this->get_region(region_id); ! layerm->slices().empty()) {
 			IroningParams ironing_params;
 			const PrintRegionConfig &config = layerm->region().config();
 			if (config.ironing && 
@@ -567,6 +806,7 @@ void Layer::make_ironing()
 				ironing_params.speed 		= config.ironing_speed;
 				ironing_params.angle 		= config.fill_angle * M_PI / 180.;
 				ironing_params.layerm 		= layerm;
+				ironing_params.region_id    = region_id;
 				by_extruder.emplace_back(ironing_params);
 			}
 		}
@@ -575,7 +815,11 @@ void Layer::make_ironing()
     FillRectilinear 	fill;
     FillParams 			fill_params;
 	fill.set_bounding_box(this->object()->bounding_box());
-	fill.layer_id 			 = this->id();
+	// Layer ID is used for orienting the infill in alternating directions.
+	// Layer::id() returns layer ID including raft layers, subtract them to make the infill direction independent
+	// from raft.
+	//FIXME ironing does not take fill angle into account. Shall it? Does it matter?
+	fill.layer_id 			 = this->id() - this->object()->get_layer(0)->id();
     fill.z 					 = this->print_z;
     fill.overlap 			 = 0;
     fill_params.density 	 = 1.;
@@ -606,7 +850,7 @@ void Layer::make_ironing()
 				if (iron_everything) {
 					// Check whether there is any non-solid hole in the regions.
 					bool internal_infill_solid = region_config.fill_density.value > 95.;
-					for (const Surface &surface : ironing_params.layerm->fill_surfaces.surfaces)
+					for (const Surface &surface : ironing_params.layerm->fill_surfaces())
 						if ((! internal_infill_solid && surface.surface_type == stInternal) || surface.surface_type == stInternalBridge || surface.surface_type == stInternalVoid) {
 							// Some fill region is not quite solid. Don't iron over the whole surface.
 							iron_completely = false;
@@ -615,10 +859,10 @@ void Layer::make_ironing()
 				}
 				if (iron_completely) {
 					// Iron everything. This is likely only good for solid transparent objects.
-					for (const Surface &surface : ironing_params.layerm->slices.surfaces)
+					for (const Surface &surface : ironing_params.layerm->slices())
 						polygons_append(polys, surface.expolygon);
 				} else {
-					for (const Surface &surface : ironing_params.layerm->slices.surfaces)
+					for (const Surface &surface : ironing_params.layerm->slices())
 						if (surface.surface_type == stTop || (iron_everything && surface.surface_type == stBottom))
 							// stBottomBridge is not being ironed on purpose, as it would likely destroy the bridges.
 							polygons_append(polys, surface.expolygon);
@@ -626,7 +870,7 @@ void Layer::make_ironing()
 				if (iron_everything && ! iron_completely) {
 					// Add solid fill surfaces. This may not be ideal, as one will not iron perimeters touching these
 					// solid fill surfaces, but it is likely better than nothing.
-					for (const Surface &surface : ironing_params.layerm->fill_surfaces.surfaces)
+					for (const Surface &surface : ironing_params.layerm->fill_surfaces())
 						if (surface.surface_type == stInternalSolid)
 							polygons_append(infills, surface.expolygon);
 				}
@@ -662,14 +906,16 @@ void Layer::make_ironing()
 			}
 	        if (! polylines.empty()) {
 		        // Save into layer.
+				auto fill_begin = uint32_t(ironing_params.layerm->fills().size());
 				ExtrusionEntityCollection *eec = nullptr;
-		        ironing_params.layerm->fills.entities.push_back(eec = new ExtrusionEntityCollection());
+		        ironing_params.layerm->m_fills.entities.push_back(eec = new ExtrusionEntityCollection());
 		        // Don't sort the ironing infill lines as they are monotonicly ordered.
 				eec->no_sort = true;
 		        extrusion_entities_append_paths(
 		            eec->entities, std::move(polylines),
-		            erIroning,
+					ExtrusionRole::Ironing,
 		            flow_mm3_per_mm, extrusion_width, float(extrusion_height));
+				insert_fills_into_islands(*this, ironing_params.region_id, fill_begin, uint32_t(ironing_params.layerm->fills().size()));
 		    }
 		}
 
