@@ -1,6 +1,22 @@
+///|/ Copyright (c) Prusa Research 2016 - 2023 Vojtěch Bubník @bubnikv, Lukáš Matěna @lukasmatena, Pavel Mikuš @Godrak, Lukáš Hejl @hejllukas, Filip Sykala @Jony01, Enrico Turri @enricoturri1966, David Kocík @kocikdav, Oleksandra Iushchenko @YuSanka
+///|/ Copyright (c) SuperSlicer 2023 Remi Durand @supermerill
+///|/ Copyright (c) 2019 Thomas Moore
+///|/ Copyright (c) 2016 Chow Loong Jin @hyperair
+///|/ Copyright (c) Slic3r 2014 - 2015 Alessandro Ranellucci @alranel
+///|/
+///|/ ported from lib/Slic3r/GCode.pm:
+///|/ Copyright (c) Slic3r 2011 - 2015 Alessandro Ranellucci @alranel
+///|/ Copyright (c) 2013 Robert Giseburt
+///|/ Copyright (c) 2012 Mark Hindess
+///|/ Copyright (c) 2012 Henrik Brix Andersen @henrikbrixandersen
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #ifndef slic3r_GCode_hpp_
 #define slic3r_GCode_hpp_
 
+#include "GCode/ExtrusionProcessor.hpp"
+#include "JumpPointSearch.hpp"
 #include "libslic3r.h"
 #include "ExPolygon.hpp"
 #include "GCodeWriter.hpp"
@@ -11,6 +27,7 @@
 #include "GCode/AvoidCrossingPerimeters.hpp"
 #include "GCode/CoolingBuffer.hpp"
 #include "GCode/FindReplace.hpp"
+#include "GCode/RetractWhenCrossingPerimeters.hpp"
 #include "GCode/SpiralVase.hpp"
 #include "GCode/ToolOrdering.hpp"
 #include "GCode/WipeTower.hpp"
@@ -32,19 +49,17 @@ class GCode;
 
 namespace { struct Item; }
 struct PrintInstance;
-class ConstPrintObjectPtrsAdaptor;
 
 class OozePrevention {
 public:
     bool enable;
-    Points standby_points;
     
     OozePrevention() : enable(false) {}
     std::string pre_toolchange(GCode &gcodegen);
     std::string post_toolchange(GCode &gcodegen);
     
 private:
-    int _get_temp(GCode &gcodegen);
+    int _get_temp(const GCode &gcodegen) const;
 };
 
 class Wipe {
@@ -53,9 +68,9 @@ public:
     Polyline path;
     
     Wipe() : enable(false) {}
-    bool has_path() const { return !this->path.points.empty(); }
-    void reset_path() { this->path = Polyline(); }
-    std::string wipe(GCode &gcodegen, bool toolchange = false);
+    bool has_path() const { return ! this->path.empty(); }
+    void reset_path() { this->path.clear(); }
+    std::string wipe(GCode &gcodegen, bool toolchange);
 };
 
 class WipeTowerIntegration {
@@ -135,14 +150,14 @@ public:
         m_enable_loop_clipping(true), 
         m_enable_cooling_markers(false), 
         m_enable_extrusion_role_markers(false),
-        m_last_processor_extrusion_role(erNone),
+        m_last_processor_extrusion_role(GCodeExtrusionRole::None),
         m_layer_count(0),
         m_layer_index(-1), 
         m_layer(nullptr),
         m_object_layer_over_raft(false),
         m_volumetric_speed(0),
         m_last_pos_defined(false),
-        m_last_extrusion_role(erNone),
+        m_last_extrusion_role(GCodeExtrusionRole::None),
         m_last_width(0.0f),
 #if ENABLE_GCODE_VIEWER_DATA_CHECKING
         m_last_mm3_per_mm(0.0),
@@ -163,14 +178,17 @@ public:
     void            set_origin(const Vec2d &pointf);
     void            set_origin(const coordf_t x, const coordf_t y) { this->set_origin(Vec2d(x, y)); }
     const Point&    last_pos() const { return m_last_pos; }
+    // Convert coordinates of the active object to G-code coordinates, possibly adjusted for extruder offset.
     Vec2d           point_to_gcode(const Point &point) const;
+    // Convert coordinates of the active object to G-code coordinates, possibly adjusted for extruder offset and quantized to G-code resolution.
+    Vec2d           point_to_gcode_quantized(const Point &point) const;
     Point           gcode_to_point(const Vec2d &point) const;
     const FullPrintConfig &config() const { return m_config; }
     const Layer*    layer() const { return m_layer; }
     GCodeWriter&    writer() { return m_writer; }
     const GCodeWriter& writer() const { return m_writer; }
-    PlaceholderParser& placeholder_parser() { return m_placeholder_parser; }
-    const PlaceholderParser& placeholder_parser() const { return m_placeholder_parser; }
+    PlaceholderParser& placeholder_parser() { return m_placeholder_parser_integration.parser; }
+    const PlaceholderParser& placeholder_parser() const { return m_placeholder_parser_integration.parser; }
     // Process a template through the placeholder parser, collect error messages to be reported
     // inside the generated string and after the G-code export finishes.
     std::string     placeholder_parser_process(const std::string &name, const std::string &templ, unsigned int current_extruder_id, const DynamicConfig *config_override = nullptr);
@@ -186,15 +204,16 @@ public:
 
     // Object and support extrusions of the same PrintObject at the same print_z.
     // public, so that it could be accessed by free helper functions from GCode.cpp
-    struct LayerToPrint
+    struct ObjectLayerToPrint
     {
-        LayerToPrint() : object_layer(nullptr), support_layer(nullptr) {}
+        ObjectLayerToPrint() : object_layer(nullptr), support_layer(nullptr) {}
         const Layer* 		object_layer;
         const SupportLayer* support_layer;
         const Layer* 		layer()   const { return (object_layer != nullptr) ? object_layer : support_layer; }
         const PrintObject* 	object()  const { return (this->layer() != nullptr) ? this->layer()->object() : nullptr; }
         coordf_t            print_z() const { return (object_layer != nullptr && support_layer != nullptr) ? 0.5 * (object_layer->print_z + support_layer->print_z) : this->layer()->print_z; }
     };
+    using ObjectsLayerToPrint = std::vector<ObjectLayerToPrint>;
 
 private:
     class GCodeOutputStream {
@@ -237,13 +256,13 @@ private:
     };
     void            _do_export(Print &print, GCodeOutputStream &file, ThumbnailsGeneratorCallback thumbnail_cb);
 
-    static std::vector<LayerToPrint>        		                   collect_layers_to_print(const PrintObject &object);
-    static std::vector<std::pair<coordf_t, std::vector<LayerToPrint>>> collect_layers_to_print(const Print &print);
+    static ObjectsLayerToPrint         		                     collect_layers_to_print(const PrintObject &object);
+    static std::vector<std::pair<coordf_t, ObjectsLayerToPrint>> collect_layers_to_print(const Print &print);
 
     LayerResult process_layer(
         const Print                     &print,
         // Set of object & print layers of the same PrintObject and with the same print_z.
-        const std::vector<LayerToPrint> &layers,
+        const ObjectsLayerToPrint       &layers,
         const LayerTools  				&layer_tools,
         const bool                       last_layer,
 		// Pairs of PrintObject index and its instance index.
@@ -255,18 +274,18 @@ private:
     // Generate G-code, run the filters (vase mode, cooling buffer), run the G-code analyser
     // and export G-code into file.
     void process_layers(
-        const Print                                                         &print,
-        const ToolOrdering                                                  &tool_ordering,
-        const std::vector<const PrintInstance*>                             &print_object_instances_ordering,
-        const std::vector<std::pair<coordf_t, std::vector<LayerToPrint>>>   &layers_to_print,
-        GCodeOutputStream                                                   &output_stream);
+        const Print                                                   &print,
+        const ToolOrdering                                            &tool_ordering,
+        const std::vector<const PrintInstance*>                       &print_object_instances_ordering,
+        const std::vector<std::pair<coordf_t, ObjectsLayerToPrint>>   &layers_to_print,
+        GCodeOutputStream                                             &output_stream);
     // Process all layers of a single object instance (sequential mode) with a parallel pipeline:
     // Generate G-code, run the filters (vase mode, cooling buffer), run the G-code analyser
     // and export G-code into file.
     void process_layers(
         const Print                             &print,
         const ToolOrdering                      &tool_ordering,
-        std::vector<LayerToPrint>                layers_to_print,
+        ObjectsLayerToPrint                      layers_to_print,
         const size_t                             single_object_idx,
         GCodeOutputStream                       &output_stream);
 
@@ -275,80 +294,52 @@ private:
     void            set_extruders(const std::vector<unsigned int> &extruder_ids);
     std::string     preamble();
     std::string     change_layer(coordf_t print_z);
-    std::string     extrude_entity(const ExtrusionEntity &entity, std::string description = "", double speed = -1.);
-    std::string     extrude_loop(ExtrusionLoop loop, std::string description, double speed = -1.);
-    std::string     extrude_multi_path(ExtrusionMultiPath multipath, std::string description = "", double speed = -1.);
-    std::string     extrude_path(ExtrusionPath path, std::string description = "", double speed = -1.);
+    std::string     extrude_entity(const ExtrusionEntity &entity, const std::string_view description, double speed = -1.);
+    std::string     extrude_loop(ExtrusionLoop loop, const std::string_view description, double speed = -1.);
+    std::string     extrude_multi_path(ExtrusionMultiPath multipath, const std::string_view description, double speed = -1.);
+    std::string     extrude_path(ExtrusionPath path, const std::string_view description, double speed = -1.);
 
-    // Extruding multiple objects with soluble / non-soluble / combined supports
-    // on a multi-material printer, trying to minimize tool switches.
-    // Following structures sort extrusions by the extruder ID, by an order of objects and object islands.
-    struct ObjectByExtruder
+    struct InstanceToPrint
     {
-        ObjectByExtruder() : support(nullptr), support_extrusion_role(erNone) {}
-        const ExtrusionEntityCollection  *support;
-        // erSupportMaterial / erSupportMaterialInterface or erMixed.
-        ExtrusionRole                     support_extrusion_role;
+        InstanceToPrint(size_t object_layer_to_print_id, const PrintObject &print_object, size_t instance_id) :
+            object_layer_to_print_id(object_layer_to_print_id), print_object(print_object), instance_id(instance_id) {}
 
-        struct Island
-        {
-            struct Region {
-            	// Non-owned references to LayerRegion::perimeters::entities
-            	// std::vector<const ExtrusionEntity*> would be better here, but there is no way in C++ to convert from std::vector<T*> std::vector<const T*> without copying.
-                ExtrusionEntitiesPtr perimeters;
-            	// Non-owned references to LayerRegion::fills::entities
-                ExtrusionEntitiesPtr infills;
-
-                std::vector<const WipingExtrusions::ExtruderPerCopy*> infills_overrides;
-                std::vector<const WipingExtrusions::ExtruderPerCopy*> perimeters_overrides;
-
-	            enum Type {
-	            	PERIMETERS,
-	            	INFILL,
-	            };
-
-                // Appends perimeter/infill entities and writes don't indices of those that are not to be extruder as part of perimeter/infill wiping
-                void append(const Type type, const ExtrusionEntityCollection* eec, const WipingExtrusions::ExtruderPerCopy* copy_extruders);
-            };
-
-
-            std::vector<Region> by_region;                                    // all extrusions for this island, grouped by regions
-
-            // Fills in by_region_per_copy_cache and returns its reference.
-            const std::vector<Region>& by_region_per_copy(std::vector<Region> &by_region_per_copy_cache, unsigned int copy, unsigned int extruder, bool wiping_entities = false) const;
-        };
-        std::vector<Island>         islands;
+        // Index into std::vector<ObjectLayerToPrint>, which contains Object and Support layers for the current print_z, collected for a single object, or for possibly multiple objects with multiple instances.
+        const size_t             object_layer_to_print_id;
+        const PrintObject       &print_object;
+        // Instance idx of the copy of a print object.
+        const size_t             instance_id;
     };
 
-	struct InstanceToPrint
-	{
-		InstanceToPrint(ObjectByExtruder &object_by_extruder, size_t layer_id, const PrintObject &print_object, size_t instance_id) :
-			object_by_extruder(object_by_extruder), layer_id(layer_id), print_object(print_object), instance_id(instance_id) {}
+    std::vector<InstanceToPrint> sort_print_object_instances(
+        // Object and Support layers for the current print_z, collected for a single object, or for possibly multiple objects with multiple instances.
+        const std::vector<ObjectLayerToPrint>           &layers,
+        // Ordering must be defined for normal (non-sequential print).
+        const std::vector<const PrintInstance*>         *ordering,
+        // For sequential print, the instance of the object to be printing has to be defined.
+        const size_t                                     single_object_instance_idx);
 
-		// Repository 
-		ObjectByExtruder		&object_by_extruder;
-		// Index into std::vector<LayerToPrint>, which contains Object and Support layers for the current print_z, collected for a single object, or for possibly multiple objects with multiple instances.
-		const size_t       		 layer_id;
-		const PrintObject 		&print_object;
-		// Instance idx of the copy of a print object.
-		const size_t			 instance_id;
-	};
+    // This function will be called for each printing extruder, possibly twice: First for wiping extrusions, second for normal extrusions.
+    void process_layer_single_object(
+        // output
+        std::string              &gcode, 
+        // Index of the extruder currently active.
+        const unsigned int        extruder_id,
+        // What object and instance is going to be printed.
+        const InstanceToPrint    &print_instance,
+        // and the object & support layer of the above.
+        const ObjectLayerToPrint &layer_to_print, 
+        // Container for extruder overrides (when wiping into object or infill).
+        const LayerTools         &layer_tools,
+        // Is any extrusion possibly marked as wiping extrusion?
+        const bool                is_anything_overridden, 
+        // Round 1 (wiping into object or infill) or round 2 (normal extrusions).
+        const bool                print_wipe_extrusions);
 
-	std::vector<InstanceToPrint> sort_print_object_instances(
-		std::vector<ObjectByExtruder> 					&objects_by_extruder,
-		// Object and Support layers for the current print_z, collected for a single object, or for possibly multiple objects with multiple instances.
-		const std::vector<LayerToPrint> 				&layers,
-		// Ordering must be defined for normal (non-sequential print).
-		const std::vector<const PrintInstance*>     	*ordering,
-		// For sequential print, the instance of the object to be printing has to be defined.
-		const size_t                     				 single_object_instance_idx);
-
-    std::string     extrude_perimeters(const Print &print, const std::vector<ObjectByExtruder::Island::Region> &by_region);
-    std::string     extrude_infill(const Print &print, const std::vector<ObjectByExtruder::Island::Region> &by_region, bool ironing);
     std::string     extrude_support(const ExtrusionEntityCollection &support_fills);
 
     std::string     travel_to(const Point &point, ExtrusionRole role, std::string comment);
-    bool            needs_retraction(const Polyline &travel, ExtrusionRole role = erNone);
+    bool            needs_retraction(const Polyline &travel, ExtrusionRole role = ExtrusionRole::None);
     std::string     retract(bool toolchange = false);
     std::string     unretract() { return m_writer.unlift() + m_writer.unretract(); }
     std::string     set_extruder(unsigned int extruder_id, double print_z);
@@ -356,6 +347,8 @@ private:
     // Cache for custom seam enforcers/blockers for each layer.
     SeamPlacer                          m_seam_placer;
 
+    ExtrusionQualityEstimator           m_extrusion_quality_estimator;
+
     /* Origin of print coordinates expressed in unscaled G-code coordinates.
        This affects the input arguments supplied to the extrude*() and travel_to()
        methods. */
@@ -364,14 +357,42 @@ private:
     // scaled G-code resolution
     double                              m_scaled_resolution;
     GCodeWriter                         m_writer;
-    PlaceholderParser                   m_placeholder_parser;
-    // For random number generator etc.
-    PlaceholderParser::ContextData      m_placeholder_parser_context;
-    // Collection of templates, on which the placeholder substitution failed.
-    std::map<std::string, std::string>  m_placeholder_parser_failed_templates;
+
+    struct PlaceholderParserIntegration {
+        void reset();
+        void init(const GCodeWriter &config);
+        void update_from_gcodewriter(const GCodeWriter &writer);
+        void validate_output_vector_variables();
+
+        PlaceholderParser                   parser;
+        // For random number generator etc.
+        PlaceholderParser::ContextData      context;
+        // Collection of templates, on which the placeholder substitution failed.
+        std::map<std::string, std::string>  failed_templates;
+        // Input/output from/to custom G-code block, for returning position, retraction etc.
+        DynamicConfig                       output_config;
+        ConfigOptionFloats                 *opt_position { nullptr };
+        ConfigOptionFloat                  *opt_zhop { nullptr };
+        ConfigOptionFloats                 *opt_e_position { nullptr };
+        ConfigOptionFloats                 *opt_e_retracted { nullptr };
+        ConfigOptionFloats                 *opt_e_restart_extra { nullptr };
+        ConfigOptionFloats                 *opt_extruded_volume { nullptr };
+        ConfigOptionFloats                 *opt_extruded_weight { nullptr };
+        ConfigOptionFloat                  *opt_extruded_volume_total { nullptr };
+        ConfigOptionFloat                  *opt_extruded_weight_total { nullptr };
+        // Caches of the data passed to the script.
+        size_t                              num_extruders;
+        std::vector<double>                 position;
+        std::vector<double>                 e_position;
+        std::vector<double>                 e_retracted;
+        std::vector<double>                 e_restart_extra;
+    } m_placeholder_parser_integration;
+
     OozePrevention                      m_ooze_prevention;
     Wipe                                m_wipe;
     AvoidCrossingPerimeters             m_avoid_crossing_perimeters;
+    JPSPathFinder                       m_avoid_crossing_curled_overhangs;
+    RetractWhenCrossingPerimeters       m_retract_when_crossing_perimeters;
     bool                                m_enable_loop_clipping;
     // If enabled, the G-code generator will put following comments at the ends
     // of the G-code lines: _EXTRUDE_SET_SPEED, _WIPE, _BRIDGE_FAN_START, _BRIDGE_FAN_END
@@ -381,7 +402,7 @@ private:
     // The Pressure Equalizer removes the markers from the final G-code.
     bool                                m_enable_extrusion_role_markers;
     // Keeps track of the last extrusion role passed to the processor
-    ExtrusionRole                       m_last_processor_extrusion_role;
+    GCodeExtrusionRole                  m_last_processor_extrusion_role;
     // How many times will change_layer() be called?
     // change_layer() will update the progress bar.
     unsigned int                        m_layer_count;
@@ -394,7 +415,7 @@ private:
     bool                                m_object_layer_over_raft;
     double                              m_volumetric_speed;
     // Support for the extrusion role markers. Which marker is active?
-    ExtrusionRole                       m_last_extrusion_role;
+    GCodeExtrusionRole                  m_last_extrusion_role;
     // Support for G-Code Processor
     float                               m_last_height{ 0.0f };
     float                               m_last_layer_z{ 0.0f };
@@ -422,32 +443,20 @@ private:
     // Index of a last object copy extruded.
     std::pair<const PrintObject*, Point> m_last_obj_copy;
 
-    bool m_silent_time_estimator_enabled;
+    bool                                m_silent_time_estimator_enabled;
 
     // Processor
-    GCodeProcessor m_processor;
+    GCodeProcessor                      m_processor;
 
-    std::string _extrude(const ExtrusionPath &path, std::string description = "", double speed = -1);
-    void print_machine_envelope(GCodeOutputStream &file, Print &print);
-    void _print_first_layer_bed_temperature(GCodeOutputStream &file, Print &print, const std::string &gcode, unsigned int first_printing_extruder_id, bool wait);
-    void _print_first_layer_extruder_temperatures(GCodeOutputStream &file, Print &print, const std::string &gcode, unsigned int first_printing_extruder_id, bool wait);
+    std::string                         _extrude(const ExtrusionPath &path, const std::string_view description, double speed = -1);
+    void                                print_machine_envelope(GCodeOutputStream &file, Print &print);
+    void                                _print_first_layer_bed_temperature(GCodeOutputStream &file, Print &print, const std::string &gcode, unsigned int first_printing_extruder_id, bool wait);
+    void                                _print_first_layer_extruder_temperatures(GCodeOutputStream &file, Print &print, const std::string &gcode, unsigned int first_printing_extruder_id, bool wait);
     // On the first printing layer. This flag triggers first layer speeds.
     bool                                on_first_layer() const { return m_layer != nullptr && m_layer->id() == 0; }
     // To control print speed of 1st object layer over raft interface.
     bool                                object_layer_over_raft() const { return m_object_layer_over_raft; }
 
-    friend ObjectByExtruder& object_by_extruder(
-        std::map<unsigned int, std::vector<ObjectByExtruder>> &by_extruder, 
-        unsigned int                                           extruder_id, 
-        size_t                                                 object_idx, 
-        size_t                                                 num_objects);
-    friend std::vector<ObjectByExtruder::Island>& object_islands_by_extruder(
-        std::map<unsigned int, std::vector<ObjectByExtruder>>  &by_extruder, 
-        unsigned int                                            extruder_id, 
-        size_t                                                  object_idx, 
-        size_t                                                  num_objects,
-        size_t                                                  num_islands);
-
     friend class Wipe;
     friend class WipeTowerIntegration;
     friend class PressureEqualizer;
