@@ -1,3 +1,7 @@
+///|/ Copyright (c) Prusa Research 2018 - 2023 Lukáš Matěna @lukasmatena, Vojtěch Bubník @bubnikv, Tomáš Mészáros @tamasmeszaros, Pavel Mikuš @Godrak, Roman Beránek @zavorka
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #ifndef slic3r_PrintBase_hpp_
 #define slic3r_PrintBase_hpp_
 
@@ -23,10 +27,29 @@ public:
 
 class PrintStateBase {
 public:
-    enum State {
-        INVALID,
-        STARTED,
-        DONE,
+    enum class State {
+        // Fresh state, either the object is new or the data of that particular milestone was cleaned up.
+        // Fresh state may transit to Started.
+        Fresh,
+        // Milestone was started and now it is being executed.
+        // Started state may transit to Canceled with invalid data or Done with valid data.
+        Started,
+        // Milestone was being executed, but now it is canceled and not yet cleaned up.
+        // Canceled state may transit to Fresh state if its invalid data is cleaned up
+        // or to Started state.
+        // Canceled and Invalidated states are of similar nature: Canceled step was Started but canceled,
+        // while Invalidated state was Done but invalidated.
+        Canceled,
+        // Milestone was finished successfully, it's data is now valid.
+        // Done state may transit to Invalidated state if its data is no more valid
+        // or to a Started state.
+        Done,
+        // Milestone was finished successfully (done), but now it is invalidated and it's data is no more valid.
+        // Invalidated state may transit to Fresh if its invalid data is cleaned up,
+        // or to state Started.
+        // Canceled and Invalidated states are of similar nature: Canceled step was Started but canceled,
+        // while Invalidated state was Done but invalidated.
+        Invalidated,
     };
 
     enum class WarningLevel {
@@ -39,9 +62,25 @@ public:
     // A new unique timestamp is being assigned to the step every time the step changes its state.
     struct StateWithTimeStamp
     {
-        StateWithTimeStamp() : state(INVALID), timestamp(0) {}
-        State       state;
-        TimeStamp   timestamp;
+        State       state { State::Fresh };
+        TimeStamp   timestamp { 0 };
+        bool        enabled { true };
+
+        bool        is_done() const { return state == State::Done; }
+        // The milestone may have some data available, but it is no more valid and it should be cleaned up to conserve memory.
+        bool        is_dirty() const { return state == State::Canceled || state == State::Invalidated; }
+
+        // If the milestone is Started or Done, invalidate it:
+        // Turn Started to Canceled, turn Done to Invalidated.
+        // Update timestamp of this milestone.
+        bool        try_invalidate() {
+            bool invalidated = this->state == State::Started || this->state == State::Done;
+            if (invalidated) {
+                this->state = this->state == State::Started ? State::Canceled : State::Invalidated;
+                this->timestamp = ++ g_last_timestamp;
+            }
+            return invalidated;
+        }
     };
 
     struct Warning
@@ -93,11 +132,11 @@ public:
     }
 
     bool is_started(StepType step, std::mutex &mtx) const {
-        return this->state_with_timestamp(step, mtx).state == STARTED;
+        return this->state_with_timestamp(step, mtx).state == State::Started;
     }
 
     bool is_done(StepType step, std::mutex &mtx) const {
-        return this->state_with_timestamp(step, mtx).state == DONE;
+        return this->state_with_timestamp(step, mtx).state == State::Done;
     }
 
     StateWithTimeStamp state_with_timestamp_unguarded(StepType step) const { 
@@ -105,17 +144,31 @@ public:
     }
 
     bool is_started_unguarded(StepType step) const {
-        return this->state_with_timestamp_unguarded(step).state == STARTED;
+        return this->state_with_timestamp_unguarded(step).state == State::Started;
     }
 
     bool is_done_unguarded(StepType step) const {
-        return this->state_with_timestamp_unguarded(step).state == DONE;
+        return this->state_with_timestamp_unguarded(step).state == State::Done;
+    }
+
+    void enable_unguarded(StepType step, bool enable) {
+        m_state[step].enabled = enable;
+    }
+
+    void enable_all_unguarded(bool enable) {
+        for (size_t istep = 0; istep < COUNT; ++ istep)
+            m_state[istep].enabled = enable;
+    }
+
+    bool is_enabled_unguarded(StepType step) const {
+        return this->state_with_timestamp_unguarded(step).enabled;
     }
 
     // Set the step as started. Block on mutex while the Print / PrintObject / PrintRegion objects are being
     // modified by the UI thread.
     // This is necessary to block until the Print::apply() updates its state, which may
     // influence the processing step being entered.
+    // Returns false if the step is not enabled or if the step has already been finished (it is done).
     template<typename ThrowIfCanceled>
     bool set_started(StepType step, std::mutex &mtx, ThrowIfCanceled throw_if_canceled) {
         std::scoped_lock<std::mutex> lock(mtx);
@@ -132,12 +185,12 @@ public:
 //
 //        assert(m_step_active == -1);
 //        for (int i = 0; i < int(COUNT); ++ i)
-//            assert(m_state[i].state != STARTED);
+//            assert(m_state[i].state != State::Started);
 #endif // NDEBUG
-        if (m_state[step].state == DONE)
-            return false;
         PrintStateBase::StateWithWarnings &state = m_state[step];
-        state.state = STARTED;
+        if (! state.enabled || state.state == State::Done)
+            return false;
+        state.state = State::Started;
         state.timestamp = ++ g_last_timestamp;
         state.mark_warnings_non_current();
         m_step_active = static_cast<int>(step);
@@ -147,17 +200,17 @@ public:
     // Set the step as done. Block on mutex while the Print / PrintObject / PrintRegion objects are being
     // modified by the UI thread.
     // Return value:
-    // 		Timestamp when this stepentered the DONE state.
+    // 		Timestamp when this step entered the Done state.
     // 		bool indicates whether the UI has to update the slicing warnings of this step or not.
 	template<typename ThrowIfCanceled>
 	std::pair<TimeStamp, bool> set_done(StepType step, std::mutex &mtx, ThrowIfCanceled throw_if_canceled) {
         std::scoped_lock<std::mutex> lock(mtx);
         // If canceled, throw before changing the step state.
         throw_if_canceled();
-        assert(m_state[step].state == STARTED);
+        assert(m_state[step].state == State::Started);
         assert(m_step_active == static_cast<int>(step));
         PrintStateBase::StateWithWarnings &state = m_state[step];
-        state.state = DONE;
+        state.state = State::Done;
         state.timestamp = ++ g_last_timestamp;
         m_step_active = -1;
         // Remove all non-current warnings.
@@ -176,16 +229,12 @@ public:
     // processing by calling the cancel callback.
     template<typename CancelationCallback>
     bool invalidate(StepType step, CancelationCallback cancel) {
-        bool invalidated = m_state[step].state != INVALID;
-        if (invalidated) {
+        if (PrintStateBase::StateWithWarnings &state = m_state[step]; state.try_invalidate()) {
 #if 0
             if (mtx.state != mtx.HELD) {
                 printf("Not held!\n");
             }
 #endif
-            PrintStateBase::StateWithWarnings &state = m_state[step];
-            state.state = INVALID;
-            state.timestamp = ++ g_last_timestamp;
             // Raise the mutex, so that the following cancel() callback could cancel
             // the background processing.
             // Internally the cancel() callback shall unlock the PrintBase::m_status_mutex to let
@@ -195,21 +244,17 @@ public:
             // It is safe to modify it.
             state.mark_warnings_non_current();
             m_step_active = -1;
-        }
-        return invalidated;
+            return true;
+        } else
+            return false;
     }
 
     template<typename CancelationCallback, typename StepTypeIterator>
     bool invalidate_multiple(StepTypeIterator step_begin, StepTypeIterator step_end, CancelationCallback cancel) {
         bool invalidated = false;
-        for (StepTypeIterator it = step_begin; it != step_end; ++ it) {
-            StateWithTimeStamp &state = m_state[*it];
-            if (state.state != INVALID) {
+        for (StepTypeIterator it = step_begin; it != step_end; ++ it)
+            if (m_state[*it].try_invalidate())
                 invalidated = true;
-                state.state = INVALID;
-                state.timestamp = ++ g_last_timestamp;
-            }
-        }
         if (invalidated) {
 #if 0
             if (mtx.state != mtx.HELD) {
@@ -237,14 +282,9 @@ public:
     template<typename CancelationCallback>
     bool invalidate_all(CancelationCallback cancel) {
         bool invalidated = false;
-        for (size_t i = 0; i < COUNT; ++ i) {
-            StateWithTimeStamp &state = m_state[i];
-            if (state.state != INVALID) {
+        for (size_t i = 0; i < COUNT; ++ i)
+            if (m_state[i].try_invalidate())
                 invalidated = true;
-                state.state = INVALID;
-                state.timestamp = ++ g_last_timestamp;
-            }
-        }
         if (invalidated) {
             cancel();
             // Now the worker thread should be stopped, therefore it cannot write into the warnings field.
@@ -256,6 +296,26 @@ public:
         return invalidated;
     }
 
+    // If the milestone is Canceled or Invalidated, return true and turn the state of the milestone to Fresh.
+    // The caller is responsible for releasing the data of the milestone that is no more valid.
+    bool query_reset_dirty_unguarded(StepType step) {
+        if (PrintStateBase::StateWithWarnings &state = m_state[step]; state.is_dirty()) {
+            state.state = State::Fresh;
+            return true;
+        } else
+            return false;
+    }
+
+    // To be called after the background thread was stopped by the user pressing the Cancel button,
+    // which in turn stops the background thread without adjusting state of the milestone being executed.
+    // This method fixes the state of the canceled milestone by setting it to a Canceled state.
+    void mark_canceled_unguarded() {
+        for (size_t i = 0; i < COUNT; ++ i) {
+            if (State &state = m_state[i].state; state == State::Started)
+                state = State::Canceled;
+        }
+    }
+
     // Update list of warnings of the current milestone with a new warning.
     // The warning may already exist in the list, marked as current or not current.
     // If it already exists, mark it as current.
@@ -267,7 +327,7 @@ public:
         std::scoped_lock<std::mutex> lock(mtx);
         assert(m_step_active != -1);
         StateWithWarnings &state = m_state[m_step_active];
-        assert(state.state == STARTED);
+        assert(state.state == State::Started);
         std::pair<StepType, bool> retval(static_cast<StepType>(m_step_active), true);
         // Does a warning of the same level and message or message_id exist already?
 		auto it = (message_id == 0) ? 
@@ -326,7 +386,7 @@ class PrintTryCancel
 {
 public:
     // calls print.throw_if_canceled().
-    void operator()();
+    void operator()() const;
 private:
     friend PrintBase;
     PrintTryCancel() = delete;
@@ -362,7 +422,7 @@ public:
     virtual std::vector<ObjectID> print_object_ids() const = 0;
 
     // Validate the print, return empty string if valid, return error if process() cannot (or should not) be started.
-    virtual std::string     validate(std::string* warning = nullptr) const { return std::string(); }
+    virtual std::string     validate(std::vector<std::string>* warnings = nullptr) const { return std::string(); }
 
     enum ApplyStatus {
         // No change after the Print::apply() call.
@@ -388,12 +448,16 @@ public:
         int                     to_print_step;
     };
     // After calling the apply() function, call set_task() to limit the task to be processed by process().
-    virtual void            set_task(const TaskParams &params) {}
+    virtual void            set_task(const TaskParams &params) = 0;
     // Perform the calculation. This is the only method that is to be called at a worker thread.
     virtual void            process() = 0;
     // Clean up after process() finished, either with success, error or if canceled.
     // The adjustments on the Print / PrintObject data due to set_task() are to be reverted here.
-    virtual void            finalize() {}
+    virtual void            finalize() = 0;
+    // Clean up print step / print object step data after
+    // 1) some print step / print object step was invalidated inside PrintBase::apply() while holding the milestone mutex locked.
+    // 2) background thread finished being canceled.
+    virtual void            cleanup() = 0;
 
     struct SlicingStatus {
 		SlicingStatus(int percent, const std::string &text, unsigned int flags = 0) : percent(percent), text(text), flags(flags) {}
@@ -511,10 +575,15 @@ private:
     friend PrintTryCancel;
 };
 
-template<typename PrintStepEnum, const size_t COUNT>
+template<typename PrintStepEnumType, const size_t COUNT>
 class PrintBaseWithState : public PrintBase
 {
 public:
+    using                           PrintStepEnum       = PrintStepEnumType;
+    static constexpr const size_t   PrintStepEnumSize   = COUNT;
+
+    PrintBaseWithState() = default;
+
     bool            is_step_done(PrintStepEnum step) const { return m_state.is_done(step, this->state_mutex()); }
 	PrintStateBase::StateWithTimeStamp step_state_with_timestamp(PrintStepEnum step) const { return m_state.state_with_timestamp(step, this->state_mutex()); }
     PrintStateBase::StateWithWarnings  step_state_with_warnings(PrintStepEnum step) const { return m_state.state_with_warnings(step, this->state_mutex()); }
@@ -549,14 +618,124 @@ protected:
             this->status_update_warnings(static_cast<int>(active_step.first), warning_level, message);
     }
 
+
+    // After calling the apply() function, set_task() may be called to limit the task to be processed by process().
+    template<typename PrintObject>
+    void set_task_impl(const TaskParams &params, std::vector<PrintObject*> &print_objects)
+    {
+        static constexpr const auto PrintObjectStepEnumSize = int(PrintObject::PrintObjectStepEnumSize);
+        using                       PrintObjectStepEnum     = typename PrintObject::PrintObjectStepEnum;
+        // Grab the lock for the Print / PrintObject milestones.
+        std::scoped_lock<std::mutex> lock(this->state_mutex());
+
+        int n_object_steps = int(params.to_object_step) + 1;
+        if (n_object_steps == 0)
+            n_object_steps = PrintObjectStepEnumSize;
+
+        if (params.single_model_object.valid()) {
+            // Find the print object to be processed with priority.
+            PrintObject *print_object = nullptr;
+            size_t       idx_print_object = 0;
+            for (; idx_print_object < print_objects.size(); ++ idx_print_object)
+                if (print_objects[idx_print_object]->model_object()->id() == params.single_model_object) {
+                    print_object = print_objects[idx_print_object];
+                    break;
+                }
+            assert(print_object != nullptr);
+            // Find out whether the priority print object is being currently processed.
+            bool running = false;
+            for (int istep = 0; istep < n_object_steps; ++ istep) {
+                if (! print_object->is_step_enabled_unguarded(PrintObjectStepEnum(istep)))
+                    // Step was skipped, cancel.
+                    break;
+                if (print_object->is_step_started_unguarded(PrintObjectStepEnum(istep))) {
+                    // No step was skipped, and a wanted step is being processed. Don't cancel.
+                    running = true;
+                    break;
+                }
+            }
+            if (! running)
+                this->call_cancel_callback();
+
+            // Now the background process is either stopped, or it is inside one of the print object steps to be calculated anyway.
+            if (params.single_model_instance_only) {
+                // Suppress all the steps of other instances.
+                for (PrintObject *po : print_objects)
+                    for (size_t istep = 0; istep < PrintObjectStepEnumSize; ++ istep)
+                        po->enable_step_unguarded(PrintObjectStepEnum(istep), false);
+            } else if (! running) {
+                // Swap the print objects, so that the selected print_object is first in the row.
+                // At this point the background processing must be stopped, so it is safe to shuffle print objects.
+                if (idx_print_object != 0)
+                    std::swap(print_objects.front(), print_objects[idx_print_object]);
+            }
+            // and set the steps for the current object.
+            for (int istep = 0; istep < n_object_steps; ++ istep)
+                print_object->enable_step_unguarded(PrintObjectStepEnum(istep), true);
+            for (int istep = n_object_steps; istep < PrintObjectStepEnumSize; ++ istep)
+                print_object->enable_step_unguarded(PrintObjectStepEnum(istep), false);
+        } else {
+            // Slicing all objects.
+            bool running = false;
+            for (PrintObject *print_object : print_objects)
+                for (int istep = 0; istep < n_object_steps; ++ istep) {
+                    if (! print_object->is_step_enabled_unguarded(PrintObjectStepEnum(istep))) {
+                        // Step may have been skipped. Restart.
+                        goto loop_end;
+                    }
+                    if (print_object->is_step_started_unguarded(PrintObjectStepEnum(istep))) {
+                        // This step is running, and the state cannot be changed due to the this->state_mutex() being locked.
+                        // It is safe to manipulate m_stepmask of other PrintObjects and Print now.
+                        running = true;
+                        goto loop_end;
+                    }
+                }
+        loop_end:
+            if (! running)
+                this->call_cancel_callback();
+            for (PrintObject *po : print_objects) {
+                for (int istep = 0; istep < n_object_steps; ++ istep)
+                    po->enable_step_unguarded(PrintObjectStepEnum(istep), true);
+                for (int istep = n_object_steps; istep < PrintObjectStepEnumSize; ++ istep)
+                    po->enable_step_unguarded(PrintObjectStepEnum(istep), false);
+            }
+        }
+
+        if (params.to_object_step != -1 || params.to_print_step != -1) {
+            // Limit the print steps.
+            size_t istep = (params.to_object_step != -1) ? 0 : size_t(params.to_print_step) + 1;
+            for (; istep < PrintStepEnumSize; ++ istep)
+                m_state.enable_unguarded(PrintStepEnum(istep), false);
+        }
+    }
+
+    // Clean up after process() finished, either with success, error or if canceled.
+    // The adjustments on the Print / PrintObject m_stepmask data due to set_task() are to be reverted here:
+    // Execution of all milestones is enabled in case some of them were suppressed for the last background execution.
+    // Also if the background processing was canceled, the current milestone that was just abandoned 
+    // in Started state is to be reset to Canceled state.
+    template<typename PrintObject>
+    void finalize_impl(std::vector<PrintObject*> &print_objects)
+    {
+        // Grab the lock for the Print / PrintObject milestones.
+        std::scoped_lock<std::mutex> lock(this->state_mutex());
+        for (auto *po : print_objects)
+            po->finalize_impl();
+        m_state.enable_all_unguarded(true);
+        m_state.mark_canceled_unguarded();
+    }
+
 private:
-    PrintState<PrintStepEnum, COUNT> m_state;
+    PrintState<PrintStepEnum, COUNT>    m_state;
 };
 
-template<typename PrintType, typename PrintObjectStepEnum, const size_t COUNT>
+template<typename PrintType, typename PrintObjectStepEnumType, const size_t COUNT>
 class PrintObjectBaseWithState : public PrintObjectBase
 {
 public:
+    using                           PrintObjectStepEnum       = PrintObjectStepEnumType;
+    static constexpr const size_t   PrintObjectStepEnumSize   = COUNT;
+
     PrintType*       print()         { return m_print; }
     const PrintType* print() const   { return m_print; }
 
@@ -565,6 +744,21 @@ public:
     PrintStateBase::StateWithTimeStamp step_state_with_timestamp(PrintObjectStepEnum step) const { return m_state.state_with_timestamp(step, PrintObjectBase::state_mutex(m_print)); }
     PrintStateBase::StateWithWarnings  step_state_with_warnings(PrintObjectStepEnum step) const { return m_state.state_with_warnings(step, PrintObjectBase::state_mutex(m_print)); }
 
+    auto last_completed_step() const
+    {
+        static_assert(COUNT > 0, "Step count should be > 0");
+        auto s = int(COUNT) - 1;
+
+        std::lock_guard lk(state_mutex(m_print));
+        while (s >= 0 && ! is_step_done_unguarded(PrintObjectStepEnum(s)))
+            --s;
+
+        if (s < 0)
+            s = COUNT;
+
+        return PrintObjectStepEnum(s);
+    }
+
 protected:
 	PrintObjectBaseWithState(PrintType *print, ModelObject *model_object) : PrintObjectBase(model_object), m_print(print) {}
 
@@ -590,6 +784,15 @@ protected:
     bool            is_step_started_unguarded(PrintObjectStepEnum step) const { return m_state.is_started_unguarded(step); }
     bool            is_step_done_unguarded(PrintObjectStepEnum step) const { return m_state.is_done_unguarded(step); }
 
+    bool            is_step_enabled_unguarded(PrintObjectStepEnum step) const { return m_state.is_enabled_unguarded(step); }
+    void            enable_step_unguarded(PrintObjectStepEnum step, bool enable) { m_state.enable_unguarded(step, enable); }
+    void            enable_all_steps_unguarded(bool enable) { m_state.enable_all_unguarded(enable); }
+    // See the comment at PrintBaseWithState::finalize_impl()
+    void            finalize_impl() { m_state.enable_all_unguarded(true); m_state.mark_canceled_unguarded(); }
+    // If the milestone is Canceled or Invalidated, return true and turn the state of the milestone to Fresh.
+    // The caller is responsible for releasing the data of the milestone that is no more valid.
+    bool            query_reset_dirty_step_unguarded(PrintObjectStepEnum step) { return m_state.query_reset_dirty_unguarded(step); }
+
     // Add a slicing warning to the active PrintObject step and send a status notification.
     // This method could be called multiple times between this->set_started() and this->set_done().
     void            active_step_add_warning(PrintStateBase::WarningLevel warning_level, const std::string &message, int message_id = 0) {
@@ -604,10 +807,10 @@ protected:
     void            throw_if_canceled() { if (m_print->canceled()) throw CanceledException(); }
 
     friend PrintType;
-    PrintType                               *m_print;
+    PrintType                                *m_print;
 
 private:
-    PrintState<PrintObjectStepEnum, COUNT>   m_state;
+    PrintState<PrintObjectStepEnum, COUNT>    m_state;
 };
 
 } // namespace Slic3r
