@@ -1,3 +1,7 @@
+///|/ Copyright (c) Prusa Research 2017 - 2023 Oleksandra Iushchenko @YuSanka, Lukáš Matěna @lukasmatena, Vojtěch Bubník @bubnikv, Vojtěch Král @vojtechkral, Enrico Turri @enricoturri1966
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #ifndef SLIC3R_GUI_FIELD_HPP
 #define SLIC3R_GUI_FIELD_HPP
 
@@ -39,7 +43,86 @@ using t_back_to_init = std::function<void(const std::string&)>;
 wxString double_to_string(double const value, const int max_precision = 4);
 wxString get_thumbnails_string(const std::vector<Vec2d>& values);
 
-class Field {
+class UndoValueUIManager
+{
+	struct UndoValueUI {
+		// Bitmap and Tooltip text for m_Undo_btn. The wxButton will be updated only if the new wxBitmap pointer differs from the currently rendered one.
+		const ScalableBitmap* undo_bitmap{ nullptr };
+		const wxString* undo_tooltip{ nullptr };
+		// Bitmap and Tooltip text for m_Undo_to_sys_btn. The wxButton will be updated only if the new wxBitmap pointer differs from the currently rendered one.
+		const ScalableBitmap* undo_to_sys_bitmap{ nullptr };
+		const wxString* undo_to_sys_tooltip{ nullptr };
+		// Color for Label. The wxColour will be updated only if the new wxColour pointer differs from the currently rendered one.
+		const wxColour* label_color{ nullptr };
+		// State of the blinker icon
+		bool					blink{ false };
+
+		bool 	set_undo_bitmap(const ScalableBitmap* bmp) {
+			if (undo_bitmap != bmp) {
+				undo_bitmap = bmp;
+				return true;
+			}
+			return false;
+		}
+
+		bool 	set_undo_to_sys_bitmap(const ScalableBitmap* bmp) {
+			if (undo_to_sys_bitmap != bmp) {
+				undo_to_sys_bitmap = bmp;
+				return true;
+			}
+			return false;
+		}
+
+		bool	set_label_colour(const wxColour* clr) {
+			if (label_color != clr) {
+				label_color = clr;
+			}
+			return false;
+		}
+
+		bool 	set_undo_tooltip(const wxString* tip) {
+			if (undo_tooltip != tip) {
+				undo_tooltip = tip;
+				return true;
+			}
+			return false;
+		}
+
+		bool 	set_undo_to_sys_tooltip(const wxString* tip) {
+			if (undo_to_sys_tooltip != tip) {
+				undo_to_sys_tooltip = tip;
+				return true;
+			}
+			return false;
+		}
+	};
+
+	UndoValueUI m_undo_ui;
+
+public:
+	UndoValueUIManager() {}
+	~UndoValueUIManager() {}
+
+	bool 	set_undo_bitmap(const ScalableBitmap* bmp)			{ return m_undo_ui.set_undo_bitmap(bmp); }
+	bool 	set_undo_to_sys_bitmap(const ScalableBitmap* bmp)	{ return m_undo_ui.set_undo_to_sys_bitmap(bmp); }
+	bool	set_label_colour(const wxColour* clr)				{ return m_undo_ui.set_label_colour(clr); }
+	bool 	set_undo_tooltip(const wxString* tip)				{ return m_undo_ui.set_undo_tooltip(tip); }
+	bool 	set_undo_to_sys_tooltip(const wxString* tip)		{ return m_undo_ui.set_undo_to_sys_tooltip(tip); }	
+
+	// ui items used for revert line value
+	bool					has_undo_ui()			const { return m_undo_ui.undo_bitmap != nullptr; }
+	const wxBitmapBundle&	undo_bitmap()			const { return m_undo_ui.undo_bitmap->bmp(); }
+	const wxString*			undo_tooltip()			const { return m_undo_ui.undo_tooltip; }
+	const wxBitmapBundle&	undo_to_sys_bitmap()	const { return m_undo_ui.undo_to_sys_bitmap->bmp(); }
+	const wxString*			undo_to_sys_tooltip()	const { return m_undo_ui.undo_to_sys_tooltip; }
+	const wxColour*			label_color()			const { return m_undo_ui.label_color; }
+	const bool				blink()					const { return m_undo_ui.blink; }
+	bool*					get_blink_ptr()				  { return &m_undo_ui.blink; }
+};
+
+
+class Field : public UndoValueUIManager
+{
 protected:
     // factory function to defer and enforce creation of derived type. 
 	virtual void	PostInitialize();
@@ -137,49 +220,6 @@ public:
 		return std::move(p); //!p;
     }
 
-    bool 	set_undo_bitmap(const ScalableBitmap *bmp) {
-    	if (m_undo_bitmap != bmp) {
-    		m_undo_bitmap = bmp;
-    		return true;
-    	}
-    	return false;
-    }
-
-    bool 	set_undo_to_sys_bitmap(const ScalableBitmap *bmp) {
-    	if (m_undo_to_sys_bitmap != bmp) {
-    		m_undo_to_sys_bitmap = bmp;
-    		return true;
-    	}
-    	return false;
-    }
-
-	bool	set_label_colour(const wxColour *clr) {
-		if (m_label_color != clr) {
-			m_label_color = clr;
-		}
-		return false;
-	}
-
-	bool 	set_undo_tooltip(const wxString *tip) {
-		if (m_undo_tooltip != tip) {
-			m_undo_tooltip = tip;
-			return true;
-		}
-		return false;
-	}
-
-	bool 	set_undo_to_sys_tooltip(const wxString *tip) {
-		if (m_undo_to_sys_tooltip != tip) {
-			m_undo_to_sys_tooltip = tip;
-			return true;
-		}
-		return false;
-	}
-
-	bool*	get_blink_ptr() {
-		return &m_blink;
-    }
-
     virtual void msw_rescale();
     virtual void sys_color_changed();
 
@@ -191,26 +231,7 @@ public:
 	static int def_width_wider()	;
 	static int def_width_thinner()	;
 
-	const ScalableBitmap*	undo_bitmap()			{ return m_undo_bitmap; }
-	const wxString*			undo_tooltip()			{ return m_undo_tooltip; }
-	const ScalableBitmap*	undo_to_sys_bitmap()	{ return m_undo_to_sys_bitmap; }
-	const wxString*			undo_to_sys_tooltip()	{ return m_undo_to_sys_tooltip; }
-	const wxColour*			label_color()			{ return m_label_color; }
-	const bool				blink()					{ return m_blink; }
-
 protected:
-	// Bitmap and Tooltip text for m_Undo_btn. The wxButton will be updated only if the new wxBitmap pointer differs from the currently rendered one.
-	const ScalableBitmap*   m_undo_bitmap = nullptr;
-	const wxString*         m_undo_tooltip = nullptr;
-	// Bitmap and Tooltip text for m_Undo_to_sys_btn. The wxButton will be updated only if the new wxBitmap pointer differs from the currently rendered one.
-    const ScalableBitmap*   m_undo_to_sys_bitmap = nullptr;
-	const wxString*		    m_undo_to_sys_tooltip = nullptr;
-
-	bool					m_blink{ false };
-
-	// Color for Label. The wxColour will be updated only if the new wxColour pointer differs from the currently rendered one.
-	const wxColour*		m_label_color = nullptr;
-
 	// current value
 	boost::any			m_value;
     // last maeningful value
@@ -313,24 +334,18 @@ public:
 	void			BUILD() override;
     /// Propagate value from field to the OptionGroupe and Config after kill_focus/ENTER
     void	        propagate_value() ;
-
+/*
     void			set_value(const std::string& value, bool change_event = false) {
 		m_disable_change_event = !change_event;
 		dynamic_cast<wxSpinCtrl*>(window)->SetValue(value);
 		m_disable_change_event = false;
     }
-    void			set_value(const boost::any& value, bool change_event = false) override {
-		m_disable_change_event = !change_event;
-		tmp_value = boost::any_cast<int>(value);
-        m_value = value;
-		dynamic_cast<wxSpinCtrl*>(window)->SetValue(tmp_value);
-		m_disable_change_event = false;
-	}
+*/
+    void            set_value(const boost::any& value, bool change_event = false) override;
+    void            set_last_meaningful_value() override;
+    void            set_na_value() override;
 
-	boost::any&		get_value() override {
-		int value = static_cast<wxSpinCtrl*>(window)->GetValue();
-		return m_value = value;
-	}
+	boost::any&		get_value() override;
 
     void            msw_rescale() override;
 
@@ -408,7 +423,7 @@ class PointCtrl : public Field {
 public:
 	PointCtrl(const ConfigOptionDef& opt, const t_config_option_key& id) : Field(opt, id) {}
 	PointCtrl(wxWindow* parent, const ConfigOptionDef& opt, const t_config_option_key& id) : Field(parent, opt, id) {}
-	~PointCtrl() {}
+	~PointCtrl();
 
 	wxSizer*		sizer{ nullptr };
 	wxTextCtrl*		x_textctrl{ nullptr };
