@@ -1,3 +1,7 @@
+///|/ Copyright (c) Prusa Research 2018 - 2023 Vojtěch Bubník @bubnikv, Lukáš Hejl @hejllukas, Oleksandra Iushchenko @YuSanka, Enrico Turri @enricoturri1966, Lukáš Matěna @lukasmatena, Vojtěch Král @vojtechkral
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #include <numeric>
 #include <algorithm>
 #include <thread>
@@ -654,7 +658,7 @@ void FirmwareDialog::priv::perform_upload()
 			}
 		})
 		.on_message([
-#ifndef __APPLE__
+#if !defined(__APPLE__) && !defined(__clang__)
 	        // clang complains when capturing constants.
 			extra_verbose,
 #endif // __APPLE__
@@ -771,7 +775,7 @@ void FirmwareDialog::priv::ensure_joined()
 const char* FirmwareDialog::priv::avr109_dev_name(Avr109Pid usb_pid) {
 	switch (usb_pid.boot) {
 		case USB_PID_MMU_BOOT:
-			return "Original Prusa MMU 2.0 Control";
+			return "Original Prusa Multi Material 2 & 3 Upgrade (bootloader)";
 		case USB_PID_CW1_BOOT:
 			return "Original Prusa CW1";
 		case USB_PID_CW1S_BOOT:
