@@ -1,3 +1,9 @@
+///|/ Copyright (c) Prusa Research 2016 - 2023 Tomáš Mészáros @tamasmeszaros, Oleksandra Iushchenko @YuSanka, Vojtěch Bubník @bubnikv, Lukáš Matěna @lukasmatena, Lukáš Hejl @hejllukas, David Kocík @kocikdav, Enrico Turri @enricoturri1966, Vojtěch Král @vojtechkral
+///|/ Copyright (c) 2018 Martin Loidl @LoidlM
+///|/ Copyright (c) Slic3r 2015 Alessandro Ranellucci @alranel
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #include "GUI.hpp"
 #include "GUI_App.hpp"
 #include "format.hpp"
@@ -10,12 +16,15 @@
 #include <boost/algorithm/string.hpp>
 #include <boost/algorithm/string/predicate.hpp>
 #include <boost/any.hpp>
+#include <boost/filesystem.hpp>
 
 #if __APPLE__
 #import <IOKit/pwr_mgt/IOPMLib.h>
 #elif _WIN32
 #define WIN32_LEAN_AND_MEAN
-#define NOMINMAX
+#ifndef NOMINMAX
+    #define NOMINMAX
+#endif
 #include <Windows.h>
 #include "boost/nowide/convert.hpp"
 #endif
@@ -129,6 +138,18 @@ void change_opt_value(DynamicPrintConfig& config, const t_config_option_key& opt
 			val = boost::any_cast<double>(value);
 			break;
 		}
+		case coFloatsOrPercents:{
+			std::string str = boost::any_cast<std::string>(value);
+			bool percent = false;
+			if (str.back() == '%') {
+				str.pop_back();
+				percent = true;
+			}
+            double val = std::stod(str); // locale-dependent (on purpose - the input is the actual content of the field)
+			ConfigOptionFloatsOrPercents* vec_new = new ConfigOptionFloatsOrPercents({ {val, percent} });
+			config.option<ConfigOptionFloatsOrPercents>(opt_key)->set_at(vec_new, opt_index, opt_index);
+			break;
+		}
 		case coPercents:{
 			ConfigOptionPercents* vec_new = new ConfigOptionPercents{ boost::any_cast<double>(value) };
 			config.option<ConfigOptionPercents>(opt_key)->set_at(vec_new, opt_index, opt_index);
@@ -267,33 +288,11 @@ static void add_config_substitutions(const ConfigSubstitutions& conf_substitutio
 		switch (def->type) {
 		case coEnum:
 		{
-			const std::vector<std::string>& labels = def->enum_labels;
-			const std::vector<std::string>& values = def->enum_values;
-			int val = conf_substitution.new_value->getInt();
-
-			bool is_infill = def->opt_key == "top_fill_pattern"	   ||
-							 def->opt_key == "bottom_fill_pattern" ||
-							 def->opt_key == "fill_pattern";
-
-			// Each infill doesn't use all list of infill declared in PrintConfig.hpp.
-			// So we should "convert" val to the correct one
-			if (is_infill) {
-				for (const auto& key_val : *def->enum_keys_map)
-					if ((int)key_val.second == val) {
-						auto it = std::find(values.begin(), values.end(), key_val.first);
-						if (it == values.end())
-							break;
-						auto idx = it - values.begin();
-						new_val = wxString("\"") + values[idx] + "\"" + " (" + from_u8(_utf8(labels[idx])) + ")";
-						break;
-					}
-				if (new_val.IsEmpty()) {
-					assert(false);
-					new_val = _L("Undefined");
-				}
-			}
-			else
-				new_val = wxString("\"") + values[val] + "\"" + " (" + from_u8(_utf8(labels[val])) + ")";
+			auto opt = def->enum_def->enum_to_index(conf_substitution.new_value->getInt());
+			new_val = opt.has_value() ?
+				wxString("\"") + def->enum_def->value(*opt) + "\"" + " (" +
+					_(from_u8(def->enum_def->label(*opt))) + ")" :
+				_L("Undefined");
 			break;
 		}
 		case coBool:
@@ -476,50 +475,132 @@ void about()
 
 void desktop_open_datadir_folder()
 {
+	boost::filesystem::path path(data_dir());
+	desktop_open_folder(std::move(path));
+}
+
+void desktop_open_folder(const boost::filesystem::path& path)
+{
+	if (!boost::filesystem::is_directory(path)) 
+		return;
+
 	// Execute command to open a file explorer, platform dependent.
-	// FIXME: The const_casts aren't needed in wxWidgets 3.1, remove them when we upgrade.
-
-	const auto path = data_dir();
 #ifdef _WIN32
-		const wxString widepath = from_u8(path);
-		const wchar_t *argv[] = { L"explorer", widepath.GetData(), nullptr };
-		::wxExecute(const_cast<wchar_t**>(argv), wxEXEC_ASYNC, nullptr);
+	const wxString widepath = path.wstring();
+	const wchar_t* argv[] = { L"explorer", widepath.GetData(), nullptr };
+	::wxExecute(const_cast<wchar_t**>(argv), wxEXEC_ASYNC, nullptr);
 #elif __APPLE__
-		const char *argv[] = { "open", path.data(), nullptr };
-		::wxExecute(const_cast<char**>(argv), wxEXEC_ASYNC, nullptr);
+	const char* argv[] = { "open", path.string().c_str(), nullptr };
+	::wxExecute(const_cast<char**>(argv), wxEXEC_ASYNC, nullptr);
 #else
-		const char *argv[] = { "xdg-open", path.data(), nullptr };
-
-		// Check if we're running in an AppImage container, if so, we need to remove AppImage's env vars,
-		// because they may mess up the environment expected by the file manager.
-		// Mostly this is about LD_LIBRARY_PATH, but we remove a few more too for good measure.
-		if (wxGetEnv("APPIMAGE", nullptr)) {
-			// We're running from AppImage
-			wxEnvVariableHashMap env_vars;
-			wxGetEnvMap(&env_vars);
-
-			env_vars.erase("APPIMAGE");
-			env_vars.erase("APPDIR");
-			env_vars.erase("LD_LIBRARY_PATH");
-			env_vars.erase("LD_PRELOAD");
-			env_vars.erase("UNION_PRELOAD");
-
-			wxExecuteEnv exec_env;
-			exec_env.env = std::move(env_vars);
-
-			wxString owd;
-			if (wxGetEnv("OWD", &owd)) {
-				// This is the original work directory from which the AppImage image was run,
-				// set it as CWD for the child process:
-				exec_env.cwd = std::move(owd);
-			}
-
-			::wxExecute(const_cast<char**>(argv), wxEXEC_ASYNC, nullptr, &exec_env);
-		} else {
-			// Looks like we're NOT running from AppImage, we'll make no changes to the environment.
-			::wxExecute(const_cast<char**>(argv), wxEXEC_ASYNC, nullptr, nullptr);
-		}
+	const char* argv[] = { "xdg-open", path.string().c_str(), nullptr };
+	desktop_execute(argv);
 #endif
 }
 
-} }
+#ifdef __linux__
+namespace {
+wxExecuteEnv get_appimage_exec_env()
+{
+	// If we're running in an AppImage container, we need to remove AppImage's env vars,
+	// because they may mess up the environment expected by the file manager.
+	// Mostly this is about LD_LIBRARY_PATH, but we remove a few more too for good measure.
+	wxEnvVariableHashMap env_vars;
+	wxGetEnvMap(&env_vars);
+
+	env_vars.erase("APPIMAGE");
+	env_vars.erase("APPDIR");
+	env_vars.erase("LD_LIBRARY_PATH");
+	env_vars.erase("LD_PRELOAD");
+	env_vars.erase("UNION_PRELOAD");
+
+	wxExecuteEnv exec_env;
+	exec_env.env = std::move(env_vars);
+
+	wxString owd;
+	if (wxGetEnv("OWD", &owd)) {
+		// This is the original work directory from which the AppImage image was run,
+		// set it as CWD for the child process:
+		exec_env.cwd = std::move(owd);
+	}
+	return exec_env;
+}
+} // namespace
+void desktop_execute(const char* argv[])
+{
+	// Check if we're running in an AppImage container, if so, we need to remove AppImage's env vars,
+	// because they may mess up the environment expected by the file manager.
+	// Mostly this is about LD_LIBRARY_PATH, but we remove a few more too for good measure.
+	if (wxGetEnv("APPIMAGE", nullptr)) {
+		// We're running from AppImage
+		wxExecuteEnv exec_env = get_appimage_exec_env();
+		::wxExecute(const_cast<char**>(argv), wxEXEC_ASYNC, nullptr, &exec_env);
+	}
+	else {
+		// Looks like we're NOT running from AppImage, we'll make no changes to the environment.
+		::wxExecute(const_cast<char**>(argv), wxEXEC_ASYNC, nullptr, nullptr);
+	}
+}
+void desktop_execute_get_result(wxString command, wxArrayString& output)
+{
+	output.Clear();
+   //Check if we're running in an AppImage container, if so, we need to remove AppImage's env vars,
+   // because they may mess up the environment expected by the file manager.
+   // Mostly this is about LD_LIBRARY_PATH, but we remove a few more too for good measure.
+	if (wxGetEnv("APPIMAGE", nullptr)) {
+		// We're running from AppImage
+		wxExecuteEnv exec_env = get_appimage_exec_env();
+		::wxExecute(command, output, wxEXEC_SYNC | wxEXEC_NOEVENTS, &exec_env);
+	} else {
+		// Looks like we're NOT running from AppImage, we'll make no changes to the environment.
+		::wxExecute(command, output, wxEXEC_SYNC | wxEXEC_NOEVENTS);
+	}
+}
+#endif // __linux__
+
+#ifdef _WIN32
+bool create_process(const boost::filesystem::path& path, const std::wstring& cmd_opt, std::string& error_msg)
+{
+	// find updater exe
+	if (boost::filesystem::exists(path)) {
+		// Using quoted string as mentioned in CreateProcessW docs.
+		std::wstring wcmd = L"\"" + path.wstring() + L"\"";
+		if (!cmd_opt.empty())
+			wcmd += L" " + cmd_opt;
+
+		// additional information
+		STARTUPINFOW si;
+		PROCESS_INFORMATION pi;
+
+		// set the size of the structures
+		ZeroMemory(&si, sizeof(si));
+		si.cb = sizeof(si);
+		ZeroMemory(&pi, sizeof(pi));
+
+		// start the program up
+		if (CreateProcessW(NULL,   // the path
+			wcmd.data(),    // Command line
+			NULL,           // Process handle not inheritable
+			NULL,           // Thread handle not inheritable
+			FALSE,          // Set handle inheritance to FALSE
+			0,              // No creation flags
+			NULL,           // Use parent's environment block
+			NULL,           // Use parent's starting directory 
+			&si,            // Pointer to STARTUPINFO structure
+			&pi             // Pointer to PROCESS_INFORMATION structure (removed extra parentheses)
+		)) {
+			// Close process and thread handles.
+			CloseHandle(pi.hProcess);
+			CloseHandle(pi.hThread);
+			return true;
+		}
+		else
+			error_msg = "CreateProcessW failed to create process " + boost::nowide::narrow(path.wstring());
+	}
+	else
+		error_msg = "Executable doesn't exists. Path: " + boost::nowide::narrow(path.wstring());
+	return false;
+}
+#endif //_WIN32
+
+} } // namespaces GUI / Slic3r
