@@ -1,3 +1,7 @@
+///|/ Copyright (c) Prusa Research 2018 - 2023 Oleksandra Iushchenko @YuSanka, Enrico Turri @enricoturri1966, Lukáš Matěna @lukasmatena, Vojtěch Bubník @bubnikv
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #ifndef slic3r_GUI_ObjectManipulation_hpp_
 #define slic3r_GUI_ObjectManipulation_hpp_
 
@@ -5,6 +9,7 @@
 
 #include "GUI_ObjectSettings.hpp"
 #include "GUI_ObjectList.hpp"
+#include "GUI_Geometry.hpp"
 #include "libslic3r/Point.hpp"
 #include <float.h>
 
@@ -57,6 +62,10 @@ public:
     void                set_value(const wxString& new_value);
     void                kill_focus(ObjectManipulation *parent);
 
+    const std::string&  get_full_opt_name() const { return m_full_opt_name; }
+
+    bool                has_opt_key(const std::string& key) { return m_opt_key == key; }
+
 private:
     double              get_value();
 };
@@ -78,6 +87,7 @@ private:
         Vec3d scale;
         Vec3d scale_rounded;
         Vec3d size;
+        Vec3d size_inches;
         Vec3d size_rounded;
 
         wxString move_label_string;
@@ -113,59 +123,58 @@ private:
     wxStaticText*   m_empty_str = nullptr;
 
     // Non-owning pointers to the reset buttons, so we can hide and show them.
-    ScalableButton* m_reset_scale_button = nullptr;
-    ScalableButton* m_reset_rotation_button = nullptr;
-    ScalableButton* m_drop_to_bed_button = nullptr;
+    ScalableButton* m_reset_scale_button{ nullptr };
+    ScalableButton* m_reset_rotation_button{ nullptr };
+    ScalableButton* m_reset_skew_button{ nullptr };
+    ScalableButton* m_drop_to_bed_button{ nullptr };
 
     wxCheckBox*     m_check_inch {nullptr};
 
-    // Mirroring buttons and their current state
-    enum MirrorButtonState {
-        mbHidden,
-        mbShown,
-        mbActive
-    };
-    std::array<std::pair<ScalableButton*, MirrorButtonState>, 3> m_mirror_buttons;
+    std::array<ScalableButton*, 3> m_mirror_buttons;
 
     // Bitmaps for the mirroring buttons.
     ScalableBitmap m_mirror_bitmap_on;
-    ScalableBitmap m_mirror_bitmap_off;
-    ScalableBitmap m_mirror_bitmap_hidden;
 
     // Needs to be updated from OnIdle?
     bool            m_dirty = false;
     // Cached labels for the delayed update, not localized!
     std::string     m_new_move_label_string;
-	std::string     m_new_rotate_label_string;
-	std::string     m_new_scale_label_string;
+    std::string     m_new_rotate_label_string;
+    std::string     m_new_scale_label_string;
     Vec3d           m_new_position;
     Vec3d           m_new_rotation;
     Vec3d           m_new_scale;
     Vec3d           m_new_size;
     bool            m_new_enabled {true};
     bool            m_uniform_scale {true};
-    // Does the object manipulation panel work in World or Local coordinates?
-    bool            m_world_coordinates = true;
+    ECoordinatesType m_coordinates_type{ ECoordinatesType::World };
     LockButton*     m_lock_bnt{ nullptr };
     choice_ctrl*    m_word_local_combo { nullptr };
 
     ScalableBitmap  m_manifold_warning_bmp;
-    wxStaticBitmap* m_fix_throught_netfab_bitmap;
+    wxStaticBitmap* m_fix_throught_netfab_bitmap{ nullptr };
+    wxStaticBitmap* m_mirror_warning_bitmap{ nullptr };
 
-#ifndef __APPLE__
     // Currently focused editor (nullptr if none)
-    ManipulationEditor* m_focused_editor {nullptr};
-#endif // __APPLE__
+    ManipulationEditor* m_focused_editor{ nullptr };
 
     wxFlexGridSizer* m_main_grid_sizer;
     wxFlexGridSizer* m_labels_grid_sizer;
 
+    wxStaticText* m_skew_label{ nullptr };
+
     // sizers, used for msw_rescale
     wxBoxSizer*     m_word_local_combo_sizer;
     std::vector<wxBoxSizer*>            m_rescalable_sizers;
 
     std::vector<ManipulationEditor*>    m_editors;
 
+    // parameters for enabling/disabling of editors
+    bool m_is_enabled                   { true };
+    bool m_is_enabled_size_and_scale    { true };
+
+    bool m_show_skew                    { false };
+
 public:
     ObjectManipulation(wxWindow* parent);
     ~ObjectManipulation() {}
@@ -173,6 +182,10 @@ public:
     void        Show(const bool show) override;
     bool        IsShown() override;
     void        UpdateAndShow(const bool show) override;
+    void        Enable(const bool enadle = true);
+    void        Disable() { Enable(false); }
+    void        DisableScale();
+    void        DisableUnuniformScale();
     void        update_ui_from_settings();
     bool        use_colors() { return m_use_colors; }
 
@@ -180,11 +193,14 @@ public:
 	// Called from the App to update the UI if dirty.
 	void		update_if_dirty();
 
-    void        set_uniform_scaling(const bool uniform_scale);
+    void        set_uniform_scaling(const bool use_uniform_scale);
     bool        get_uniform_scaling() const { return m_uniform_scale; }
-    // Does the object manipulation panel work in World or Local coordinates?
-    void        set_world_coordinates(const bool world_coordinates) { m_world_coordinates = world_coordinates; this->UpdateAndShow(true); }
-    bool        get_world_coordinates() const { return m_world_coordinates; }
+
+    void             set_coordinates_type(ECoordinatesType type);
+    ECoordinatesType get_coordinates_type() const;
+    bool             is_world_coordinates() const { return m_coordinates_type == ECoordinatesType::World; }
+    bool             is_instance_coordinates() const { return m_coordinates_type == ECoordinatesType::Instance; }
+    bool             is_local_coordinates() const { return m_coordinates_type == ECoordinatesType::Local; }
 
     void reset_cache() { m_cache.reset(); }
 #ifndef __APPLE__
@@ -200,11 +216,20 @@ public:
     void sys_color_changed();
     void on_change(const std::string& opt_key, int axis, double new_value);
     void set_focused_editor(ManipulationEditor* focused_editor) {
-#ifndef __APPLE__
         m_focused_editor = focused_editor;
-#endif // __APPLE__        
     }
 
+    ManipulationEditor* get_focused_editor() { return m_focused_editor; }
+
+    static wxString coordinate_type_str(ECoordinatesType type);
+
+    bool is_enabled()               const { return m_is_enabled; }
+    bool is_enabled_size_and_scale()const { return m_is_enabled_size_and_scale; }
+
+#if ENABLE_OBJECT_MANIPULATION_DEBUG
+    void render_debug_window();
+#endif // ENABLE_OBJECT_MANIPULATION_DEBUG
+
 private:
     void reset_settings_value();
     void update_settings_value(const Selection& selection);
@@ -220,6 +245,9 @@ private:
     void change_scale_value(int axis, double value);
     void change_size_value(int axis, double value);
     void do_scale(int axis, const Vec3d &scale) const;
+    void do_size(int axis, const Vec3d& scale) const;
+
+    void set_coordinates_type(const wxString& type_string);
 };
 
 }}
