@@ -1,3 +1,7 @@
+///|/ Copyright (c) Prusa Research 2021 - 2023 Oleksandra Iushchenko @YuSanka, Enrico Turri @enricoturri1966, Filip Sykala @Jony01, Vojtěch Bubník @bubnikv, Lukáš Matěna @lukasmatena
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #include "GalleryDialog.hpp"
 
 #include <cstddef>
@@ -65,7 +69,7 @@ bool GalleryDropTarget::OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& f
 }
 
 
-GalleryDialog::GalleryDialog(wxWindow* parent, bool modify_gallery/* = false*/) :
+GalleryDialog::GalleryDialog(wxWindow* parent) :
     DPIDialog(parent, wxID_ANY, _L("Shape Gallery"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER)
 {
 #ifndef _WIN32
@@ -94,12 +98,9 @@ GalleryDialog::GalleryDialog(wxWindow* parent, bool modify_gallery/* = false*/)
 #endif
 
     wxStdDialogButtonSizer* buttons = this->CreateStdDialogButtonSizer(wxOK | wxCLOSE);
-    wxButton* ok_btn = static_cast<wxButton*>(FindWindowById(wxID_OK, this));
-    ok_btn->Bind(wxEVT_UPDATE_UI, [this](wxUpdateUIEvent& evt) { evt.Enable(!m_selected_items.empty()); });
-    if (modify_gallery) {
-        ok_btn->SetLabel(_L("Add to bed"));
-        ok_btn->SetToolTip(_L("Add selected shape(s) to the bed"));
-    }
+    m_ok_btn = static_cast<wxButton*>(FindWindowById(wxID_OK, this));
+    m_ok_btn->Bind(wxEVT_UPDATE_UI, [this](wxUpdateUIEvent& evt) { evt.Enable(!m_selected_items.empty()); });
+
     static_cast<wxButton*>(FindWindowById(wxID_CLOSE, this))->Bind(wxEVT_BUTTON, [this](wxCommandEvent&){ this->EndModal(wxID_CLOSE); });
     this->SetEscapeId(wxID_CLOSE);
     auto add_btn = [this, buttons]( size_t pos, int& ID, wxString title, wxString tooltip,
@@ -137,7 +138,20 @@ GalleryDialog::GalleryDialog(wxWindow* parent, bool modify_gallery/* = false*/)
 }
 
 GalleryDialog::~GalleryDialog()
-{   
+{
+    // From wxWidgets docs:
+    // The method void wxListCtrl::SetImageList(wxImageList* imageList, int which)
+    // does not take ownership of the image list, you have to delete it yourself.
+    if (m_image_list)
+        delete m_image_list;
+}
+
+int GalleryDialog::show(bool show_from_menu) 
+{
+    m_ok_btn->SetLabel(  show_from_menu ? _L("Add to bed")                       : _L("OK"));
+    m_ok_btn->SetToolTip(show_from_menu ? _L("Add selected shape(s) to the bed") : "");
+
+    return this->ShowModal();
 }
 
 bool GalleryDialog::can_delete() 
@@ -157,8 +171,9 @@ bool GalleryDialog::can_change_thumbnail()
 
 void GalleryDialog::on_dpi_changed(const wxRect& suggested_rect)
 {
-    const int& em = em_unit();
+    update();
 
+    const int& em = em_unit();
     msw_buttons_rescale(this, em, { ID_BTN_ADD_CUSTOM_SHAPE, ID_BTN_DEL_CUSTOM_SHAPE, ID_BTN_REPLACE_CUSTOM_PNG, wxID_OK, wxID_CLOSE });
 
     wxSize size = wxSize(50 * em, 35 * em);
@@ -169,13 +184,14 @@ void GalleryDialog::on_dpi_changed(const wxRect& suggested_rect)
     Refresh();
 }
 
-static void add_lock(wxImage& image) 
+static void add_lock(wxImage& image, wxWindow* parent_win) 
 {
-    int lock_sz = 22;
+    wxBitmapBundle* bmp_bndl = get_bmp_bundle("lock", 22);
 #ifdef __APPLE__
-    lock_sz /= mac_max_scaling_factor();
+    wxBitmap bmp = bmp_bndl->GetBitmap(bmp_bndl->GetDefaultSize() * mac_max_scaling_factor());
+#else
+    wxBitmap bmp = bmp_bndl->GetBitmapFor(parent_win);
 #endif
-    wxBitmap bmp = create_scaled_bitmap("lock", nullptr, lock_sz);
 
     wxImage lock_image = bmp.ConvertToImage();
     if (!lock_image.IsOk() || lock_image.GetWidth() == 0 || lock_image.GetHeight() == 0)
@@ -213,21 +229,28 @@ static void add_lock(wxImage& image)
     }
 }
 
-static void add_default_image(wxImageList* img_list, bool is_system)
+static void add_default_image(wxImageList* img_list, bool is_system, wxWindow* parent_win)
 {
-    int sz = IMG_PX_CNT;
+    wxBitmapBundle* bmp_bndl = get_bmp_bundle("cog", IMG_PX_CNT);
 #ifdef __APPLE__
-    sz /= mac_max_scaling_factor();
+    wxBitmap bmp = bmp_bndl->GetBitmap(bmp_bndl->GetDefaultSize() * mac_max_scaling_factor());
+#else
+    wxBitmap bmp = bmp_bndl->GetBitmapFor(parent_win);
 #endif
-    wxBitmap bmp = create_scaled_bitmap("cog", nullptr, sz, true);
 
+    bmp = bmp.ConvertToDisabled();
     if (is_system) {
         wxImage image = bmp.ConvertToImage();
         if (image.IsOk() && image.GetWidth() != 0 && image.GetHeight() != 0) {
-            add_lock(image);
+            add_lock(image, parent_win);
+#ifdef __APPLE__
+            bmp = wxBitmap(std::move(image), -1, mac_max_scaling_factor());
+#else
             bmp = wxBitmap(std::move(image));
+#endif
         }
     }
+
     img_list->Add(bmp);
 };
 
@@ -274,9 +297,8 @@ static void generate_thumbnail_from_model(const std::string& filename)
 
     GLVolumeCollection volumes;
     volumes.volumes.push_back(new GLVolume());
-    GLVolume* volume = volumes.volumes[0];
-    volume->indexed_vertex_array.load_mesh(model.mesh());
-    volume->indexed_vertex_array.finalize_geometry(true);
+    GLVolume* volume = volumes.volumes.back();
+    volume->model.init_from(model.mesh());
     volume->set_instance_transformation(model.objects[0]->instances[0]->get_transformation());
     volume->set_volume_transformation(model.objects[0]->volumes[0]->get_transformation());
 
@@ -340,8 +362,13 @@ void GalleryDialog::load_label_icon_list()
 
     // Make an image list containing large icons
 
+#ifdef __APPLE__
+    m_image_list = new wxImageList(IMG_PX_CNT, IMG_PX_CNT);
+    int px_cnt = IMG_PX_CNT * mac_max_scaling_factor();
+#else
     int px_cnt = (int)(em_unit() * IMG_PX_CNT * 0.1f + 0.5f);
     m_image_list = new wxImageList(px_cnt, px_cnt);
+#endif
 
     for (const auto& item : list_items) {
         fs::path model_path = fs::path((item.is_system ? m_sys_dir_path : m_cust_dir_path) + item.name);
@@ -358,7 +385,7 @@ void GalleryDialog::load_label_icon_list()
             if (can_generate_thumbnail)
                 generate_thumbnail_from_model(model_name);
             else {
-                add_default_image(m_image_list, item.is_system);
+                add_default_image(m_image_list, item.is_system, this);
                 continue;
             }
         }
@@ -367,14 +394,18 @@ void GalleryDialog::load_label_icon_list()
         if (!image.CanRead(from_u8(img_name)) ||
             !image.LoadFile(from_u8(img_name), wxBITMAP_TYPE_PNG) ||
             image.GetWidth() == 0 || image.GetHeight() == 0) {
-            add_default_image(m_image_list, item.is_system);
+            add_default_image(m_image_list, item.is_system, this);
             continue;
         }
         image.Rescale(px_cnt, px_cnt, wxIMAGE_QUALITY_BILINEAR);
 
         if (item.is_system)
-            add_lock(image);
+            add_lock(image, this);
+#ifdef __APPLE__
+        wxBitmap bmp = wxBitmap(std::move(image), -1, mac_max_scaling_factor());
+#else
         wxBitmap bmp = wxBitmap(std::move(image));
+#endif
         m_image_list->Add(bmp);
     }
 
