@@ -1,3 +1,8 @@
+///|/ Copyright (c) Prusa Research 2018 - 2023 Oleksandra Iushchenko @YuSanka, Lukáš Matěna @lukasmatena, Vojtěch Bubník @bubnikv, Enrico Turri @enricoturri1966, Lukáš Hejl @hejllukas, David Kocík @kocikdav, Tomáš Mészáros @tamasmeszaros, Vojtěch Král @vojtechkral
+///|/ Copyright (c) 2020 Gianni Ceccarelli @dakkar
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #include "ObjectDataViewModel.hpp"
 #include "wxExtensions.hpp"
 #include "BitmapCache.hpp"
@@ -19,6 +24,14 @@ wxDEFINE_EVENT(wxCUSTOMEVT_LAST_VOLUME_IS_DELETED, wxCommandEvent);
 
 BitmapCache* m_bitmap_cache = nullptr;
 
+wxBitmapBundle* find_bndl(const std::string& bmp_name)
+{
+    if (!m_bitmap_cache)
+        m_bitmap_cache = new BitmapCache;
+
+    return m_bitmap_cache->find_bndl(bmp_name);
+}
+
 // *****************************************************************************
 // ----------------------------------------------------------------------------
 // ObjectDataViewModelNode
@@ -34,10 +47,19 @@ void ObjectDataViewModelNode::init_container()
 #endif  //__WXGTK__
 }
 
+void ObjectDataViewModelNode::invalidate_container()
+{
+#ifndef __WXGTK__
+    if (this->GetChildCount() == 0)
+        this->m_container = false;
+#endif //__WXGTK__
+}
+
 static constexpr char LayerRootIcon[]   = "edit_layers_all";
 static constexpr char LayerIcon[]       = "edit_layers_some";
 static constexpr char WarningIcon[]     = "exclamation";
 static constexpr char WarningManifoldIcon[] = "exclamation_manifold";
+static constexpr char LockIcon[]        = "cut_";
 
 struct InfoItemAtributes {
     std::string name;
@@ -48,6 +70,7 @@ const std::map<InfoItemType, InfoItemAtributes> INFO_ITEMS{
 //           info_item Type                         info_item Name              info_item BitmapName
             { InfoItemType::CustomSupports,      {L("Paint-on supports"),       "fdm_supports_" },     },
             { InfoItemType::CustomSeam,          {L("Paint-on seam"),           "seam_" },             },
+            { InfoItemType::CutConnectors,       {L("Connectors"),              "cut_connectors" },    },
             { InfoItemType::MmuSegmentation,     {L("Multimaterial painting"),  "mmu_segmentation_"},  },
             { InfoItemType::Sinking,             {L("Sinking"),                 "sinking"},            },
             { InfoItemType::VariableLayerHeight, {L("Variable layer height"),   "layers"},             },
@@ -56,19 +79,17 @@ const std::map<InfoItemType, InfoItemAtributes> INFO_ITEMS{
 ObjectDataViewModelNode::ObjectDataViewModelNode(ObjectDataViewModelNode*   parent,
                                                  const wxString&            sub_obj_name,
                                                  Slic3r::ModelVolumeType    type,
-                                                 const wxBitmap&            bmp,
+                                                 const bool                 is_text_volume,
                                                  const wxString&            extruder,
-                                                 const int                  idx/* = -1*/,
-                                                 const std::string&         warning_icon_name /*= std::string*/) :
+                                                 const int                  idx/* = -1*/) :
     m_parent(parent),
     m_name(sub_obj_name),
     m_type(itVolume),
     m_volume_type(type),
+    m_is_text_volume(is_text_volume),
     m_idx(idx),
-    m_extruder(type == Slic3r::ModelVolumeType::MODEL_PART || type == Slic3r::ModelVolumeType::PARAMETER_MODIFIER ? extruder : ""),
-    m_warning_icon_name(warning_icon_name)
+    m_extruder(type == Slic3r::ModelVolumeType::MODEL_PART || type == Slic3r::ModelVolumeType::PARAMETER_MODIFIER ? extruder : "")
 {
-    m_bmp = bmp;
     set_action_and_extruder_icons();
     init_container();
 }
@@ -101,7 +122,7 @@ ObjectDataViewModelNode::ObjectDataViewModelNode(ObjectDataViewModelNode* parent
     }
     else if (type == itLayerRoot)
     {
-        m_bmp = create_scaled_bitmap(LayerRootIcon);    // FIXME: pass window ptr
+        m_bmp = *get_bmp_bundle(LayerRootIcon);
         m_name = _(L("Layers"));
     }
     else if (type == itInfo)
@@ -132,7 +153,7 @@ ObjectDataViewModelNode::ObjectDataViewModelNode(ObjectDataViewModelNode* parent
     }
     const std::string label_range = (boost::format(" %.2f-%.2f ") % layer_range.first % layer_range.second).str();
     m_name = _(L("Range")) + label_range + "(" + _(L("mm")) + ")";
-    m_bmp = create_scaled_bitmap(LayerIcon);    // FIXME: pass window ptr
+    m_bmp = *get_bmp_bundle(LayerIcon);
 
     set_action_and_extruder_icons();
     init_container();
@@ -151,7 +172,7 @@ void ObjectDataViewModelNode::set_action_and_extruder_icons()
 {
     m_action_icon_name = m_type & itObject              ? "advanced_plus" : 
                          m_type & (itVolume | itLayer)  ? "cog" : /*m_type & itInstance*/ "set_separate_obj";
-    m_action_icon = create_scaled_bitmap(m_action_icon_name);    // FIXME: pass window ptr
+    m_action_icon = *get_bmp_bundle(m_action_icon_name);
 
     // set extruder bitmap
     set_extruder_icon();
@@ -170,14 +191,7 @@ void ObjectDataViewModelNode::set_printable_icon(PrintIndicator printable)
 {
     m_printable = printable;
     m_printable_icon = m_printable == piUndef ? m_empty_bmp :
-                       create_scaled_bitmap(m_printable == piPrintable ? "eye_open.png" : "eye_closed.png");
-}
-
-void ObjectDataViewModelNode::set_warning_icon(const std::string& warning_icon_name)
-{
-    m_warning_icon_name = warning_icon_name;
-    if (warning_icon_name.empty())
-        m_bmp = m_empty_bmp;
+                       *get_bmp_bundle(m_printable == piPrintable ? "eye_open" : "eye_closed");
 }
 
 void ObjectDataViewModelNode::update_settings_digest_bitmaps()
@@ -185,14 +199,14 @@ void ObjectDataViewModelNode::update_settings_digest_bitmaps()
     m_bmp = m_empty_bmp;
 
     std::string scaled_bitmap_name = m_name.ToUTF8().data();
-    scaled_bitmap_name += "-em" + std::to_string(wxGetApp().em_unit()) + (wxGetApp().dark_mode() ? "-dm" : "");
+    scaled_bitmap_name += (wxGetApp().dark_mode() ? "-dm" : "");
 
-    wxBitmap *bmp = m_bitmap_cache->find(scaled_bitmap_name);
+    wxBitmapBundle *bmp = find_bndl(scaled_bitmap_name);
     if (bmp == nullptr) {
-        std::vector<wxBitmap> bmps;
+        std::vector<wxBitmapBundle*> bmps;
         for (auto& category : m_opt_categories)
-            bmps.emplace_back(SettingsFactory::get_category_bitmap(category, false));
-        bmp = m_bitmap_cache->insert(scaled_bitmap_name, bmps);
+            bmps.emplace_back(SettingsFactory::get_category_bitmap(category));
+        bmp = m_bitmap_cache->insert_bndl(scaled_bitmap_name, bmps);
     }
 
     m_bmp = *bmp;
@@ -216,13 +230,13 @@ bool ObjectDataViewModelNode::update_settings_digest(const std::vector<std::stri
     return true;
 }
 
-void ObjectDataViewModelNode::msw_rescale()
+void ObjectDataViewModelNode::sys_color_changed()
 {
     if (!m_action_icon_name.empty())
-        m_action_icon = create_scaled_bitmap(m_action_icon_name);
+        m_action_icon = *get_bmp_bundle(m_action_icon_name);
 
     if (m_printable != piUndef)
-        m_printable_icon = create_scaled_bitmap(m_printable == piPrintable ? "eye_open.png" : "eye_closed.png");
+        m_printable_icon = *get_bmp_bundle(m_printable == piPrintable ? "eye_open" : "eye_closed");
 
     if (!m_opt_categories.empty())
         update_settings_digest_bitmaps();
@@ -235,7 +249,7 @@ bool ObjectDataViewModelNode::SetValue(const wxVariant& variant, unsigned col)
     switch (col)
     {
     case colPrint:
-        m_printable_icon << variant;
+//        m_printable_icon << variant;
         return true;
     case colName: {
         DataViewBitmapText data;
@@ -250,7 +264,7 @@ bool ObjectDataViewModelNode::SetValue(const wxVariant& variant, unsigned col)
         m_extruder = data.GetText() == "0" ? _(L("default")) : data.GetText();
         return true; }
     case colEditing:
-        m_action_icon << variant;
+//        m_action_icon << variant;
         return true;
     default:
         printf("MyObjectTreeModel::SetValue: wrong column");
@@ -290,7 +304,7 @@ void ObjectDataViewModelNode::UpdateExtruderAndColorIcon(wxString extruder /*= "
 
     if (extruder_idx > 0) --extruder_idx;
     // Create the bitmap with color bars.
-    std::vector<wxBitmap*> bmps = get_extruder_color_icons(false);// use wide icons
+    std::vector<wxBitmapBundle*> bmps = get_extruder_color_icons();// use wide icons
     if (bmps.empty()) {
         m_extruder_bmp = wxNullBitmap;
         return;
@@ -322,14 +336,14 @@ static int get_root_idx(ObjectDataViewModelNode *parent_node, const ItemType roo
 
 ObjectDataViewModel::ObjectDataViewModel()
 {
-    m_bitmap_cache = new Slic3r::GUI::BitmapCache;
-
     m_volume_bmps = MenuFactory::get_volume_bitmaps();
-    m_warning_bmp = create_scaled_bitmap(WarningIcon);
-    m_warning_manifold_bmp = create_scaled_bitmap(WarningManifoldIcon);
+    m_text_volume_bmps = MenuFactory::get_text_volume_bitmaps();
+    m_warning_bmp = *get_bmp_bundle(WarningIcon);
+    m_warning_manifold_bmp = *get_bmp_bundle(WarningManifoldIcon);
+    m_lock_bmp    = *get_bmp_bundle(LockIcon);
 
     for (auto item : INFO_ITEMS)
-        m_info_bmps[item.first] = create_scaled_bitmap(item.second.bmp_name);
+        m_info_bmps[item.first] = get_bmp_bundle(item.second.bmp_name);
 }
 
 ObjectDataViewModel::~ObjectDataViewModel()
@@ -340,19 +354,55 @@ ObjectDataViewModel::~ObjectDataViewModel()
     m_bitmap_cache = nullptr;
 }
 
-wxBitmap& ObjectDataViewModel::GetWarningBitmap(const std::string& warning_icon_name)
+void ObjectDataViewModel::UpdateBitmapForNode(ObjectDataViewModelNode* node)
 {
-    return warning_icon_name.empty() ? m_empty_bmp : warning_icon_name == WarningIcon ? m_warning_bmp : m_warning_manifold_bmp;
+    int vol_type = static_cast<int>(node->GetVolumeType());
+    bool is_volume_node = vol_type >= 0;
+
+    if (!node->has_warning_icon() && !node->has_lock()) {
+        node->SetBitmap(is_volume_node ? (node->is_text_volume() ? *m_text_volume_bmps.at(vol_type) : *m_volume_bmps.at(vol_type)) : m_empty_bmp);
+        return;
+    }
+
+    std::string scaled_bitmap_name = std::string();
+    if (node->has_warning_icon())
+        scaled_bitmap_name += node->warning_icon_name();
+    if (node->has_lock()) 
+        scaled_bitmap_name += LockIcon;
+    if (is_volume_node)
+        scaled_bitmap_name += std::to_string(vol_type);
+    scaled_bitmap_name += (wxGetApp().dark_mode() ? "-dm" : "-lm");
+
+    wxBitmapBundle* bmp = find_bndl(scaled_bitmap_name);
+    if (!bmp) {
+        std::vector<wxBitmapBundle*> bmps;
+        if (node->has_warning_icon())
+            bmps.emplace_back(node->warning_icon_name() == WarningIcon ? &m_warning_bmp : &m_warning_manifold_bmp);
+        if (node->has_lock())
+            bmps.emplace_back(&m_lock_bmp);
+        if (is_volume_node)
+            bmps.emplace_back(node->is_text_volume() ? m_text_volume_bmps[vol_type] : m_volume_bmps[vol_type]);
+        bmp = m_bitmap_cache->insert_bndl(scaled_bitmap_name, bmps);
+    }
+
+    node->SetBitmap(*bmp);
 }
 
-wxDataViewItem ObjectDataViewModel::Add(const wxString &name, 
-                                        const int extruder,
-                                        const std::string& warning_icon_name/* = std::string()*/ )
+void ObjectDataViewModel::UpdateBitmapForNode(ObjectDataViewModelNode* node, const std::string& warning_icon_name, bool has_lock)
 {
-    const wxString extruder_str = extruder == 0 ? _L("default") : wxString::Format("%d", extruder);
-	auto root = new ObjectDataViewModelNode(name, extruder_str);
+    node->SetWarningIconName(warning_icon_name);
+    node->SetLock(has_lock);
+    UpdateBitmapForNode(node);
+}
+
+wxDataViewItem ObjectDataViewModel::AddObject(const wxString &name, 
+                                        const wxString& extruder,
+                                        const std::string& warning_icon_name,
+                                        const bool has_lock)
+{
+	auto root = new ObjectDataViewModelNode(name, extruder);
     // Add warning icon if detected auto-repaire
-    root->SetWarningBitmap(GetWarningBitmap(warning_icon_name), warning_icon_name);
+    UpdateBitmapForNode(root, warning_icon_name, has_lock);
 
     m_objects.push_back(root);
 	// notify control
@@ -365,42 +415,30 @@ wxDataViewItem ObjectDataViewModel::Add(const wxString &name,
 
 wxDataViewItem ObjectDataViewModel::AddVolumeChild( const wxDataViewItem &parent_item,
                                                     const wxString &name,
+                                                    const int volume_idx,
                                                     const Slic3r::ModelVolumeType volume_type,
-                                                    const std::string& warning_icon_name/* = std::string()*/,
-                                                    const int extruder/* = 0*/,
-                                                    const bool create_frst_child/* = true*/)
+                                                    const bool is_text_volume,
+                                                    const std::string& warning_icon_name,
+                                                    const wxString& extruder)
 {
 	ObjectDataViewModelNode *root = static_cast<ObjectDataViewModelNode*>(parent_item.GetID());
 	if (!root) return wxDataViewItem(0);
 
-    wxString extruder_str = extruder == 0 ? _(L("default")) : wxString::Format("%d", extruder);
-
     // get insertion position according to the existed Layers and/or Instances Items
     int insert_position = get_root_idx(root, itLayerRoot);
     if (insert_position < 0)
         insert_position = get_root_idx(root, itInstanceRoot);
 
-    if (create_frst_child && root->m_volumes_cnt == 0)
-    {
-        const Slic3r::ModelVolumeType type = Slic3r::ModelVolumeType::MODEL_PART;
-        const auto node = new ObjectDataViewModelNode(root, root->m_name, type, GetVolumeIcon(type, root->m_warning_icon_name), extruder_str, 0, root->m_warning_icon_name);
-
-        insert_position < 0 ? root->Append(node) : root->Insert(node, insert_position);
-		// notify control
-		const wxDataViewItem child((void*)node);
-		ItemAdded(parent_item, child);
-
-        root->m_volumes_cnt++;
-        if (insert_position >= 0) insert_position++;
-	}
-
-    const auto node = new ObjectDataViewModelNode(root, name, volume_type, GetVolumeIcon(volume_type, warning_icon_name), extruder_str, root->m_volumes_cnt, warning_icon_name);
+    const auto node = new ObjectDataViewModelNode(root, name, volume_type, is_text_volume, extruder, volume_idx);
+    UpdateBitmapForNode(node, warning_icon_name, root->has_lock() && volume_type < ModelVolumeType::PARAMETER_MODIFIER);
     insert_position < 0 ? root->Append(node) : root->Insert(node, insert_position);
 
     // if part with errors is added, but object wasn't marked, then mark it
-    if (!warning_icon_name.empty() && warning_icon_name != root->m_warning_icon_name && 
-        (root->m_warning_icon_name.empty() || root->m_warning_icon_name == WarningManifoldIcon) )
-        root->SetWarningBitmap(GetWarningBitmap(warning_icon_name), warning_icon_name);
+    if (!warning_icon_name.empty() && warning_icon_name != root->warning_icon_name() &&
+        (!root->has_warning_icon() || root->warning_icon_name() == WarningManifoldIcon)) {
+        root->SetWarningIconName(warning_icon_name);
+        UpdateBitmapForNode(root);
+    }
 
 	// notify control
     const wxDataViewItem child((void*)node);
@@ -429,7 +467,7 @@ wxDataViewItem ObjectDataViewModel::AddInfoChild(const wxDataViewItem &parent_it
     }
 
     root->Insert(node, idx+1);
-    node->SetBitmap(m_info_bmps.at(info_type));
+    node->SetBitmap(*m_info_bmps.at(info_type));
     // notify control
     const wxDataViewItem child((void*)node);
     ItemAdded(parent_item, child);
@@ -598,14 +636,12 @@ wxDataViewItem ObjectDataViewModel::AddLayersRoot(const wxDataViewItem &parent_i
 
 wxDataViewItem ObjectDataViewModel::AddLayersChild(const wxDataViewItem &parent_item, 
                                                    const t_layer_height_range& layer_range,
-                                                   const int extruder/* = 0*/, 
+                                                   const wxString& extruder,
                                                    const int index /* = -1*/)
 {
     ObjectDataViewModelNode *parent_node = static_cast<ObjectDataViewModelNode*>(parent_item.GetID());
     if (!parent_node) return wxDataViewItem(0);
 
-    wxString extruder_str = extruder == 0 ? _(L("default")) : wxString::Format("%d", extruder);
-
     // get LayerRoot node
     ObjectDataViewModelNode *layer_root_node;
     wxDataViewItem layer_root_item;
@@ -622,7 +658,7 @@ wxDataViewItem ObjectDataViewModel::AddLayersChild(const wxDataViewItem &parent_
     }
 
     // Add layer node
-    ObjectDataViewModelNode *layer_node = new ObjectDataViewModelNode(layer_root_node, layer_range, index, extruder_str);
+    ObjectDataViewModelNode *layer_node = new ObjectDataViewModelNode(layer_root_node, layer_range, index, extruder);
     if (index < 0)
         layer_root_node->Append(layer_node);
     else
@@ -711,10 +747,7 @@ wxDataViewItem ObjectDataViewModel::Delete(const wxDataViewItem &item)
             delete node_parent;
             ret_item = wxDataViewItem(obj_node);
 
-#ifndef __WXGTK__
-            if (obj_node->GetChildCount() == 0)
-                obj_node->m_container = false;
-#endif //__WXGTK__
+            obj_node->invalidate_container();
             ItemDeleted(ret_item, wxDataViewItem(node_parent));
             return ret_item;
         }
@@ -730,10 +763,7 @@ wxDataViewItem ObjectDataViewModel::Delete(const wxDataViewItem &item)
             delete node_parent;
             ret_item = wxDataViewItem(obj_node);
 
-#ifndef __WXGTK__
-            if (obj_node->GetChildCount() == 0)
-                obj_node->m_container = false;
-#endif //__WXGTK__
+            obj_node->invalidate_container();
             ItemDeleted(ret_item, wxDataViewItem(node_parent));
             return ret_item;
         }
@@ -747,18 +777,15 @@ wxDataViewItem ObjectDataViewModel::Delete(const wxDataViewItem &item)
 
             // get index of the last VolumeItem in CildrenList
             size_t vol_idx = GetItemIndexForFirstVolume(node_parent);
+            ObjectDataViewModelNode *last_child_node = node_parent->GetNthChild(vol_idx);
 
             // delete this last volume
-            ObjectDataViewModelNode *last_child_node = node_parent->GetNthChild(vol_idx);
             DeleteSettings(wxDataViewItem(last_child_node));
             node_parent->GetChildren().Remove(last_child_node);
             node_parent->m_volumes_cnt = 0;
             delete last_child_node;
 
-#ifndef __WXGTK__
-            if (node_parent->GetChildCount() == 0)
-                node_parent->m_container = false;
-#endif //__WXGTK__
+            node_parent->invalidate_container();
             ItemDeleted(parent, wxDataViewItem(last_child_node));
 
             wxCommandEvent event(wxCUSTOMEVT_LAST_VOLUME_IS_DELETED);
@@ -793,10 +820,7 @@ wxDataViewItem ObjectDataViewModel::Delete(const wxDataViewItem &item)
 
 	// set m_containet to FALSE if parent has no child
 	if (node_parent) {
-#ifndef __WXGTK__
-        if (node_parent->GetChildCount() == 0)
-            node_parent->m_container = false;
-#endif //__WXGTK__
+        node_parent->invalidate_container();
 		ret_item = parent;
 	}
 
@@ -838,10 +862,7 @@ wxDataViewItem ObjectDataViewModel::DeleteLastInstance(const wxDataViewItem &par
         parent_node->set_printable_icon(last_inst_printable);
         ItemDeleted(parent_item, inst_root_item);
         ItemChanged(parent_item);
-#ifndef __WXGTK__
-        if (parent_node->GetChildCount() == 0)
-            parent_node->m_container = false;
-#endif //__WXGTK__
+        parent_node->invalidate_container();
     }
 
     // update object_node printable property
@@ -886,10 +907,7 @@ void ObjectDataViewModel::DeleteChildren(wxDataViewItem& parent)
         ItemDeleted(parent, item);
     }
 
-    // set m_containet to FALSE if parent has no child
-#ifndef __WXGTK__
-        root->m_container = false;
-#endif //__WXGTK__
+    root->invalidate_container();
 }
 
 void ObjectDataViewModel::DeleteVolumeChildren(wxDataViewItem& parent)
@@ -919,11 +937,7 @@ void ObjectDataViewModel::DeleteVolumeChildren(wxDataViewItem& parent)
         ItemDeleted(parent, item);
     }
     root->m_volumes_cnt = 0;
-
-    // set m_containet to FALSE if parent has no child
-#ifndef __WXGTK__
-    root->m_container = false;
-#endif //__WXGTK__
+    root->invalidate_container();
 }
 
 void ObjectDataViewModel::DeleteSettings(const wxDataViewItem& parent)
@@ -1033,6 +1047,16 @@ int  ObjectDataViewModel::GetItemIdByLayerRange(const int obj_idx, const t_layer
     return GetLayerIdByItem(item);
 }
 
+wxString ObjectDataViewModel::GetItemName(const wxDataViewItem& item) const
+{
+    if (!item.IsOk())
+        return wxEmptyString;
+    ObjectDataViewModelNode* node = static_cast<ObjectDataViewModelNode*>(item.GetID());
+    if (!node)
+        return wxEmptyString;
+    return node->GetName();
+}
+
 int ObjectDataViewModel::GetIdByItem(const wxDataViewItem& item) const
 {
 	if(!item.IsOk())
@@ -1237,7 +1261,7 @@ wxString ObjectDataViewModel::GetName(const wxDataViewItem &item) const
 	return node->m_name;
 }
 
-wxBitmap& ObjectDataViewModel::GetBitmap(const wxDataViewItem &item) const
+wxBitmapBundle& ObjectDataViewModel::GetBitmap(const wxDataViewItem &item) const
 {
     ObjectDataViewModelNode *node = static_cast<ObjectDataViewModelNode*>(item.GetID());
     return node->m_bmp;
@@ -1278,16 +1302,16 @@ void ObjectDataViewModel::GetValue(wxVariant &variant, const wxDataViewItem &ite
 	switch (col)
 	{
 	case colPrint:
-		variant << node->m_printable_icon;
+		variant << node->m_printable_icon.GetBitmapFor(m_ctrl);
 		break;
 	case colName:
-        variant << DataViewBitmapText(node->m_name, node->m_bmp);
+        variant << DataViewBitmapText(node->m_name, node->m_bmp.GetBitmapFor(m_ctrl));
 		break;
 	case colExtruder:
-		variant << DataViewBitmapText(node->m_extruder, node->m_extruder_bmp);
+		variant << DataViewBitmapText(node->m_extruder, node->m_extruder_bmp.GetBitmapFor(m_ctrl));
 		break;
 	case colEditing:
-		variant << node->m_action_icon;
+		variant << node->m_action_icon.GetBitmapFor(m_ctrl);
 		break;
 	default:
 		;
@@ -1611,22 +1635,6 @@ void ObjectDataViewModel::UpdateSettingsDigest(const wxDataViewItem &item,
     ItemChanged(item);
 }
 
-void ObjectDataViewModel::SetVolumeType(const wxDataViewItem &item, const Slic3r::ModelVolumeType volume_type)
-{
-    if (!item.IsOk() || GetItemType(item) != itVolume) 
-        return;
-
-    ObjectDataViewModelNode *node = static_cast<ObjectDataViewModelNode*>(item.GetID());
-    node->SetVolumeType(volume_type);
-    node->SetBitmap(m_volume_bmps[int(volume_type)]);
-    if (volume_type != Slic3r::ModelVolumeType::MODEL_PART && volume_type != Slic3r::ModelVolumeType::PARAMETER_MODIFIER)
-        node->SetExtruder("");          // hide extruder
-    else if (node->GetExtruder().IsEmpty())
-        node->SetExtruder("default");   // show extruder ans set it to default
-    node->UpdateExtruderAndColorIcon();
-    ItemChanged(item);
-}
-
 ModelVolumeType ObjectDataViewModel::GetVolumeType(const wxDataViewItem& item)
 {
     if (!item.IsOk() || GetItemType(item) != itVolume) 
@@ -1676,14 +1684,16 @@ wxDataViewItem ObjectDataViewModel::SetObjectPrintableState(
     return obj_item;
 }
 
-void ObjectDataViewModel::Rescale()
+void ObjectDataViewModel::UpdateBitmaps()
 {
     m_volume_bmps = MenuFactory::get_volume_bitmaps();
-    m_warning_bmp = create_scaled_bitmap(WarningIcon);
-    m_warning_manifold_bmp = create_scaled_bitmap(WarningManifoldIcon);
+    m_text_volume_bmps = MenuFactory::get_text_volume_bitmaps();
+    m_warning_bmp = *get_bmp_bundle(WarningIcon);
+    m_warning_manifold_bmp = *get_bmp_bundle(WarningManifoldIcon);
+    m_lock_bmp    = *get_bmp_bundle(LockIcon);
 
     for (auto item : INFO_ITEMS)
-        m_info_bmps[item.first] = create_scaled_bitmap(item.second.bmp_name);
+        m_info_bmps[item.first] = get_bmp_bundle(item.second.bmp_name);
 
     wxDataViewItemArray all_items;
     GetAllChildren(wxDataViewItem(0), all_items);
@@ -1694,24 +1704,22 @@ void ObjectDataViewModel::Rescale()
             continue;
 
         ObjectDataViewModelNode *node = static_cast<ObjectDataViewModelNode*>(item.GetID());
-        node->msw_rescale();
+        node->sys_color_changed();
 
         switch (node->m_type)
         {
         case itObject:
-            if (node->m_bmp.IsOk()) node->m_bmp = GetWarningBitmap(node->m_warning_icon_name);
-            break;
         case itVolume:
-            node->m_bmp = GetVolumeIcon(node->m_volume_type, node->m_warning_icon_name);
+            UpdateBitmapForNode(node);
             break;
         case itLayerRoot:
-            node->m_bmp = create_scaled_bitmap(LayerRootIcon);
+            node->m_bmp = *get_bmp_bundle(LayerRootIcon);
             break;
         case itLayer:
-            node->m_bmp = create_scaled_bitmap(LayerIcon);
+            node->m_bmp = *get_bmp_bundle(LayerIcon);
             break;
         case itInfo:
-            node->m_bmp = m_info_bmps.at(node->m_info_item_type);
+            node->m_bmp = *m_info_bmps.at(node->m_info_item_type);
             break;
         default: break;
         }
@@ -1720,27 +1728,6 @@ void ObjectDataViewModel::Rescale()
     }
 }
 
-wxBitmap ObjectDataViewModel::GetVolumeIcon(const Slic3r::ModelVolumeType vol_type, const std::string& warning_icon_name/* = std::string()*/)
-{
-    if (warning_icon_name.empty())
-        return m_volume_bmps[static_cast<int>(vol_type)];
-
-    std::string scaled_bitmap_name = warning_icon_name + std::to_string(static_cast<int>(vol_type));
-    scaled_bitmap_name += "-em" + std::to_string(wxGetApp().em_unit()) + (wxGetApp().dark_mode() ? "-dm" : "-lm");
-
-    wxBitmap *bmp = m_bitmap_cache->find(scaled_bitmap_name);
-    if (bmp == nullptr) {
-        std::vector<wxBitmap> bmps;
-
-        bmps.emplace_back(GetWarningBitmap(warning_icon_name));
-        bmps.emplace_back(m_volume_bmps[static_cast<int>(vol_type)]);
-
-        bmp = m_bitmap_cache->insert(scaled_bitmap_name, bmps);
-    }
-
-    return *bmp;
-}
-
 void ObjectDataViewModel::AddWarningIcon(const wxDataViewItem& item, const std::string& warning_icon_name)
 {
     if (!item.IsOk())
@@ -1748,13 +1735,14 @@ void ObjectDataViewModel::AddWarningIcon(const wxDataViewItem& item, const std::
     ObjectDataViewModelNode *node = static_cast<ObjectDataViewModelNode*>(item.GetID());
 
     if (node->GetType() & itObject) {
-        node->SetWarningBitmap(GetWarningBitmap(warning_icon_name), warning_icon_name);
+        UpdateBitmapForNode(node, warning_icon_name, node->has_lock());
         return;
     }
 
     if (node->GetType() & itVolume) {
-        node->SetWarningBitmap(GetVolumeIcon(node->GetVolumeType(), warning_icon_name), warning_icon_name);
-        node->GetParent()->SetWarningBitmap(GetWarningBitmap(warning_icon_name), warning_icon_name);
+        UpdateBitmapForNode(node, warning_icon_name, node->has_lock());
+        if (ObjectDataViewModelNode* parent = node->GetParent())
+            UpdateBitmapForNode(parent, warning_icon_name, parent->has_lock());
         return;
     }
 }
@@ -1769,12 +1757,9 @@ void ObjectDataViewModel::DeleteWarningIcon(const wxDataViewItem& item, const bo
     if (!node->GetBitmap().IsOk() || !(node->GetType() & (itVolume | itObject)))
         return;
 
-    if (node->GetType() & itVolume) {
-        node->SetWarningBitmap(m_volume_bmps[static_cast<int>(node->volume_type())], "");
-        return;
-    }
+    node->SetWarningIconName(std::string());
+    UpdateBitmapForNode(node);
 
-    node->SetWarningBitmap(wxNullBitmap, "");
     if (unmark_object)
     {
         wxDataViewItemArray children;
@@ -1801,6 +1786,26 @@ void ObjectDataViewModel::UpdateWarningIcon(const wxDataViewItem& item, const st
         AddWarningIcon(item, warning_icon_name);
 }
 
+void ObjectDataViewModel::UpdateLockIcon(const wxDataViewItem& item, bool has_lock)
+{
+    if (!item.IsOk())
+        return;
+    ObjectDataViewModelNode* node = static_cast<ObjectDataViewModelNode*>(item.GetID());
+    if (node->has_lock() == has_lock)
+        return;
+
+    node->SetLock(has_lock);
+    UpdateBitmapForNode(node);
+
+    if (node->GetType() & itObject) {
+        wxDataViewItemArray children;
+        GetChildren(item, children);
+        for (const wxDataViewItem& child : children)
+            UpdateLockIcon(child, has_lock);
+    }
+    ItemChanged(item);
+}
+
 } // namespace GUI
 } // namespace Slic3r
 
