@@ -1,3 +1,7 @@
+///|/ Copyright (c) Prusa Research 2018 - 2023 Oleksandra Iushchenko @YuSanka, Lukáš Matěna @lukasmatena, Lukáš Hejl @hejllukas, Enrico Turri @enricoturri1966, David Kocík @kocikdav, Vojtěch Bubník @bubnikv, Tomáš Mészáros @tamasmeszaros, Vojtěch Král @vojtechkral
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #ifndef slic3r_GUI_ObjectDataViewModel_hpp_
 #define slic3r_GUI_ObjectDataViewModel_hpp_
 
@@ -51,6 +55,7 @@ enum class InfoItemType
     Undef,
     CustomSupports,
     CustomSeam,
+    CutConnectors,
     MmuSegmentation,
     Sinking,
     VariableLayerHeight
@@ -63,25 +68,27 @@ class ObjectDataViewModelNode
 {
     ObjectDataViewModelNode*	    m_parent;
     MyObjectTreeModelNodePtrArray   m_children;
-    wxBitmap                        m_empty_bmp;
+    wxBitmapBundle                  m_empty_bmp;
     size_t                          m_volumes_cnt = 0;
     std::vector< std::string >      m_opt_categories;
     t_layer_height_range            m_layer_range = { 0.0f, 0.0f };
 
     wxString				        m_name;
-    wxBitmap&                       m_bmp = m_empty_bmp;
+    wxBitmapBundle&                 m_bmp = m_empty_bmp;
     ItemType				        m_type;
     int                             m_idx = -1;
     bool					        m_container = false;
     wxString				        m_extruder = "default";
-    wxBitmap                        m_extruder_bmp;
-    wxBitmap				        m_action_icon;
+    wxBitmapBundle                  m_extruder_bmp;
+    wxBitmapBundle				    m_action_icon;
     PrintIndicator                  m_printable {piUndef};
-    wxBitmap				        m_printable_icon;
+    wxBitmapBundle                  m_printable_icon;
     std::string                     m_warning_icon_name{ "" };
+    bool                            m_has_lock{false};
 
     std::string                     m_action_icon_name = "";
-    ModelVolumeType                 m_volume_type;
+    ModelVolumeType                 m_volume_type{ -1 };
+    bool                            m_is_text_volume{ false };
     InfoItemType                    m_info_item_type {InfoItemType::Undef};
 
 public:
@@ -99,10 +106,9 @@ public:
     ObjectDataViewModelNode(ObjectDataViewModelNode* parent,
                             const wxString& sub_obj_name,
                             Slic3r::ModelVolumeType type,
-                            const wxBitmap& bmp,
+                            const bool is_text_volume,
                             const wxString& extruder,
-                            const int idx = -1,
-                            const std::string& warning_icon_name = std::string());
+                            const int idx = -1 );
 
     ObjectDataViewModelNode(ObjectDataViewModelNode* parent,
                             const t_layer_height_range& layer_range,
@@ -128,7 +134,9 @@ public:
     }
 
 	void init_container();
-	bool IsContainer() const
+    void invalidate_container();
+
+    bool IsContainer() const
 	{
 		return m_container;
 	}
@@ -179,16 +187,17 @@ public:
 
     bool            SetValue(const wxVariant &variant, unsigned int col);
     void            SetVolumeType(ModelVolumeType type) { m_volume_type = type; }
-    void            SetBitmap(const wxBitmap &icon) { m_bmp = icon; }
+    void            SetBitmap(const wxBitmapBundle &icon) { m_bmp = icon; }
     void            SetExtruder(const wxString &extruder) { m_extruder = extruder; }
-    void            SetWarningBitmap(const wxBitmap& icon, const std::string& warning_icon_name) { m_bmp = icon; m_warning_icon_name = warning_icon_name; }
-    const wxBitmap& GetBitmap() const               { return m_bmp; }
+    void            SetWarningIconName(const std::string& warning_icon_name) { m_warning_icon_name = warning_icon_name; }
+    void            SetLock(bool has_lock)                                   { m_has_lock = has_lock; }
+    const wxBitmapBundle& GetBitmap() const         { return m_bmp; }
     const wxString& GetName() const                 { return m_name; }
     ItemType        GetType() const                 { return m_type; }
     InfoItemType    GetInfoItemType() const         { return m_info_item_type; }
 	void			SetIdx(const int& idx);
 	int             GetIdx() const                  { return m_idx; }
-    ModelVolumeType GetVolumeType()                 { return m_volume_type; }
+    ModelVolumeType GetVolumeType() const           { return m_volume_type; }
 	t_layer_height_range    GetLayerRange() const   { return m_layer_range; }
     wxString        GetExtruder()                   { return m_extruder; }
     PrintIndicator  IsPrintable() const             { return m_printable; }
@@ -228,19 +237,20 @@ public:
     void        set_extruder_icon();
 	// Set printable icon for node
     void        set_printable_icon(PrintIndicator printable);
-    // Set warning icon for node
-    void        set_warning_icon(const std::string& warning_icon);
 
     void        update_settings_digest_bitmaps();
     bool        update_settings_digest(const std::vector<std::string>& categories);
     int         volume_type() const { return int(m_volume_type); }
-    void        msw_rescale();
+    bool        is_text_volume() const { return m_is_text_volume; }
+    void        sys_color_changed();
 
 #ifndef NDEBUG
     bool 		valid();
 #endif /* NDEBUG */
     bool        invalid() const { return m_idx < -1; }
-    bool        has_warning_icon() const { return !m_warning_icon_name.empty(); }
+    bool        has_warning_icon() const            { return !m_warning_icon_name.empty(); }
+    bool        has_lock() const                    { return m_has_lock; }
+    const std::string& warning_icon_name() const    { return m_warning_icon_name; }
 
 private:
     friend class ObjectDataViewModel;
@@ -257,11 +267,13 @@ wxDECLARE_EVENT(wxCUSTOMEVT_LAST_VOLUME_IS_DELETED, wxCommandEvent);
 class ObjectDataViewModel :public wxDataViewModel
 {
     std::vector<ObjectDataViewModelNode*>       m_objects;
-    std::vector<wxBitmap>                       m_volume_bmps;
-    std::map<InfoItemType, wxBitmap>            m_info_bmps;
-    wxBitmap                                    m_empty_bmp;
-    wxBitmap                                    m_warning_bmp;
-    wxBitmap                                    m_warning_manifold_bmp;
+    std::vector<wxBitmapBundle*>                m_volume_bmps;
+    std::vector<wxBitmapBundle*>                m_text_volume_bmps;
+    std::map<InfoItemType, wxBitmapBundle*>     m_info_bmps;
+    wxBitmapBundle                              m_empty_bmp;
+    wxBitmapBundle                              m_warning_bmp;
+    wxBitmapBundle                              m_warning_manifold_bmp;
+    wxBitmapBundle                              m_lock_bmp;
 
     wxDataViewCtrl*                             m_ctrl { nullptr };
 
@@ -269,15 +281,17 @@ public:
     ObjectDataViewModel();
     ~ObjectDataViewModel();
 
-    wxDataViewItem Add( const wxString &name,
-                        const int extruder,
-                        const std::string& warning_icon_name = std::string());
+    wxDataViewItem AddObject( const wxString &name,
+                        const wxString& extruder,
+                        const std::string& warning_icon_name,
+                        const bool has_lock);
     wxDataViewItem AddVolumeChild(  const wxDataViewItem &parent_item,
                                     const wxString &name,
+                                    const int volume_idx,
                                     const Slic3r::ModelVolumeType volume_type,
-                                    const std::string& warning_icon_name = std::string(),
-                                    const int extruder = 0,
-                                    const bool create_frst_child = true);
+                                    const bool is_text_volume,
+                                    const std::string& warning_icon_name,
+                                    const wxString& extruder);
     wxDataViewItem AddSettingsChild(const wxDataViewItem &parent_item);
     wxDataViewItem AddInfoChild(const wxDataViewItem &parent_item, InfoItemType info_type);
     wxDataViewItem AddInstanceChild(const wxDataViewItem &parent_item, size_t num);
@@ -285,7 +299,7 @@ public:
     wxDataViewItem AddLayersRoot(const wxDataViewItem &parent_item);
     wxDataViewItem AddLayersChild(  const wxDataViewItem &parent_item,
                                     const t_layer_height_range& layer_range,
-                                    const int extruder = 0,
+                                    const wxString& extruder,
                                     const int index = -1);
     size_t         GetItemIndexForFirstVolume(ObjectDataViewModelNode* node_parent);
     wxDataViewItem Delete(const wxDataViewItem &item);
@@ -301,6 +315,7 @@ public:
     wxDataViewItem GetItemByLayerId(int obj_idx, int layer_idx);
     wxDataViewItem GetItemByLayerRange(const int obj_idx, const t_layer_height_range& layer_range);
     int  GetItemIdByLayerRange(const int obj_idx, const t_layer_height_range& layer_range);
+    wxString GetItemName(const wxDataViewItem& item) const;
     int  GetIdByItem(const wxDataViewItem& item) const;
     int  GetIdByItemAndType(const wxDataViewItem& item, const ItemType type) const;
     int  GetObjectIdByItem(const wxDataViewItem& item) const;
@@ -315,7 +330,7 @@ public:
     // helper method for wxLog
 
     wxString    GetName(const wxDataViewItem &item) const;
-    wxBitmap&   GetBitmap(const wxDataViewItem &item) const;
+    wxBitmapBundle&   GetBitmap(const wxDataViewItem &item) const;
     wxString    GetExtruder(const wxDataViewItem &item) const;
     int         GetExtruderNumber(const wxDataViewItem &item) const;
 
@@ -374,7 +389,6 @@ public:
     void    UpdateObjectPrintable(wxDataViewItem parent_item);
     void    UpdateInstancesPrintable(wxDataViewItem parent_item);
 
-    void    SetVolumeType(const wxDataViewItem &item, const Slic3r::ModelVolumeType type);
     ModelVolumeType GetVolumeType(const wxDataViewItem &item);
     wxDataViewItem SetPrintableState( PrintIndicator printable, int obj_idx,
                                       int subobj_idx = -1, 
@@ -382,14 +396,13 @@ public:
     wxDataViewItem SetObjectPrintableState(PrintIndicator printable, wxDataViewItem obj_item);
 
     void    SetAssociatedControl(wxDataViewCtrl* ctrl) { m_ctrl = ctrl; }
-    // Rescale bitmaps for existing Items
-    void    Rescale();
+    // Rescale bitmaps for existing Items 
+    void    UpdateBitmaps();
 
-    wxBitmap    GetVolumeIcon(const Slic3r::ModelVolumeType vol_type,
-                              const std::string& warning_icon_name = std::string());
     void        AddWarningIcon(const wxDataViewItem& item, const std::string& warning_name);
     void        DeleteWarningIcon(const wxDataViewItem& item, const bool unmark_object = false);
     void        UpdateWarningIcon(const wxDataViewItem& item, const std::string& warning_name);
+    void        UpdateLockIcon(const wxDataViewItem& item, bool has_lock);
     bool        HasWarningIcon(const wxDataViewItem& item) const;
     t_layer_height_range    GetLayerRangeByItem(const wxDataViewItem& item) const;
 
@@ -403,7 +416,8 @@ private:
     wxDataViewItem  AddInstanceRoot(const wxDataViewItem& parent_item);
     void            AddAllChildren(const wxDataViewItem& parent);
 
-    wxBitmap&       GetWarningBitmap(const std::string& warning_icon_name);
+    void            UpdateBitmapForNode(ObjectDataViewModelNode* node);
+    void            UpdateBitmapForNode(ObjectDataViewModelNode* node, const std::string& warning_icon_name, bool has_lock);
 };
 
 
