@@ -1,3 +1,15 @@
+///|/ Copyright (c) Prusa Research 2017 - 2023 Oleksandra Iushchenko @YuSanka, Vojtěch Bubník @bubnikv, Lukáš Hejl @hejllukas, Tomáš Mészáros @tamasmeszaros, David Kocík @kocikdav, Lukáš Matěna @lukasmatena, Vojtěch Král @vojtechkral, Enrico Turri @enricoturri1966
+///|/ Copyright (c) 2018 Martin Loidl @LoidlM
+///|/
+///|/ ported from lib/Slic3r/GUI/OptionsGroup.pm:
+///|/ Copyright (c) Prusa Research 2016 - 2018 Vojtěch Bubník @bubnikv, Oleksandra Iushchenko @YuSanka
+///|/ Copyright (c) Slic3r 2011 - 2015 Alessandro Ranellucci @alranel
+///|/ Copyright (c) 2013 Scott Penrose
+///|/ Copyright (c) 2012 Henrik Brix Andersen @henrikbrixandersen
+///|/ Copyright (c) 2011 Richard Goodwin
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #include "OptionsGroup.hpp"
 #include "ConfigExceptions.hpp"
 #include "Plater.hpp"
@@ -32,16 +44,15 @@ const t_field& OptionsGroup::build_field(const t_config_option_key& id, const Co
     // Check the gui_type field first, fall through
     // is the normal type.
     switch (opt.gui_type) {
+    case ConfigOptionDef::GUIType::select_close:
     case ConfigOptionDef::GUIType::select_open:
+    case ConfigOptionDef::GUIType::f_enum_open:
+    case ConfigOptionDef::GUIType::i_enum_open:
         m_fields.emplace(id, Choice::Create<Choice>(this->ctrl_parent(), opt, id));
         break;
     case ConfigOptionDef::GUIType::color:
         m_fields.emplace(id, ColourPicker::Create<ColourPicker>(this->ctrl_parent(), opt, id));
         break;
-    case ConfigOptionDef::GUIType::f_enum_open:
-    case ConfigOptionDef::GUIType::i_enum_open:
-        m_fields.emplace(id, Choice::Create<Choice>(this->ctrl_parent(), opt, id));
-        break;
     case ConfigOptionDef::GUIType::slider:
         m_fields.emplace(id, SliderCtrl::Create<SliderCtrl>(this->ctrl_parent(), opt, id));
         break;
@@ -54,6 +65,7 @@ const t_field& OptionsGroup::build_field(const t_config_option_key& id, const Co
     default:
         switch (opt.type) {
             case coFloatOrPercent:
+            case coFloatsOrPercents:
             case coFloat:
             case coFloats:
 			case coPercent:
@@ -101,7 +113,7 @@ const t_field& OptionsGroup::build_field(const t_config_option_key& id, const Co
 			this->back_to_initial_value(opt_id);
 	};
 	field->m_back_to_sys_value = [this](std::string opt_id) {
-		if (!this->m_disabled)
+		if (!m_disabled)
 			this->back_to_sys_value(opt_id);
 	};
 
@@ -118,6 +130,38 @@ OptionsGroup::OptionsGroup(	wxWindow* _parent, const wxString& title,
 {
 }
 
+Option::Option(const ConfigOptionDef& _opt, t_config_option_key id) : opt(_opt), opt_id(id)
+{
+    if (!opt.tooltip.empty()) {
+        wxString tooltip;
+        if (opt.opt_key.rfind("branching", 0) == 0)
+            tooltip = _L("Unavailable for this method.") + "\n";
+        tooltip += _(opt.tooltip);
+
+        edit_tooltip(tooltip);
+
+        opt.tooltip = into_u8(tooltip);
+    }
+}
+
+void Line::clear()
+{
+    if (near_label_widget_win)
+        near_label_widget_win = nullptr;
+
+    if (widget_sizer) {
+        widget_sizer->Clear(true);
+        delete widget_sizer;
+        widget_sizer = nullptr;
+    }
+
+    if (extra_widget_sizer) {
+        extra_widget_sizer->Clear(true);
+        delete extra_widget_sizer;
+        extra_widget_sizer = nullptr;
+    }
+}
+
 wxWindow* OptionsGroup::ctrl_parent() const
 {
 	return this->custom_ctrl && m_use_custom_ctrl_as_parent ? static_cast<wxWindow*>(this->custom_ctrl) : (this->stb ? static_cast<wxWindow*>(this->stb) : this->parent());
@@ -231,7 +275,7 @@ void OptionsGroup::activate_line(Line& line)
 		}
     }
 
-	auto option_set = line.get_options();
+    const std::vector<Option>& option_set = line.get_options();
 	bool is_legend_line = option_set.front().opt.gui_type == ConfigOptionDef::GUIType::legend;
 
     if (!custom_ctrl && m_use_custom_ctrl) {
@@ -263,15 +307,13 @@ void OptionsGroup::activate_line(Line& line)
 		return;
 	}
 
-    auto grid_sizer = m_grid_sizer;
-
     if (custom_ctrl)
         m_use_custom_ctrl_as_parent = true;
 
     // if we have an extra column, build it
     if (extra_column) {
-		m_extra_column_item_ptrs.push_back(extra_column(this->ctrl_parent(), line));
-		grid_sizer->Add(m_extra_column_item_ptrs.back(), 0, wxALIGN_CENTER_VERTICAL | wxRIGHT, 3);
+        m_extra_column_item_ptrs.push_back(extra_column(this->ctrl_parent(), line));
+        m_grid_sizer->Add(m_extra_column_item_ptrs.back(), 0, wxALIGN_CENTER_VERTICAL | wxRIGHT, 3);
 	}
 
 	// Build a label if we have it
@@ -298,12 +340,12 @@ void OptionsGroup::activate_line(Line& line)
                 label->Wrap(label_width * wxGetApp().em_unit()); // avoid a Linux/GTK bug
             }
             if (!line.near_label_widget)
-                grid_sizer->Add(label, 0, (staticbox ? 0 : wxALIGN_RIGHT | wxRIGHT) | wxALIGN_CENTER_VERTICAL, line.label.IsEmpty() ? 0 : 5);
+                m_grid_sizer->Add(label, 0, (staticbox ? 0 : wxALIGN_RIGHT | wxRIGHT) | wxALIGN_CENTER_VERTICAL, line.label.IsEmpty() ? 0 : 5);
             else if (!line.label.IsEmpty()) {
                 // If we're here, we have some widget near the label
                 // so we need a horizontal sizer to arrange these things
                 auto sizer = new wxBoxSizer(wxHORIZONTAL);
-                grid_sizer->Add(sizer, 0, wxEXPAND | (staticbox ? wxALL : wxBOTTOM | wxTOP | wxLEFT), staticbox ? 0 : 1);
+                m_grid_sizer->Add(sizer, 0, wxEXPAND | (staticbox ? wxALL : wxBOTTOM | wxTOP | wxLEFT), staticbox ? 0 : 1);
                 sizer->Add(label, 0, (staticbox ? 0 : wxALIGN_RIGHT | wxRIGHT) | wxALIGN_CENTER_VERTICAL, 5);
             }
             if (label != nullptr && line.label_tooltip != "")
@@ -317,15 +359,19 @@ void OptionsGroup::activate_line(Line& line)
         if (custom_ctrl)
             line.widget_sizer = wgt;
         else
-            grid_sizer->Add(wgt, 0, wxEXPAND | wxBOTTOM | wxTOP, (wxOSX || line.label.IsEmpty()) ? 0 : 5);
+            m_grid_sizer->Add(wgt, 0, wxEXPAND | wxBOTTOM | wxTOP, (wxOSX || line.label.IsEmpty()) ? 0 : 5);
 		return;
 	}
 
 	// If we're here, we have more than one option or a single option with sidetext
     // so we need a horizontal sizer to arrange these things
-	auto sizer = new wxBoxSizer(wxHORIZONTAL);
-    if (!custom_ctrl)
-        grid_sizer->Add(sizer, 0, wxEXPAND | (staticbox ? wxALL : wxBOTTOM | wxTOP | wxLEFT), staticbox ? 0 : 1);
+    wxBoxSizer* h_sizer{ nullptr };
+    if (!custom_ctrl) {
+        // but this sizer is currently used just for NON-custom_ctrl cases
+        h_sizer = new wxBoxSizer(wxHORIZONTAL);
+        m_grid_sizer->Add(h_sizer, 0, wxEXPAND | (staticbox ? wxALL : wxBOTTOM | wxTOP | wxLEFT), staticbox ? 0 : 1);
+    }
+
     // If we have a single option with no sidetext just add it directly to the grid sizer
     if (option_set.size() == 1 && option_set.front().opt.sidetext.size() == 0 &&
 		option_set.front().side_widget == nullptr && line.get_extra_widgets().size() == 0) {
@@ -334,39 +380,37 @@ void OptionsGroup::activate_line(Line& line)
 
         if (!custom_ctrl) {
             if (is_window_field(field))
-                sizer->Add(field->getWindow(), option.opt.full_width ? 1 : 0,
+                h_sizer->Add(field->getWindow(), option.opt.full_width ? 1 : 0,
                     wxBOTTOM | wxTOP | (option.opt.full_width ? int(wxEXPAND) : int(wxALIGN_CENTER_VERTICAL)), (wxOSX || !staticbox) ? 0 : 2);
             if (is_sizer_field(field))
-                sizer->Add(field->getSizer(), 1, (option.opt.full_width ? int(wxEXPAND) : int(wxALIGN_CENTER_VERTICAL)), 0);
+                h_sizer->Add(field->getSizer(), 1, (option.opt.full_width ? int(wxEXPAND) : int(wxALIGN_CENTER_VERTICAL)), 0);
         }
         return;
 	}
 
-    bool is_multioption_line = option_set.size() > 1;
-    for (auto opt : option_set) {
-		ConfigOptionDef option = opt.opt;
-        wxSizer* sizer_tmp = sizer;
-		// add label if any
-		if ((is_multioption_line || line.label.IsEmpty()) && !option.label.empty() && !custom_ctrl) {
-//!			To correct translation by context have to use wxGETTEXT_IN_CONTEXT macro from wxWidget 3.1.1
-			wxString str_label = (option.label == L_CONTEXT("Top", "Layers") || option.label == L_CONTEXT("Bottom", "Layers")) ?
-				_CTX(option.label, "Layers") :
-				_(option.label);
-			label = new wxStaticText(this->ctrl_parent(), wxID_ANY, str_label + ": ", wxDefaultPosition, //wxDefaultSize);
-				wxSize(sublabel_width != -1 ? sublabel_width * wxGetApp().em_unit() : -1, -1), wxALIGN_RIGHT);
-			label->SetBackgroundStyle(wxBG_STYLE_PAINT);
-            label->SetFont(wxGetApp().normal_font());
-			sizer_tmp->Add(label, 0, wxALIGN_CENTER_VERTICAL, 0);
-		}
+    for (const Option& opt : option_set) {
+        // add field
+        auto& field = build_field(opt);
 
-		// add field
-		const Option& opt_ref = opt;
-		auto& field = build_field(opt_ref);
         if (!custom_ctrl) {
+            ConfigOptionDef option = opt.opt;
+            // add label if any
+            if ((option_set.size() > 1 || line.label.IsEmpty()) && !option.label.empty()) {
+                // those two parameter names require localization with context
+                wxString str_label = (option.label == "Top" || option.label == "Bottom") ?
+                    _CTX(option.label, "Layers") :
+                    _(option.label);
+                label = new wxStaticText(this->ctrl_parent(), wxID_ANY, str_label + ": ", wxDefaultPosition, //wxDefaultSize);
+                    wxSize(sublabel_width != -1 ? sublabel_width * wxGetApp().em_unit() : -1, -1), wxALIGN_RIGHT);
+                label->SetBackgroundStyle(wxBG_STYLE_PAINT);
+                label->SetFont(wxGetApp().normal_font());
+                h_sizer->Add(label, 0, wxALIGN_CENTER_VERTICAL, 0);
+            }
+
             if (option_set.size() == 1 && option_set.front().opt.full_width)
             {
                 const auto v_sizer = new wxBoxSizer(wxVERTICAL);
-                sizer_tmp->Add(v_sizer, 1, wxEXPAND);
+                h_sizer->Add(v_sizer, 1, wxEXPAND);
                 is_sizer_field(field) ?
                     v_sizer->Add(field->getSizer(), 0, wxEXPAND) :
                     v_sizer->Add(field->getWindow(), 0, wxEXPAND);
@@ -374,8 +418,8 @@ void OptionsGroup::activate_line(Line& line)
             }
 
             is_sizer_field(field) ?
-                sizer_tmp->Add(field->getSizer(), 0, wxALIGN_CENTER_VERTICAL, 0) :
-                sizer_tmp->Add(field->getWindow(), 0, wxALIGN_CENTER_VERTICAL, 0);
+                h_sizer->Add(field->getSizer(), 0, wxALIGN_CENTER_VERTICAL, 0) :
+                h_sizer->Add(field->getWindow(), 0, wxALIGN_CENTER_VERTICAL, 0);
 
             // add sidetext if any
             if (!option.sidetext.empty() || sidetext_width > 0) {
@@ -383,35 +427,35 @@ void OptionsGroup::activate_line(Line& line)
                     wxSize(sidetext_width != -1 ? sidetext_width * wxGetApp().em_unit() : -1, -1), wxALIGN_LEFT);
                 sidetext->SetBackgroundStyle(wxBG_STYLE_PAINT);
                 sidetext->SetFont(wxGetApp().normal_font());
-                sizer_tmp->Add(sidetext, 0, wxLEFT | wxALIGN_CENTER_VERTICAL, 4);
+                h_sizer->Add(sidetext, 0, wxLEFT | wxALIGN_CENTER_VERTICAL, 4);
             }
 
             // add side widget if any
             if (opt.side_widget != nullptr) {
-                sizer_tmp->Add(opt.side_widget(this->ctrl_parent())/*!.target<wxWindow>()*/, 0, wxLEFT | wxALIGN_CENTER_VERTICAL, 1);    //! requires verification
+                h_sizer->Add(opt.side_widget(this->ctrl_parent())/*!.target<wxWindow>()*/, 0, wxLEFT | wxALIGN_CENTER_VERTICAL, 1);    //! requires verification
             }
 
             if (opt.opt_id != option_set.back().opt_id) //! istead of (opt != option_set.back())
-                sizer_tmp->AddSpacer(6);
+                h_sizer->AddSpacer(6);
         }
-	}
+    }
 
-	// add extra sizers if any
-	for (auto extra_widget : line.get_extra_widgets())
+    // add extra sizers if any
+    for (auto extra_widget : line.get_extra_widgets())
     {
         if (line.get_extra_widgets().size() == 1 && !staticbox)
         {
             // extra widget for non-staticbox option group (like for the frequently used parameters on the sidebar) should be wxALIGN_RIGHT
             const auto v_sizer = new wxBoxSizer(wxVERTICAL);
-            sizer->Add(v_sizer, option_set.size() == 1 ? 0 : 1, wxEXPAND);
+            h_sizer->Add(v_sizer, option_set.size() == 1 ? 0 : 1, wxEXPAND);
             v_sizer->Add(extra_widget(this->ctrl_parent()), 0, wxALIGN_RIGHT);
             return;
         }
 
         line.extra_widget_sizer = extra_widget(this->ctrl_parent());
         if (!custom_ctrl)
-            sizer->Add(line.extra_widget_sizer, 0, wxLEFT | wxALIGN_CENTER_VERTICAL, 4);        //! requires verification
-	}
+            h_sizer->Add(line.extra_widget_sizer, 0, wxLEFT | wxALIGN_CENTER_VERTICAL, 4);        //! requires verification
+    }
 }
 
 // create all controls for the option group from the m_lines
@@ -478,20 +522,8 @@ void OptionsGroup::clear(bool destroy_custom_ctrl)
 	m_grid_sizer = nullptr;
 	sizer = nullptr;
 
-	for (Line& line : m_lines) {
-        if (line.near_label_widget_win)
-            line.near_label_widget_win = nullptr;
-
-        if (line.widget_sizer) {
-            line.widget_sizer->Clear(true);
-            line.widget_sizer = nullptr;
-        }
-
-        if (line.extra_widget_sizer) {
-            line.extra_widget_sizer->Clear(true);
-            line.extra_widget_sizer = nullptr;
-        }
-	}
+    for (Line& line : m_lines)
+        line.clear();
 
     if (custom_ctrl) {
         for (auto const &item : m_fields) {
@@ -511,9 +543,8 @@ void OptionsGroup::clear(bool destroy_custom_ctrl)
 
 Line OptionsGroup::create_single_option_line(const Option& option, const std::string& path/* = std::string()*/) const
 {
-    wxString tooltip = _(option.opt.tooltip);
-    edit_tooltip(tooltip);
-	Line retval{ _(option.opt.label), tooltip };
+    Line retval{ _(option.opt.label), from_u8(option.opt.tooltip) };
+
 	retval.label_path = path;
     retval.append_option(option);
     return retval;
@@ -591,6 +622,12 @@ void ConfigOptionsGroup::back_to_sys_value(const std::string& opt_key)
 void ConfigOptionsGroup::back_to_config_value(const DynamicPrintConfig& config, const std::string& opt_key)
 {
 	boost::any value;
+	if (opt_key == "bed_shape") {
+        for (const std::string& key : {"bed_custom_texture", "bed_custom_model"}) {
+            value = config.opt_string(key);
+            this->change_opt_value(key, value);
+        }
+	}
 	if (opt_key == "extruders_count") {
 		auto   *nozzle_diameter = dynamic_cast<const ConfigOptionFloats*>(config.option("nozzle_diameter"));
 		value = int(nozzle_diameter->values.size());
@@ -600,6 +637,7 @@ void ConfigOptionsGroup::back_to_config_value(const DynamicPrintConfig& config,
              is_option_without_field(opt_key) ) {
         value = get_config_value(config, opt_key);
         this->change_opt_value(opt_key, value);
+        OptionsGroup::on_change_OG(opt_key, value);
         return;
     }
 	else
@@ -721,7 +759,6 @@ void ConfigOptionsGroup::msw_rescale()
                 // check if window is ScalableButton
                 ScalableButton* sc_btn = dynamic_cast<ScalableButton*>(win);
                 if (sc_btn) {
-                    sc_btn->msw_rescale();
                     sc_btn->SetSize(sc_btn->GetBestSize());
                     return;
                 }
@@ -766,7 +803,7 @@ void ConfigOptionsGroup::sys_color_changed()
                 wxWindow* win = item->GetWindow();
                 // check if window is ScalableButton
                 if (ScalableButton* sc_btn = dynamic_cast<ScalableButton*>(win)) {
-                    sc_btn->msw_rescale();
+                    sc_btn->sys_color_changed();
                     return;
                 }
                 wxGetApp().UpdateDarkUI(win, dynamic_cast<wxButton*>(win) != nullptr);
@@ -855,6 +892,14 @@ boost::any ConfigOptionsGroup::get_config_value(const DynamicPrintConfig& config
 		ret = text_value;
 		break;
 	}
+	case coFloatsOrPercents:{
+		const auto& val = config.option<ConfigOptionFloatsOrPercents>(opt_key)->get_at(idx);
+        text_value = double_to_string(val.value);
+		if (val.percent)
+			text_value += "%";
+		ret = text_value;
+		break;
+	}
 	case coPercent:{
 		double val = config.option<ConfigOptionPercent>(opt_key)->value;
 		text_value = wxString::Format(_T("%i"), int(val));
@@ -980,10 +1025,18 @@ bool OptionsGroup::launch_browser(const std::string& path_end)
     return wxGetApp().open_browser_with_warning_dialog(OptionsGroup::get_url(path_end), wxGetApp().mainframe->m_tabpanel);
 }
 
+// list of options, which doesn't have a related filed
+static const std::set<std::string> options_without_field = {
+    "compatible_printers",
+    "compatible_prints",
+    "bed_shape",
+    "filament_ramming_parameters",
+    "gcode_substitutions",
+};
+
 bool OptionsGroup::is_option_without_field(const std::string& opt_key)
 {
-    return  opt_key!= "thumbnails" // "thumbnails" has related field
-            && PresetCollection::is_independent_from_extruder_number_option(opt_key);
+    return  options_without_field.find(opt_key) != options_without_field.end();
 }
 
 
@@ -1010,34 +1063,62 @@ void ogStaticText::SetText(const wxString& value, bool wrap/* = true*/)
 
 void ogStaticText::SetPathEnd(const std::string& link)
 {
+#ifndef __linux__
+
+    Bind(wxEVT_ENTER_WINDOW, [this, link](wxMouseEvent& event) {
+        SetToolTip(OptionsGroup::get_url(get_app_config()->get("suppress_hyperlinks") != "1" ? link : std::string()));
+        FocusText(true);
+        event.Skip();
+    });
+    Bind(wxEVT_LEAVE_WINDOW, [this](wxMouseEvent& event) { FocusText(false); event.Skip(); });
+
     Bind(wxEVT_LEFT_DOWN, [this](wxMouseEvent& event) {
         if (HasCapture())
             return;
         this->CaptureMouse();
         event.Skip();
-    } );
+    });
     Bind(wxEVT_LEFT_UP, [link, this](wxMouseEvent& event) {
         if (!HasCapture())
             return;
         ReleaseMouse();
         OptionsGroup::launch_browser(link);
         event.Skip();
-    } );
-    Bind(wxEVT_ENTER_WINDOW, [this, link](wxMouseEvent& event) {
-        SetToolTip(OptionsGroup::get_url(get_app_config()->get("suppress_hyperlinks") != "1" ? link : std::string()));
-        FocusText(true); 
-        event.Skip(); 
     });
-    Bind(wxEVT_LEAVE_WINDOW, [this](wxMouseEvent& event) { FocusText(false); event.Skip(); });
+
+#else
+
+    // Workaround: On Linux wxStaticText doesn't receive wxEVT_ENTER(LEAVE)_WINDOW events,
+    // so implement this behaviour trough wxEVT_MOTION events for this control and it's parent
+    Bind(wxEVT_MOTION, [link, this](wxMouseEvent& event) {
+        SetToolTip(OptionsGroup::get_url(!get_app_config()->get_bool("suppress_hyperlinks") ? link : std::string()));
+        FocusText(true);
+        event.Skip();
+    });
+    GetParent()->Bind(wxEVT_MOTION, [this](wxMouseEvent& event) {
+        FocusText(false);
+        event.Skip();
+    });
+
+    // On Linux a mouse capturing causes a totally application freeze
+    Bind(wxEVT_LEFT_UP, [link, this](wxMouseEvent& event) {
+        OptionsGroup::launch_browser(link);
+        event.Skip();
+    });
+
+#endif
 }
 
 void ogStaticText::FocusText(bool focus)
 {
-    if (get_app_config()->get("suppress_hyperlinks") == "1")
+    if (get_app_config()->get_bool("suppress_hyperlinks"))
         return;
 
     SetFont(focus ? Slic3r::GUI::wxGetApp().link_font() :
-                    Slic3r::GUI::wxGetApp().normal_font());
+        Slic3r::GUI::wxGetApp().normal_font());
+#ifdef __linux__
+    this->GetContainingSizer()->Layout();
+#endif
     Refresh();
 }
 
