@@ -1,3 +1,7 @@
+///|/ Copyright (c) Prusa Research 2020 - 2022 David Kocík @kocikdav, Oleksandra Iushchenko @YuSanka, Vojtěch Bubník @bubnikv
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #ifndef slic3r_PhysicalPrinterDialog_hpp_
 #define slic3r_PhysicalPrinterDialog_hpp_
 
@@ -48,8 +52,7 @@ public:
     void                SuppressDelete();
     void                AllowDelete();
 
-    void                msw_rescale();
-    void                on_sys_color_changed() {};
+    void                on_sys_color_changed();
 };
 
 
@@ -98,9 +101,8 @@ public:
     void        DeletePreset(PresetForPrinter* preset_for_printer);
 protected:
     void on_dpi_changed(const wxRect& suggested_rect) override;
-    void on_sys_color_changed() override {};
+    void on_sys_color_changed() override;
 
-    bool had_all_mk3;
 };
 
 
