@@ -1,3 +1,7 @@
+///|/ Copyright (c) Prusa Research 2018 - 2023 Oleksandra Iushchenko @YuSanka, Lukáš Matěna @lukasmatena, David Kocík @kocikdav, Vojtěch Bubník @bubnikv, Lukáš Hejl @hejllukas, Enrico Turri @enricoturri1966, Vojtěch Král @vojtechkral
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #include "PrintHostDialogs.hpp"
 
 #include <algorithm>
@@ -27,6 +31,7 @@
 #include "libslic3r/AppConfig.hpp"
 #include "NotificationManager.hpp"
 #include "ExtraRenderers.hpp"
+#include "format.hpp"
 
 namespace fs = boost::filesystem;
 
@@ -35,13 +40,16 @@ namespace GUI {
 
 static const char *CONFIG_KEY_PATH  = "printhost_path";
 static const char *CONFIG_KEY_GROUP = "printhost_group";
+static const char* CONFIG_KEY_STORAGE = "printhost_storage";
 
-PrintHostSendDialog::PrintHostSendDialog(const fs::path &path, PrintHostPostUploadActions post_actions, const wxArrayString &groups)
+PrintHostSendDialog::PrintHostSendDialog(const fs::path &path, PrintHostPostUploadActions post_actions, const wxArrayString &groups, const wxArrayString& storage_paths, const wxArrayString& storage_names)
     : MsgDialog(static_cast<wxWindow*>(wxGetApp().mainframe), _L("Send G-Code to printer host"), _L("Upload to Printer Host with the following filename:"), 0) // Set style = 0 to avoid default creation of the "OK" button. 
                                                                                                                                                                // All buttons will be added later in this constructor 
     , txt_filename(new wxTextCtrl(this, wxID_ANY))
     , combo_groups(!groups.IsEmpty() ? new wxComboBox(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, groups, wxCB_READONLY) : nullptr)
+    , combo_storage(storage_names.GetCount() > 1 ? new wxComboBox(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, storage_names, wxCB_READONLY) : nullptr)
     , post_upload_action(PrintHostPostUploadAction::None)
+    , m_paths(storage_paths)
 {
 #ifdef __APPLE__
     txt_filename->OSXDisableAllSmartSubstitutions();
@@ -65,6 +73,23 @@ PrintHostSendDialog::PrintHostSendDialog(const fs::path &path, PrintHostPostUplo
             combo_groups->SetValue(recent_group);
     }
 
+    if (combo_storage != nullptr) {
+        // PrusaLink specific: User needs to choose a storage
+        auto* label_group = new wxStaticText(this, wxID_ANY, _L("Upload to storage") + ":");
+        content_sizer->Add(label_group);
+        content_sizer->Add(combo_storage, 0, wxBOTTOM, 2 * VERT_SPACING);
+        combo_storage->SetValue(storage_names.front());
+        wxString recent_storage = from_u8(app_config->get("recent", CONFIG_KEY_STORAGE));
+        if (!recent_storage.empty())
+            combo_storage->SetValue(recent_storage); 
+    } else if (storage_names.GetCount() == 1){
+        // PrusaLink specific: Show which storage has been detected.
+        auto* label_group = new wxStaticText(this, wxID_ANY, _L("Upload to storage") + ": " + storage_names.front());
+        content_sizer->Add(label_group);
+        m_preselected_storage = storage_paths.front();
+    }
+
+
     wxString recent_path = from_u8(app_config->get("recent", CONFIG_KEY_PATH));
     if (recent_path.Length() > 0 && recent_path[recent_path.Length() - 1] != '/') {
         recent_path += '/';
@@ -97,6 +122,16 @@ PrintHostSendDialog::PrintHostSendDialog(const fs::path &path, PrintHostPostUplo
     });
     txt_filename->SetFocus();
     
+    if (post_actions.has(PrintHostPostUploadAction::QueuePrint)) {
+        auto* btn_print = add_button(wxID_ADD, false, _L("Upload to Queue"));
+        btn_print->Bind(wxEVT_BUTTON, [this, validate_path](wxCommandEvent&) {
+            if (validate_path(txt_filename->GetValue())) {
+                post_upload_action = PrintHostPostUploadAction::QueuePrint;
+                EndDialog(wxID_OK);
+            }
+            });
+    }
+
     if (post_actions.has(PrintHostPostUploadAction::StartPrint)) {
         auto* btn_print = add_button(wxID_YES, false, _L("Upload and Print"));
         btn_print->Bind(wxEVT_BUTTON, [this, validate_path](wxCommandEvent&) {
@@ -162,6 +197,15 @@ std::string PrintHostSendDialog::group() const
     }
 }
 
+std::string PrintHostSendDialog::storage() const
+{
+    if (!combo_storage)
+        return GUI::format("%1%", m_preselected_storage);
+    if (combo_storage->GetSelection() < 0 || combo_storage->GetSelection() >= int(m_paths.size()))
+        return {};
+    return boost::nowide::narrow(m_paths[combo_storage->GetSelection()]);
+}
+
 void PrintHostSendDialog::EndModal(int ret)
 {
     if (ret == wxID_OK) {
@@ -180,16 +224,19 @@ void PrintHostSendDialog::EndModal(int ret)
             wxString group = combo_groups->GetValue();
             app_config->set("recent", CONFIG_KEY_GROUP, into_u8(group));
         }
+        if (combo_storage != nullptr) {
+            wxString storage = combo_storage->GetValue();
+            app_config->set("recent", CONFIG_KEY_STORAGE, into_u8(storage));
+        }
     }
 
     MsgDialog::EndModal(ret);
 }
 
-
-
 wxDEFINE_EVENT(EVT_PRINTHOST_PROGRESS, PrintHostQueueDialog::Event);
 wxDEFINE_EVENT(EVT_PRINTHOST_ERROR,    PrintHostQueueDialog::Event);
 wxDEFINE_EVENT(EVT_PRINTHOST_CANCEL,   PrintHostQueueDialog::Event);
+wxDEFINE_EVENT(EVT_PRINTHOST_INFO,  PrintHostQueueDialog::Event);
 
 PrintHostQueueDialog::Event::Event(wxEventType eventType, int winid, size_t job_id)
     : wxEvent(winid, eventType)
@@ -205,7 +252,14 @@ PrintHostQueueDialog::Event::Event(wxEventType eventType, int winid, size_t job_
 PrintHostQueueDialog::Event::Event(wxEventType eventType, int winid, size_t job_id, wxString error)
     : wxEvent(winid, eventType)
     , job_id(job_id)
-    , error(std::move(error))
+    , status(std::move(error))
+{}
+
+PrintHostQueueDialog::Event::Event(wxEventType eventType, int winid, size_t job_id, wxString tag, wxString status)
+    : wxEvent(winid, eventType)
+    , job_id(job_id)
+    , tag(std::move(tag))
+    , status(std::move(status))
 {}
 
 wxEvent *PrintHostQueueDialog::Event::Clone() const
@@ -218,16 +272,17 @@ PrintHostQueueDialog::PrintHostQueueDialog(wxWindow *parent)
     , on_progress_evt(this, EVT_PRINTHOST_PROGRESS, &PrintHostQueueDialog::on_progress, this)
     , on_error_evt(this, EVT_PRINTHOST_ERROR, &PrintHostQueueDialog::on_error, this)
     , on_cancel_evt(this, EVT_PRINTHOST_CANCEL, &PrintHostQueueDialog::on_cancel, this)
+    , on_info_evt(this, EVT_PRINTHOST_INFO, &PrintHostQueueDialog::on_info, this)
 {
     const auto em = GetTextExtent("m").x;
 
     auto *topsizer = new wxBoxSizer(wxVERTICAL);
 
     std::vector<int> widths;
-    widths.reserve(6);
+    widths.reserve(7);
     if (!load_user_data(UDT_COLS, widths)) {
         widths.clear();
-        for (size_t i = 0; i < 6; i++)
+        for (size_t i = 0; i < 7; i++)
             widths.push_back(-1);
     }
 
@@ -250,7 +305,8 @@ PrintHostQueueDialog::PrintHostQueueDialog(wxWindow *parent)
     append_text_column(_L("Host"),  widths[3]);
     append_text_column(_CTX(L_CONTEXT("Size", "OfFile"), "OfFile"), widths[4]);
     append_text_column(_L("Filename"),      widths[5]);
-    append_text_column(_L("Error Message"), -1, wxALIGN_CENTER, wxDATAVIEW_COL_HIDDEN);
+    append_text_column(_L("Message"), widths[6]);
+    //append_text_column(_L("Error Message"), -1, wxALIGN_CENTER, wxDATAVIEW_COL_HIDDEN);
  
     auto *btnsizer = new wxBoxSizer(wxHORIZONTAL);
     btn_cancel = new wxButton(this, wxID_DELETE, _L("Cancel selected"));
@@ -421,7 +477,7 @@ void PrintHostQueueDialog::on_error(Event &evt)
 
     set_state(evt.job_id, ST_ERROR);
 
-    auto errormsg = from_u8((boost::format("%1%\n%2%") % _utf8(L("Error uploading to print host:")) % std::string(evt.error.ToUTF8())).str());
+    auto errormsg = format_wxstr("%1%\n%2%", _L("Error uploading to print host") + ":", evt.status);
     job_list->SetValue(wxVariant(0), evt.job_id, COL_PROGRESS);
     job_list->SetValue(wxVariant(errormsg), evt.job_id, COL_ERRORMSG);    // Stashes the error message into a hidden column for later
 
@@ -450,6 +506,29 @@ void PrintHostQueueDialog::on_cancel(Event &evt)
     wxGetApp().notification_manager()->upload_job_notification_show_canceled(evt.job_id + 1, boost::nowide::narrow(nm.GetString()), boost::nowide::narrow(hst.GetString()));
 }
 
+void PrintHostQueueDialog::on_info(Event& evt)
+{
+    wxCHECK_RET(evt.job_id < (size_t)job_list->GetItemCount(), "Out of bounds access to job list");
+    
+    if (evt.tag == L"resolve") {
+        wxVariant hst(evt.status);
+        job_list->SetValue(hst, evt.job_id, COL_HOST);
+        wxGetApp().notification_manager()->set_upload_job_notification_host(evt.job_id + 1, boost::nowide::narrow(evt.status));
+    } else if (evt.tag == L"complete") {
+        wxVariant hst(evt.status);
+        job_list->SetValue(hst, evt.job_id, COL_ERRORMSG);
+        wxGetApp().notification_manager()->set_upload_job_notification_completed(evt.job_id + 1);
+        wxGetApp().notification_manager()->set_upload_job_notification_status(evt.job_id + 1, boost::nowide::narrow(evt.status));
+    } else if(evt.tag == L"complete_with_warning"){
+        wxVariant hst(evt.status);
+        job_list->SetValue(hst, evt.job_id, COL_ERRORMSG);
+        wxGetApp().notification_manager()->set_upload_job_notification_completed_with_warning(evt.job_id + 1);
+        wxGetApp().notification_manager()->set_upload_job_notification_status(evt.job_id + 1, boost::nowide::narrow(evt.status));
+    } else if (evt.tag == L"set_complete_off") {
+        wxGetApp().notification_manager()->set_upload_job_notification_comp_on_100(evt.job_id + 1, false);
+    }
+}
+
 void PrintHostQueueDialog::get_active_jobs(std::vector<std::pair<std::string, std::string>>& ret)
 {
     int ic = job_list->GetItemCount();
@@ -461,7 +540,6 @@ void PrintHostQueueDialog::get_active_jobs(std::vector<std::pair<std::string, st
         if(st == JobState::ST_NEW || st == JobState::ST_PROGRESS)
             ret.emplace_back(upload_names[i]);       
     }
-    //job_list->data
 }
 void PrintHostQueueDialog::save_user_data(int udt)
 {
@@ -490,8 +568,11 @@ bool PrintHostQueueDialog::load_user_data(int udt, std::vector<int>& vector)
     auto* app_config = wxGetApp().app_config;
     auto hasget = [app_config](const std::string& name, std::vector<int>& vector)->bool {
         if (app_config->has(name)) {
-            vector.push_back(std::stoi(app_config->get(name)));
-            return true;
+            std::string val = app_config->get(name);
+            if (!val.empty() || val[0]!='\0') {
+                vector.push_back(std::stoi(val));
+                return true;
+            }
         }
         return false;
     };
@@ -510,7 +591,7 @@ bool PrintHostQueueDialog::load_user_data(int udt, std::vector<int>& vector)
     }
     if (udt & UserDataType::UDT_COLS)
     {
-        for (size_t i = 0; i < 6; i++)
+        for (size_t i = 0; i < 7; i++)
         {
             if (!hasget("print_host_queue_dialog_column_" + std::to_string(i), vector))
                 return false;
