@@ -1,8 +1,13 @@
+///|/ Copyright (c) Prusa Research 2019 - 2023 Enrico Turri @enricoturri1966, Oleksandra Iushchenko @YuSanka, Filip Sykala @Jony01, Lukáš Matěna @lukasmatena, Vojtěch Bubník @bubnikv
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #ifndef slic3r_GUI_Selection_hpp_
 #define slic3r_GUI_Selection_hpp_
 
 #include "libslic3r/Geometry.hpp"
-#include "GLModel.hpp"
+#include "GUI_Geometry.hpp"
+#include "CoordAxes.hpp"
 
 #include <set>
 #include <optional>
@@ -12,6 +17,7 @@ namespace Slic3r {
 class Shader;
 class Model;
 class ModelObject;
+class ModelVolume;
 class GLVolume;
 class GLArrow;
 class GLCurvedArrow;
@@ -24,58 +30,6 @@ using ModelObjectPtrs = std::vector<ModelObject*>;
 
 
 namespace GUI {
-class TransformationType
-{
-public:
-    enum Enum {
-        // Transforming in a world coordinate system
-        World = 0,
-        // Transforming in a local coordinate system
-        Local = 1,
-        // Absolute transformations, allowed in local coordinate system only.
-        Absolute = 0,
-        // Relative transformations, allowed in both local and world coordinate system.
-        Relative = 2,
-        // For group selection, the transformation is performed as if the group made a single solid body.
-        Joint = 0,
-        // For group selection, the transformation is performed on each object independently.
-        Independent = 4,
-
-        World_Relative_Joint = World | Relative | Joint,
-        World_Relative_Independent = World | Relative | Independent,
-        Local_Absolute_Joint = Local | Absolute | Joint,
-        Local_Absolute_Independent = Local | Absolute | Independent,
-        Local_Relative_Joint = Local | Relative | Joint,
-        Local_Relative_Independent = Local | Relative | Independent,
-    };
-
-    TransformationType() : m_value(World) {}
-    TransformationType(Enum value) : m_value(value) {}
-    TransformationType& operator=(Enum value) { m_value = value; return *this; }
-
-    Enum operator()() const { return m_value; }
-    bool has(Enum v) const { return ((unsigned int)m_value & (unsigned int)v) != 0; }
-
-    void set_world()        { this->remove(Local); }
-    void set_local()        { this->add(Local); }
-    void set_absolute()     { this->remove(Relative); }
-    void set_relative()     { this->add(Relative); }
-    void set_joint()        { this->remove(Independent); }
-    void set_independent()  { this->add(Independent); }
-
-    bool world()        const { return !this->has(Local); }
-    bool local()        const { return this->has(Local); }
-    bool absolute()     const { return !this->has(Relative); }
-    bool relative()     const { return this->has(Relative); }
-    bool joint()        const { return !this->has(Independent); }
-    bool independent()  const { return this->has(Independent); }
-
-private:
-    void add(Enum v) { m_value = Enum((unsigned int)m_value | (unsigned int)v); }
-    void remove(Enum v) { m_value = Enum((unsigned int)m_value & (~(unsigned int)v)); }
-
-    Enum    m_value;
-};
 
 class Selection
 {
@@ -107,46 +61,15 @@ public:
 private:
     struct VolumeCache
     {
-    private:
-        struct TransformCache
-        {
-            Vec3d position;
-            Vec3d rotation;
-            Vec3d scaling_factor;
-            Vec3d mirror;
-            Transform3d rotation_matrix;
-            Transform3d scale_matrix;
-            Transform3d mirror_matrix;
-            Transform3d full_matrix;
-
-            TransformCache();
-            explicit TransformCache(const Geometry::Transformation& transform);
-        };
-
-        TransformCache m_volume;
-        TransformCache m_instance;
-
-    public:
         VolumeCache() = default;
         VolumeCache(const Geometry::Transformation& volume_transform, const Geometry::Transformation& instance_transform);
 
-        const Vec3d& get_volume_position() const { return m_volume.position; }
-        const Vec3d& get_volume_rotation() const { return m_volume.rotation; }
-        const Vec3d& get_volume_scaling_factor() const { return m_volume.scaling_factor; }
-        const Vec3d& get_volume_mirror() const { return m_volume.mirror; }
-        const Transform3d& get_volume_rotation_matrix() const { return m_volume.rotation_matrix; }
-        const Transform3d& get_volume_scale_matrix() const { return m_volume.scale_matrix; }
-        const Transform3d& get_volume_mirror_matrix() const { return m_volume.mirror_matrix; }
-        const Transform3d& get_volume_full_matrix() const { return m_volume.full_matrix; }
+        const Geometry::Transformation& get_volume_transform() const { return m_volume; }
+        const Geometry::Transformation& get_instance_transform() const { return m_instance; }
 
-        const Vec3d& get_instance_position() const { return m_instance.position; }
-        const Vec3d& get_instance_rotation() const { return m_instance.rotation; }
-        const Vec3d& get_instance_scaling_factor() const { return m_instance.scaling_factor; }
-        const Vec3d& get_instance_mirror() const { return m_instance.mirror; }
-        const Transform3d& get_instance_rotation_matrix() const { return m_instance.rotation_matrix; }
-        const Transform3d& get_instance_scale_matrix() const { return m_instance.scale_matrix; }
-        const Transform3d& get_instance_mirror_matrix() const { return m_instance.mirror_matrix; }
-        const Transform3d& get_instance_full_matrix() const { return m_instance.full_matrix; }
+    private:
+        Geometry::Transformation m_volume;
+        Geometry::Transformation m_instance;
     };
 
 public:
@@ -207,20 +130,42 @@ private:
     Cache m_cache;
     Clipboard m_clipboard;
     std::optional<BoundingBoxf3> m_bounding_box;
-    // Bounding box of a selection, with no instance scaling applied. This bounding box
-    // is useful for absolute scaling of tilted objects in world coordinate space.
+    // Bounding box of a single full instance selection, in world coordinates, with no instance scaling applied.
+    // This bounding box is useful for absolute scaling of tilted objects in world coordinate space.
+    // Modifiers are NOT taken in account
     std::optional<BoundingBoxf3> m_unscaled_instance_bounding_box;
+    // Bounding box of a single full instance selection, in world coordinates.
+    // Modifiers are NOT taken in account
     std::optional<BoundingBoxf3> m_scaled_instance_bounding_box;
+    // Bounding box of a single full instance selection, in world coordinates, with no instance scaling applied.
+    // Modifiers are taken in account
+    std::optional<BoundingBoxf3> m_full_unscaled_instance_bounding_box;
+    // Bounding box of a single full instance selection, in world coordinates.
+    // Modifiers are taken in account
+    std::optional<BoundingBoxf3> m_full_scaled_instance_bounding_box;
+    // Bounding box of a single full instance selection, in local coordinates, with no instance scaling applied.
+    // Modifiers are taken in account
+    std::optional<BoundingBoxf3> m_full_unscaled_instance_local_bounding_box;
+    // Bounding box aligned to the axis of the currently selected reference system (World/Object/Part)
+    // and transform to place and orient it in world coordinates
+    std::optional<std::pair<BoundingBoxf3, Transform3d>> m_bounding_box_in_current_reference_system;
 
 #if ENABLE_RENDER_SELECTION_CENTER
     GLModel m_vbo_sphere;
 #endif // ENABLE_RENDER_SELECTION_CENTER
 
+    CoordAxes m_axes;
     GLModel m_arrow;
     GLModel m_curved_arrow;
+    GLModel m_box;
+    struct Planes
+    {
+        std::array<Vec3f, 2> check_points{ Vec3f::Zero(), Vec3f::Zero() };
+        std::array<GLModel, 2> models;
+    };
+    Planes m_planes;
 
     float m_scale_factor;
-    bool m_dragging;
 
 public:
     Selection();
@@ -259,7 +204,7 @@ public:
     void set_deserialized(EMode mode, const std::vector<std::pair<size_t, size_t>> &volumes_and_instances);
 
     // Update the selection based on the new instance IDs.
-	void instances_changed(const std::vector<size_t> &instance_ids_selected);
+    void instances_changed(const std::vector<size_t> &instance_ids_selected);
     // Update the selection based on the map from old indices to new indices after m_volumes changed.
     // If the current selection is by instance, this call may select newly added volumes, if they belong to already selected instances.
     void volumes_changed(const std::vector<size_t> &map_volume_old_to_new);
@@ -277,21 +222,35 @@ public:
     bool is_single_volume() const { return m_type == SingleVolume; }
     bool is_multiple_volume() const { return m_type == MultipleVolume; }
     bool is_any_volume() const { return is_single_volume() || is_multiple_volume(); }
+    bool is_any_connector() const;
+    bool is_any_cut_volume() const;
     bool is_mixed() const { return m_type == Mixed; }
     bool is_from_single_instance() const { return get_instance_idx() != -1; }
     bool is_from_single_object() const;
     bool is_sla_compliant() const;
     bool is_instance_mode() const { return m_mode == Instance; }
+    bool is_single_volume_or_modifier() const { return is_single_volume() || is_single_modifier(); }
+    bool is_single_volume_instance() const { return is_single_full_instance() && m_list.size() == 1; }
+    bool is_single_text() const;
 
     bool contains_volume(unsigned int volume_idx) const { return m_list.find(volume_idx) != m_list.end(); }
     // returns true if the selection contains all the given indices
     bool contains_all_volumes(const std::vector<unsigned int>& volume_idxs) const;
     // returns true if the selection contains at least one of the given indices
     bool contains_any_volume(const std::vector<unsigned int>& volume_idxs) const;
+    // returns true if the selection contains any sinking volume
+    bool contains_sinking_volumes(bool ignore_modifiers = true) const;
     // returns true if the selection contains all and only the given indices
     bool matches(const std::vector<unsigned int>& volume_idxs) const;
 
-    bool requires_uniform_scale() const;
+    enum class EUniformScaleRequiredReason : unsigned char
+    {
+        NotRequired,
+        InstanceNotAxisAligned_World,
+        VolumeNotAxisAligned_World,
+        VolumeNotAxisAligned_Instance,
+        MultipleSelection,
+    };
 
     // Returns the the object id if the selection is from a single object, otherwise is -1
     int get_object_idx() const;
@@ -303,37 +262,64 @@ public:
 
     const IndicesList& get_volume_idxs() const { return m_list; }
     const GLVolume* get_volume(unsigned int volume_idx) const;
+    const GLVolume* get_first_volume() const { return get_volume(*m_list.begin()); }
+    GLVolume* get_volume(unsigned int volume_idx);
 
     const ObjectIdxsToInstanceIdxsMap& get_content() const { return m_cache.content; }
 
     unsigned int volumes_count() const { return (unsigned int)m_list.size(); }
     const BoundingBoxf3& get_bounding_box() const;
-    // Bounding box of a selection, with no instance scaling applied. This bounding box
-    // is useful for absolute scaling of tilted objects in world coordinate space.
+    // Bounding box of a single full instance selection, in world coordinates, with no instance scaling applied.
+    // This bounding box is useful for absolute scaling of tilted objects in world coordinate space.
+    // Modifiers are NOT taken in account
     const BoundingBoxf3& get_unscaled_instance_bounding_box() const;
+    // Bounding box of a single full instance selection, in world coordinates.
+    // Modifiers are NOT taken in account
     const BoundingBoxf3& get_scaled_instance_bounding_box() const;
+    // Bounding box of a single full instance selection, in world coordinates, with no instance scaling applied.
+    // Modifiers are taken in account
+    const BoundingBoxf3& get_full_unscaled_instance_bounding_box() const;
+    // Bounding box of a single full instance selection, in world coordinates.
+    // Modifiers are taken in account
+    const BoundingBoxf3& get_full_scaled_instance_bounding_box() const;
+    // Bounding box of a single full instance selection, in local coordinates, with no instance scaling applied.
+    // Modifiers are taken in account
+    const BoundingBoxf3& get_full_unscaled_instance_local_bounding_box() const;
+    // Returns the bounding box aligned to the axes of the currently selected reference system (World/Object/Part)
+    // and the transform to place and orient it in world coordinates
+    const std::pair<BoundingBoxf3, Transform3d>& get_bounding_box_in_current_reference_system() const;
+    // Returns the bounding box aligned to the axes of the given reference system
+    // and the transform to place and orient it in world coordinates
+    std::pair<BoundingBoxf3, Transform3d> get_bounding_box_in_reference_system(ECoordinatesType type) const;
 
-    void start_dragging();
-    void stop_dragging() { m_dragging = false; }
-    bool is_dragging() const { return m_dragging; }
+    // Returns the screen space bounding box
+    BoundingBoxf get_screen_space_bounding_box();
 
-    void translate(const Vec3d& displacement, bool local = false);
+    void setup_cache();
+
+    void translate(const Vec3d& displacement, TransformationType transformation_type);
     void rotate(const Vec3d& rotation, TransformationType transformation_type);
     void flattening_rotate(const Vec3d& normal);
     void scale(const Vec3d& scale, TransformationType transformation_type);
     void scale_to_fit_print_volume(const BuildVolume& volume);
-    void mirror(Axis axis);
-
-    void translate(unsigned int object_idx, const Vec3d& displacement);
+    void scale_and_translate(const Vec3d& scale, const Vec3d& world_translation, TransformationType transformation_type);
+    void mirror(Axis axis, TransformationType transformation_type);
+    void reset_skew();
     void translate(unsigned int object_idx, unsigned int instance_idx, const Vec3d& displacement);
 
+    // returns:
+    // -1 if the user refused to proceed with baking when asked
+    // 0 if the baking was performed
+    // 1 if no baking was needed
+    int bake_transform_if_needed() const;
+
     void erase();
 
-    void render(float scale_factor = 1.0) const;
+    void render(float scale_factor = 1.0);
+    void render_sidebar_hints(const std::string& sidebar_field);
 #if ENABLE_RENDER_SELECTION_CENTER
-    void render_center(bool gizmo_is_dragging) const;
+    void render_center(bool gizmo_is_dragging);
 #endif // ENABLE_RENDER_SELECTION_CENTER
-    void render_sidebar_hints(const std::string& sidebar_field) const;
 
     bool requires_local_axes() const;
 
@@ -352,6 +338,12 @@ public:
     std::vector<unsigned int> get_missing_volume_idxs_from(const std::vector<unsigned int>& volume_idxs) const;
     // returns the list of idxs of the volumes contained in the given list but not in the selection
     std::vector<unsigned int> get_unselected_volume_idxs_from(const std::vector<unsigned int>& volume_idxs) const;
+    // returns the list of idxs of the objects which are in the selection
+    std::set<unsigned int> get_object_idxs() const;
+
+#if ENABLE_MATRICES_DEBUG
+    void render_debug_window() const;
+#endif // ENABLE_MATRICES_DEBUG
 
 private:
     void update_valid();
@@ -362,21 +354,30 @@ private:
     void do_remove_volume(unsigned int volume_idx);
     void do_remove_instance(unsigned int object_idx, unsigned int instance_idx);
     void do_remove_object(unsigned int object_idx);
-    void set_bounding_boxes_dirty() { m_bounding_box.reset(); m_unscaled_instance_bounding_box.reset(); m_scaled_instance_bounding_box.reset(); }
+    void set_bounding_boxes_dirty() {
+        m_bounding_box.reset();
+        m_unscaled_instance_bounding_box.reset(); m_scaled_instance_bounding_box.reset();
+        m_full_unscaled_instance_bounding_box.reset(); m_full_scaled_instance_bounding_box.reset();
+        m_full_unscaled_instance_local_bounding_box.reset();
+        m_bounding_box_in_current_reference_system.reset();
+    }
+    void render_synchronized_volumes();
+    void render_bounding_box(const BoundingBoxf3& box, const Transform3d& trafo, const ColorRGB& color);
     void render_selected_volumes() const;
-    void render_synchronized_volumes() const;
     void render_bounding_box(const BoundingBoxf3& box, float* color) const;
-    void render_sidebar_position_hints(const std::string& sidebar_field) const;
-    void render_sidebar_rotation_hints(const std::string& sidebar_field) const;
-    void render_sidebar_scale_hints(const std::string& sidebar_field) const;
-    void render_sidebar_layers_hints(const std::string& sidebar_field) const;
+    void render_sidebar_position_hints(const std::string& sidebar_field, GLShaderProgram& shader, const Transform3d& matrix);
+    void render_sidebar_rotation_hints(const std::string& sidebar_field, GLShaderProgram& shader, const Transform3d& matrix);
+    void render_sidebar_scale_hints(const std::string& sidebar_field, GLShaderProgram& shader, const Transform3d& matrix);
+    void render_sidebar_layers_hints(const std::string& sidebar_field, GLShaderProgram& shader);
 
 public:
-    enum SyncRotationType {
+    enum class SyncRotationType {
         // Do not synchronize rotation. Either not rotating at all, or rotating by world Z axis.
-        SYNC_ROTATION_NONE = 0,
+        NONE = 0,
         // Synchronize after rotation by an axis not parallel with Z.
-        SYNC_ROTATION_GENERAL = 1,
+        GENERAL = 1,
+        // Synchronize after rotation reset.
+        RESET = 2
     };
     void synchronize_unselected_instances(SyncRotationType sync_rotation_type);
     void synchronize_unselected_volumes();
@@ -388,8 +389,16 @@ private:
 
     void paste_volumes_from_clipboard();
     void paste_objects_from_clipboard();
+
+    void transform_instance_relative(GLVolume& volume, const VolumeCache& volume_data, TransformationType transformation_type,
+        const Transform3d& transform, const Vec3d& world_pivot);
+    void transform_volume_relative(GLVolume& volume, const VolumeCache& volume_data, TransformationType transformation_type,
+        const Transform3d& transform, const Vec3d& world_pivot);
 };
 
+ModelVolume *get_selected_volume(const Selection &selection);
+const GLVolume *get_selected_gl_volume(const Selection &selection);
+
 } // namespace GUI
 } // namespace Slic3r
 
