@@ -1,3 +1,7 @@
+///|/ Copyright (c) Prusa Research 2018 - 2023 David Kocík @kocikdav, Lukáš Hejl @hejllukas, Oleksandra Iushchenko @YuSanka, Vojtěch Král @vojtechkral, Vojtěch Bubník @bubnikv
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #ifndef slic3r_UpdateDialogs_hpp_
 #define slic3r_UpdateDialogs_hpp_
 
@@ -6,6 +10,8 @@
 #include <vector>
 #include <wx/hyperlink.h>
 
+#include <boost/filesystem.hpp>
+
 #include "libslic3r/Semver.hpp"
 #include "MsgDialog.hpp"
 
@@ -37,6 +43,43 @@ private:
 };
 
 
+class AppUpdateAvailableDialog : public MsgDialog
+{
+public:
+	AppUpdateAvailableDialog(const Semver& ver_current, const Semver& ver_online, bool from_user);
+	AppUpdateAvailableDialog(AppUpdateAvailableDialog&&) = delete;
+	AppUpdateAvailableDialog(const AppUpdateAvailableDialog&) = delete;
+	AppUpdateAvailableDialog& operator=(AppUpdateAvailableDialog&&) = delete;
+	AppUpdateAvailableDialog& operator=(const AppUpdateAvailableDialog&) = delete;
+	virtual ~AppUpdateAvailableDialog();
+
+	// Tells whether the user checked the "don't bother me again" checkbox
+	bool disable_version_check() const;
+	static wxSize AUAD_size;
+private:
+	wxCheckBox* cbox {nullptr};
+};
+
+class AppUpdateDownloadDialog : public MsgDialog
+{
+public:
+	AppUpdateDownloadDialog(const Semver& ver_online, boost::filesystem::path& path);
+	AppUpdateDownloadDialog(AppUpdateDownloadDialog&&) = delete;
+	AppUpdateDownloadDialog(const AppUpdateDownloadDialog&) = delete;
+	AppUpdateDownloadDialog& operator=(AppUpdateDownloadDialog&&) = delete;
+	AppUpdateDownloadDialog& operator=(const AppUpdateDownloadDialog&) = delete;
+	virtual ~AppUpdateDownloadDialog();
+
+	// Tells whether the user checked the "don't bother me again" checkbox
+	bool		run_after_download() const;
+	boost::filesystem::path	get_download_path() const;
+
+private:
+	wxCheckBox* cbox_run;
+	wxTextCtrl* txtctrl_path;
+	wxString filename;
+};
+
 // Confirmation dialog informing about configuration update. Lists updated bundles & their versions.
 class MsgUpdateConfig : public MsgDialog
 {
@@ -129,6 +172,18 @@ public:
 	~MsgNoUpdates();
 };
 
+// Informs about absence of new version online.
+class MsgNoAppUpdates : public MsgDialog
+{
+public:
+	MsgNoAppUpdates();
+	MsgNoAppUpdates(MsgNoAppUpdates&&) = delete;
+	MsgNoAppUpdates(const MsgNoAppUpdates&) = delete;
+	MsgNoAppUpdates& operator=(MsgNoUpdates&&) = delete;
+	MsgNoAppUpdates& operator=(const MsgNoAppUpdates&) = delete;
+	~MsgNoAppUpdates();
+};
+
 }
 }
 
