@@ -15,7 +15,7 @@
 //#include "libnest2d/tools/benchmark.h"
 #include "libslic3r/SVG.hpp"
 
-#include "../libnest2d/printer_parts.hpp"
+#include "../data/prusaparts.hpp"
 
 #include <unordered_set>
 
@@ -83,7 +83,7 @@ TEST_CASE("Line::perpendicular_to", "[Geometry]") {
 
 TEST_CASE("Polygon::contains works properly", "[Geometry]"){
    // this test was failing on Windows (GH #1950)
-    Slic3r::Polygon polygon(std::vector<Point>({
+    Slic3r::Polygon polygon(Points({
         Point(207802834,-57084522),
         Point(196528149,-37556190),
         Point(173626821,-25420928),
@@ -145,7 +145,7 @@ SCENARIO("polygon_is_convex works") {
 
 TEST_CASE("Creating a polyline generates the obvious lines", "[Geometry]"){
     Slic3r::Polyline polyline;
-    polyline.points = std::vector<Point>({Point(0, 0), Point(10, 0), Point(20, 0)});
+    polyline.points = Points({Point(0, 0), Point(10, 0), Point(20, 0)});
     REQUIRE(polyline.lines().at(0).a == Point(0,0));
     REQUIRE(polyline.lines().at(0).b == Point(10,0));
     REQUIRE(polyline.lines().at(1).a == Point(10,0));
@@ -153,7 +153,7 @@ TEST_CASE("Creating a polyline generates the obvious lines", "[Geometry]"){
 }
 
 TEST_CASE("Splitting a Polygon generates a polyline correctly", "[Geometry]"){
-    Slic3r::Polygon polygon(std::vector<Point>({Point(0, 0), Point(10, 0), Point(5, 5)}));
+    Slic3r::Polygon polygon(Points({Point(0, 0), Point(10, 0), Point(5, 5)}));
     Slic3r::Polyline split = polygon.split_at_index(1);
     REQUIRE(split.points[0]==Point(10,0));
     REQUIRE(split.points[1]==Point(5,5));
@@ -162,18 +162,38 @@ TEST_CASE("Splitting a Polygon generates a polyline correctly", "[Geometry]"){
 }
 
 
-TEST_CASE("Bounding boxes are scaled appropriately", "[Geometry]"){
-    BoundingBox bb(std::vector<Point>({Point(0, 1), Point(10, 2), Point(20, 2)}));
-    bb.scale(2);
-    REQUIRE(bb.min == Point(0,2));
-    REQUIRE(bb.max == Point(40,4));
+SCENARIO("BoundingBox", "[Geometry]") {
+    WHEN("Bounding boxes are scaled") {
+        BoundingBox bb(Points({Point(0, 1), Point(10, 2), Point(20, 2)}));
+        bb.scale(2);
+        REQUIRE(bb.min == Point(0,2));
+        REQUIRE(bb.max == Point(40,4));
+    }
+    WHEN("BoundingBox constructed from points") {
+        BoundingBox bb(Points{ {100,200}, {100, 200}, {500, -600} });
+        THEN("minimum is correct") {
+            REQUIRE(bb.min == Point{100,-600});
+        }
+        THEN("maximum is correct") {
+            REQUIRE(bb.max == Point{500,200});
+        }
+    }
+    WHEN("BoundingBox constructed from a single point") {
+        BoundingBox bb;
+        bb.merge({10, 10});
+        THEN("minimum equals to the only defined point") {
+            REQUIRE(bb.min == Point{10,10});
+        }
+        THEN("maximum equals to the only defined point") {
+            REQUIRE(bb.max == Point{10,10});
+        }
+    }
 }
 
-
 TEST_CASE("Offseting a line generates a polygon correctly", "[Geometry]"){
 	Slic3r::Polyline tmp = { Point(10,10), Point(20,10) };
     Slic3r::Polygon area = offset(tmp,5).at(0);
-    REQUIRE(area.area() == Slic3r::Polygon(std::vector<Point>({Point(10,5),Point(20,5),Point(20,15),Point(10,15)})).area());
+    REQUIRE(area.area() == Slic3r::Polygon(Points({Point(10,5),Point(20,5),Point(20,15),Point(10,15)})).area());
 }
 
 SCENARIO("Circle Fit, TaubinFit with Newton's method", "[Geometry]") {
@@ -288,7 +308,7 @@ TEST_CASE("smallest_enclosing_circle_welzl", "[Geometry]") {
 
 SCENARIO("Path chaining", "[Geometry]") {
 	GIVEN("A path") {
-		std::vector<Point> points = { Point(26,26),Point(52,26),Point(0,26),Point(26,52),Point(26,0),Point(0,52),Point(52,52),Point(52,0) };
+		Points points = { Point(26,26),Point(52,26),Point(0,26),Point(26,52),Point(26,0),Point(0,52),Point(52,52),Point(52,0) };
 		THEN("Chained with no diagonals (thus 26 units long)") {
 			std::vector<Points::size_type> indices = chain_points(points);
 			for (Points::size_type i = 0; i + 1 < indices.size(); ++ i) {
@@ -373,76 +393,112 @@ SCENARIO("Line distances", "[Geometry]"){
     }
 }
 
+SCENARIO("Calculating angles", "[Geometry]")
+{
+    GIVEN(("Vectors 30 degrees apart"))
+    {
+        std::vector<std::pair<Point, Point>> pts {
+            { {1000, 0}, { 866, 500 } },
+            { { 866, 500 }, { 500, 866 } },
+            { { 500, 866 }, { 0, 1000 } },
+            { { -500, 866 }, { -866, 500 } }
+        };
+
+        THEN("Angle detected is 30 degrees")
+        {
+            for (auto &p : pts)
+                REQUIRE(is_approx(angle(p.first, p.second), M_PI / 6.));
+        }
+    }
+
+    GIVEN(("Vectors 30 degrees apart"))
+    {
+        std::vector<std::pair<Point, Point>> pts {
+            { { 866, 500 }, {1000, 0} },
+            { { 500, 866 }, { 866, 500 } },
+            { { 0, 1000 }, { 500, 866 } },
+            { { -866, 500 }, { -500, 866 } }
+        };
+
+        THEN("Angle detected is -30 degrees")
+        {
+            for (auto &p : pts)
+                REQUIRE(is_approx(angle(p.first, p.second), - M_PI / 6.));
+        }
+    }
+}
+
 SCENARIO("Polygon convex/concave detection", "[Geometry]"){
+    static constexpr const double angle_threshold = M_PI / 3.;
     GIVEN(("A Square with dimension 100")){
-        auto square = Slic3r::Polygon /*new_scale*/(std::vector<Point>({
+        auto square = Slic3r::Polygon /*new_scale*/(Points({
             Point(100,100),
             Point(200,100),
             Point(200,200),
             Point(100,200)}));
         THEN("It has 4 convex points counterclockwise"){
-            REQUIRE(square.concave_points(PI*4/3).size() == 0);
-            REQUIRE(square.convex_points(PI*2/3).size() == 4);
+            REQUIRE(square.concave_points(angle_threshold).size() == 0);
+            REQUIRE(square.convex_points(angle_threshold).size() == 4);
         }
         THEN("It has 4 concave points clockwise"){
             square.make_clockwise();
-            REQUIRE(square.concave_points(PI*4/3).size() == 4);
-            REQUIRE(square.convex_points(PI*2/3).size() == 0);
+            REQUIRE(square.concave_points(angle_threshold).size() == 4);
+            REQUIRE(square.convex_points(angle_threshold).size() == 0);
         }
     }
     GIVEN("A Square with an extra colinearvertex"){
-        auto square = Slic3r::Polygon /*new_scale*/(std::vector<Point>({
+        auto square = Slic3r::Polygon /*new_scale*/(Points({
             Point(150,100),
             Point(200,100),
             Point(200,200),
             Point(100,200),
             Point(100,100)}));
         THEN("It has 4 convex points counterclockwise"){
-            REQUIRE(square.concave_points(PI*4/3).size() == 0);
-            REQUIRE(square.convex_points(PI*2/3).size() == 4);
+            REQUIRE(square.concave_points(angle_threshold).size() == 0);
+            REQUIRE(square.convex_points(angle_threshold).size() == 4);
         }
     }
     GIVEN("A Square with an extra collinear vertex in different order"){
-        auto square = Slic3r::Polygon /*new_scale*/(std::vector<Point>({
+        auto square = Slic3r::Polygon /*new_scale*/(Points({
             Point(200,200),
             Point(100,200),
             Point(100,100),
             Point(150,100),
             Point(200,100)}));
         THEN("It has 4 convex points counterclockwise"){
-            REQUIRE(square.concave_points(PI*4/3).size() == 0);
-            REQUIRE(square.convex_points(PI*2/3).size() == 4);
+            REQUIRE(square.concave_points(angle_threshold).size() == 0);
+            REQUIRE(square.convex_points(angle_threshold).size() == 4);
         }
     }
 
     GIVEN("A triangle"){
-        auto triangle = Slic3r::Polygon(std::vector<Point>({
+        auto triangle = Slic3r::Polygon(Points({
             Point(16000170,26257364),
             Point(714223,461012),
             Point(31286371,461008)
         }));
         THEN("it has three convex vertices"){
-            REQUIRE(triangle.concave_points(PI*4/3).size() == 0);
-            REQUIRE(triangle.convex_points(PI*2/3).size() == 3);
+            REQUIRE(triangle.concave_points(angle_threshold).size() == 0);
+            REQUIRE(triangle.convex_points(angle_threshold).size() == 3);
         }
     }
 
     GIVEN("A triangle with an extra collinear point"){
-        auto triangle = Slic3r::Polygon(std::vector<Point>({
+        auto triangle = Slic3r::Polygon(Points({
             Point(16000170,26257364),
             Point(714223,461012),
             Point(20000000,461012),
             Point(31286371,461012)
         }));
         THEN("it has three convex vertices"){
-            REQUIRE(triangle.concave_points(PI*4/3).size() == 0);
-            REQUIRE(triangle.convex_points(PI*2/3).size() == 3);
+            REQUIRE(triangle.concave_points(angle_threshold).size() == 0);
+            REQUIRE(triangle.convex_points(angle_threshold).size() == 3);
         }
     }
     GIVEN("A polygon with concave vertices with angles of specifically 4/3pi"){
         // Two concave vertices of this polygon have angle = PI*4/3, so this test fails
         // if epsilon is not used.
-        auto polygon = Slic3r::Polygon(std::vector<Point>({
+        auto polygon = Slic3r::Polygon(Points({
             Point(60246458,14802768),Point(64477191,12360001),
             Point(63727343,11060995),Point(64086449,10853608),
             Point(66393722,14850069),Point(66034704,15057334),
@@ -453,14 +509,14 @@ SCENARIO("Polygon convex/concave detection", "[Geometry]"){
             Point(38092663,692699),Point(52100125,692699)
         }));
         THEN("the correct number of points are detected"){
-            REQUIRE(polygon.concave_points(PI*4/3).size() == 6);
-            REQUIRE(polygon.convex_points(PI*2/3).size() == 10);
+            REQUIRE(polygon.concave_points(angle_threshold).size() == 6);
+            REQUIRE(polygon.convex_points(angle_threshold).size() == 10);
         }
     }
 }
 
 TEST_CASE("Triangle Simplification does not result in less than 3 points", "[Geometry]"){
-    auto triangle = Slic3r::Polygon(std::vector<Point>({
+    auto triangle = Slic3r::Polygon(Points({
         Point(16000170,26257364), Point(714223,461012), Point(31286371,461008)
     }));
     REQUIRE(triangle.simplify(250000).at(0).points.size() == 3);
@@ -627,15 +683,15 @@ struct Pair
 template<> struct std::hash<Pair> {
     size_t operator()(const Pair &c) const
     {
-        return c.first * PRINTER_PART_POLYGONS.size() + c.second;
+        return c.first * PRUSA_PART_POLYGONS.size() + c.second;
     }
 };
 
 TEST_CASE("Convex polygon intersection test prusa polygons", "[Geometry][Rotcalip]") {
 
     // Overlap of the same polygon should always be an intersection
-    for (size_t i = 0; i < PRINTER_PART_POLYGONS.size(); ++i) {
-        Polygon P = PRINTER_PART_POLYGONS[i];
+    for (size_t i = 0; i < PRUSA_PART_POLYGONS.size(); ++i) {
+        Polygon P = PRUSA_PART_POLYGONS[i];
         P = Geometry::convex_hull(P.points);
         bool res = Geometry::convex_polygons_intersect(P, P);
         if (!res) {
@@ -647,8 +703,8 @@ TEST_CASE("Convex polygon intersection test prusa polygons", "[Geometry][Rotcali
     }
 
     std::unordered_set<Pair> combos;
-    for (size_t i = 0; i < PRINTER_PART_POLYGONS.size(); ++i) {
-        for (size_t j = 0; j < PRINTER_PART_POLYGONS.size(); ++j) {
+    for (size_t i = 0; i < PRUSA_PART_POLYGONS.size(); ++i) {
+        for (size_t j = 0; j < PRUSA_PART_POLYGONS.size(); ++j) {
             if (i != j) {
                 size_t a = std::min(i, j), b = std::max(i, j);
                 combos.insert(Pair{a, b});
@@ -658,7 +714,7 @@ TEST_CASE("Convex polygon intersection test prusa polygons", "[Geometry][Rotcali
 
     // All disjoint
     for (const auto &combo : combos) {
-        Polygon A = PRINTER_PART_POLYGONS[combo.first], B = PRINTER_PART_POLYGONS[combo.second];
+        Polygon A = PRUSA_PART_POLYGONS[combo.first], B = PRUSA_PART_POLYGONS[combo.second];
         A = Geometry::convex_hull(A.points);
         B = Geometry::convex_hull(B.points);
 
@@ -685,7 +741,7 @@ TEST_CASE("Convex polygon intersection test prusa polygons", "[Geometry][Rotcali
 
     // All intersecting
     for (const auto &combo : combos) {
-        Polygon A = PRINTER_PART_POLYGONS[combo.first], B = PRINTER_PART_POLYGONS[combo.second];
+        Polygon A = PRUSA_PART_POLYGONS[combo.first], B = PRUSA_PART_POLYGONS[combo.second];
         A = Geometry::convex_hull(A.points);
         B = Geometry::convex_hull(B.points);
 
@@ -708,3 +764,27 @@ TEST_CASE("Convex polygon intersection test prusa polygons", "[Geometry][Rotcali
         REQUIRE(res == ref);
     }
 }
+
+
+TEST_CASE("Euler angles roundtrip", "[Geometry]") {
+    std::vector<Vec3d> euler_angles_vec = {{M_PI/2.,  -M_PI,    0.},
+                                           {M_PI,     -M_PI,    0.},
+                                           {M_PI,     -M_PI,    2*M_PI},
+                                           {0.,       0.,       M_PI},
+                                           {M_PI,     M_PI/2.,  0.},
+                                           {0.2,      0.3,      -0.5}};
+
+    // Also include all combinations of zero and +-pi/2:
+    for (double x : {0., M_PI/2., -M_PI/2.})
+       for (double y : {0., M_PI/2., -M_PI/2.})
+          for (double z : {0., M_PI/2., -M_PI/2.})
+              euler_angles_vec.emplace_back(x, y, z);
+
+    for (Vec3d& euler_angles : euler_angles_vec) {
+        Transform3d trafo1 = Geometry::rotation_transform(euler_angles);
+        euler_angles = Geometry::extract_rotation(trafo1);
+        Transform3d trafo2 = Geometry::rotation_transform(euler_angles);
+
+        REQUIRE(trafo1.isApprox(trafo2));
+    }
+}
